***********************************************************************
***********************************************************************
*       This program caclulates the Higgs boson masses in the 
*             complex NMSSM up to two-loop contributions of the
*       order alpha_s alpha_t +(alpha_t+ alpha_lambda+alpha_kappa)^2
**********************************************************************
***********************************************************************
*For details on the renormalization etc, see also
*  JHEP 08 (2019) 114, JHEP 1505 (2015) 128, Comput.Phys.Commun. 185 (2014) 12,
* 3372 -3391, JHEP 1210 (2012) 122, Phys.Rev. D85 (2012) 075024
*
*A mixed renormalization scheme is applied:
*  Depending on which input parameters are given there are two 
*  renormalization schemes:
*   1) Mass of the charged Higgs boson given in the Block EXTPAR:
*      The vector boson masses MW and MZ, as well as the
*      electric charge and the mass of the charged Higgs boson are 
*      renormalized on-shell. The imaginary parts of Alambda and
*      Akappa are renormalized via on-shell conditions for the 
*      CP-odd tadpoles.
*   2) Realpart of Alambda is given in Block EXTPAR:
*      The vector boson masses MW and MZ and the
*      electric charge are renormalized on-shell. 
*      The realpart of Alambda is interpreted as a DR value and
*      the mass of the charged Higgs boson is calculated in a 
*      mixed scheme. The imaginary parts of Alambda and
*      Akappa are renormalized via on-shell conditions for the 
*      CP-odd tadpoles.
*
* If no input file is specified in the command line (first argument)
*  the standard file 'inp.dat' is used. This file has to follow slha
*  conventions (extended for complex NMSSM)
*
* The following parameters are calculated from the parameters read in
* from the SLHA file and used throughout the program:
*
*   SM Input Parameters:
*      Gf           Fermi coupling constant
*      Mz           pole mass of the Z boson
*      1/alpha_em   inverse of the electric coupling constant
*      Mw           pole mass of the W boson
*   Masses of the SM Fermions (m_e,m_mu,m_tau,m_u,m_d,m_s,m_c,m_t,m_b)
*   Switches:
*     MHpmInput=TRUE/FALSE    MHpm/Alambda is consideres as input
*                             set automatically depending on which
*                             parameter is given in slha input file
*     CPcon=TRUE/FALSE        CP conservation is turned on/off
*                             switch 5 in Block MODSEL
*                             0: CP-conserving
*                             2: general CP-violation
*     onelswitch=TRUE/FALSE   sets precision of Higgs mass calculation
*                             switch 6 in Block MODSEL
*                             1: one-loop
*                             2: two-loop order alpha_s alpha_t (default)
*     mQrun=TRUE/FALSE        selects renormalization scheme of stop/top-sector
*                             switch 7 in Block MODSEL
*                             1: DR scheme (default)
*                             2: OS scheme
*   Higgs Sector Parameters:
*     phiu, phis    phases of the vev's
*     tanb          (ratio vu/vd)
*     abslambda     absolute value of lambda
*     phil	    phase of lambda
*     abskappa      absolute value of kappa
*     phik          phase of kappa
*     ReAl          realpart of Alambda
*                   (value is ignored if MHpm is given)
*     MHpm          on-shell mass of charged Higgs boson
*     ReAk          Realpart of Akappa
*     mueff         absolute value of mueff=|<S>*lambda|
*   Soft SUSY Breaking Parameters
*     absM1,phiM1      absolute value and phase of M1
*     absM2,phiM2      absolute value and phase of M2
*     absM3,phiM2      absolute value and phase of M3
*     ml_i             i=1,2,3 left-handed-sleptons
*     mer_i            i=1,2,3 right-handed-sleptons
*     mq_i             i=1,2,3 left-handed-squarks
*     mur_i            i=1,2,3 right-handed-uptype-squarks
*     mdr_i            i=1,2,3 right-handed-downtype-squarks
*     absAe_i,phiAe_i  i=1,2,3 trilinear breaking parameter sleptons
*     absAu_i,phiAu_i  i=1,2,3 trilinear breaking parameter uptype squarks
*     absAd_i,phiAd_i  i=1,2,3 trilinear breaking parameter downtype squarks
*   Renormalization Scale
*     Qs             all above input parameters are assumed to be given
*                    at this scale
*
*
* Subroutines:
*
*  Subroutine treelevelmasses:
*     calculates all tree-level masses of the NMSSM
*
*  Subroutine Hmass:
*     calculates the loop Higgs masses
*
* One-loop part:
*
*  Subroutine couplings:
*     calculates all couplings of the Higgs sector needed for one-loop part
*
*  Subroutine Counterterms:
*     calculates all finite parts of the one-loop on-shell counterterms
*     dMW2, dMZ2, dZe, dMHpm2, dthd, dthu, dths, dtad, dtas
*
*  Subroutine counterM:
*     calcualtes the counterterm mass matrix of the Higgs sector at one-loop level
*
*  Subroutine MassMatrix1l:
*     caclualtes the one-loop mass matrix of the Higgs sector
*
*  Subroutine runpara:
*     calculated the running parameters needed for the subsequent RGEs
*     running within hdecay
*
*  Subroutines to diagonalize complex matrices 
*  (Routines by Thomas Hahn arXiv:physics/0607103)
*
*
* Functions:
*
*  function HSelf
*   caclulates the one-loop Higgs self energy for external momentum p2
*
*  function DHSelf
*   derivative of the one-loop Higgs self energy with respect to p2
*
*  funciton HSelfren
*   renormalized one-loop self energ
*
*  function iter
*   iterative procedure to obtain the loop masses
*
*  fucntion a_sMS
*   cacluates the running a_sMS in the SM at the scale Q
*   (needed for the calculation of the running top and bottom mass)
*
*  function mtMS(Q), mbMS(Q), Un(n,Q2,Q1), MStoDR(mMS,mDR,Q), mtbeff
*    running of the top and bottom masses
*
*  loop functions and their derivatives
*   for the one-loop part: A0,B0,B1,B00,DB0,DB1,BD00
*
***********************************************************************

c MMM changed 10/8/2013
c      program CalcMasses
      subroutine CalcMasses(input,output)
c end MMM changed 10/8/2013      

      implicit none


      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
     
      double precision RAk,RAl,arg
      
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
     
      double precision absM3, phiM3
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
     
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double precision Qs,Qinp
       
      double precision Mh1l(5),ZH1l(5,5)
      
      double precision MHpm1l
      
      double precision ReAlambdaDR,ReAkappaDR,ImAlambdaDR,ImAkappaDR
      
      double precision pi
      integer nin,nout,ninlha
      parameter (pi = 2d0*Asin(1d0))
      parameter (nin=43,nout=44,ninlha=45)
 
c MMM changed 10/8/2013    
      logical MHpmInput,CPcon,mQrun,phasetest,onelswitch 
      integer EDMswitch
c end MMM changed 10/8/2013

      double precision alphainv,ml(3), mer(3), mq(3), mdr(3), mur(3),
     &                 alphainv_run,mtpole,mbMSmb
      
      character*120 output, input
          
c     variables from SLHA_read_leshouches  
      double precision minval(1:20),smval(1:24),massval(1:50),
     .                 nmixval(5,5),umixval(2,2),vmixval(2,2),
     .                 stopmixval(2,2),sbotmixval(2,2),staumixval(2,2),
     .                 hmixval(1:10),gaugeval(1:3),msoftval(1:100),
     .                 auval(3,3),adval(3,3),aeval(3,3),yuval(3,3),
     .                 ydval(3,3),yeval(3,3),qvalue(1:20),extval(0:100),
     .                 cmplxv(1:10)
      double precision imnmixval(5,5),imumixval(2,2),imvmixval(2,2),
     .                 imstopmixval(2,2),imsbotmixval(2,2),
     .                 imstaumixval(2,2),imextval(0:100),
     .                 immsoftval(1:100)
c MMM changed 10/8/2013
      double precision nmhmixval(3,3),nmamixval(2,3)
      double precision nhmixcval(5,5)
      integer modselval(10)
c end MMM changed 10/8/2013
      integer check(1:33),imod(1:2)
      character spinfo1*100,spinfo2*100,mincom(1:20)*20,
     .          extcom(0:100)*20,softcom(1:100)*20,hmixcom(1:10)*20,
     .          imextcom(0:100)*20,imsoftcom(1:100)*20,
     .          cmplxvcom(1:10)*20
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /gluino/ absM3,phiM3
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
     
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /topoles/ mtpole,mbMsmb
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /H1loop/ MHpm1l,MH1l,ZH1l
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
      common /slhapara/ RAl,RAk
      common /DRpara/ ReAlambdaDR,ReAkappaDR,ImAlambdaDR,ImAkappaDR
      common /scale/ Qs
      common /EDMselec/ EDMswitch

      COMMON/SLHA_leshouches0_NMSSM/spinfo1,spinfo2,modselval
      COMMON/SLHA_leshouches1_NMSSM/extcom,mincom,softcom,hmixcom
      COMMON/SLHA_leshouches1_CNMSSM/imextcom,imsoftcom,cmplxvcom
      COMMON/SLHA_leshouches2_NMSSM/minval,extval,smval,massval,
     .                   nmixval,umixval,vmixval,stopmixval,sbotmixval,
     .                   staumixval,hmixval,gaugeval,msoftval,auval,
     .                   adval,aeval,yuval,ydval,yeval,qvalue,imod
      COMMON/SLHA_leshouches2_CNMSSM/cmplxv
      COMMON/SLHA_leshouches3_NMSSM/nmhmixval,nmamixval
      COMMON/SLHA_leshouches3_CNMSSM/nhmixcval
      COMMON/SLHA_leshouches4_CNMSSM/imextval,imnmixval,imumixval,
     .     imvmixval,imstopmixval,imsbotmixval,imstaumixval,immsoftval
      COMMON/SLHA_checkval_CNMSSM/check
c nhung add for atat 22/3/2019
      integer ilevel,igauge 
      common/ilevelATAT/ilevel,igauge
c end nhung add 22/3/2019
c nhung added for atlambda 6/2021
      double precision regfac
      common/ATlambda/regfac
c end nhung added 6/2021
      
      print*
      print*, '*******************************************************'
     .        ,'**************'
      print*, ' 1/ Calculation of the loop Higgs masses in'
     .       ,' the (complex) NMSSM.  '
      print*, '*******************************************************'
     .        ,'**************'
      print*

c reading in SLHA-file
      open(ninlha,file=input)

      call SLHA_read_leshouches_NMSSM(ninlha)
     
c check that NMSSM is selected
      if(modselval(3).ne.1) then
      print*,'* Warning: This programs can only calculate the Higgs boso
     .n masses in the NMSSM!'
      endif


c Switch to select CP-violation or conservation
      if(modselval(5).eq.0) then
      CPcon=.TRUE.
      elseif(modselval(5).eq.2) then
      CPcon=.FALSE.
      else
      print*, '* Warning: Please specify whether you assume CP-conservat
     .ion!'
      print*, '  Switch 5 in Block MODSEL:'
      print*, '  0: CP-conservation assumed'
      print*, '  2: general CP-violation'
      call exit
      end if
      
      if(modselval(6).eq.1) then
      onelswitch=.TRUE.
      elseif(modselval(6).eq.2) then
      onelswitch=.FALSE.
c nhung add to turn off atat correction 22/3/2019
      ilevel=2
c end nhung add 22/3/2019
      elseif(modselval(6).eq.4) then
      onelswitch=.FALSE.
      ilevel=4
      else
      print*, '* Warning: Please specify the desired loop level for the
     . Higgs mass calculation'
      print*, '  Switch 6 in Block MODSEL:'
      print*, '  1: one-loop masses'
      print*, '  2: two-loop masses'
      print*, '  Default: Calculating two-loop masses of order 
     . O(alpha_t alpha_s+alpha_t^2)'
c nhung add to turn on atat correction 22/3/2019
      ilevel =3 
c end nhung add 22/3/2019
      write(*,*) 'modselval(6)',modselval(6)
      end if

      
      if(modselval(10).eq.1) then
      EDMswitch=1
      elseif(modselval(10).eq.2) then
      EDMswitch=2
      else
      EDMswitch=0
      end if
      
c SM parameters
      Gf=smval(2)
      Mz=smval(4)
      Mw=smval(9)
      alphainv=smval(1)
      alpha=1/alphainv
      alphainv_run=alphainv
      alphaS=smval(3)
     
      me=smval(11)
      mmu=smval(13)
      mtau=smval(7)
      mu=smval(22)
      md=smval(21)
      ms=smval(23)
      mc=smval(24)
      mtpole=smval(6)
      mbMSmb=smval(5)
      
      tanb=extval(25)
     
c check if all necessary input is supplied correctly
      call inputtest(CPcon,alphainv,'alhpa^(-1)',.FALSE.)
      call inputtest(CPcon,Gf,'G_F       ',.FALSE.)
      call inputtest(CPcon,alphas,'alpha_s   ',.FALSE.)
      call inputtest(CPcon,Mz,'Mz        ',.FALSE.)
      call inputtest(CPcon,Mw,'Mw        ',.FALSE.)
      call inputtest(CPcon,mbMSmb,'m,_b       ',.FALSE.)
      call inputtest(CPcon,mtpole,'m_t       ',.FALSE.)
      call inputtest(CPcon,mtau,'m_tau     ',.FALSE.)
      call inputtest(CPcon,me,'m_e       ',.FALSE.)
      call inputtest(CPcon,mmu,'m_mu      ',.FALSE.)
      call inputtest(CPcon,md,'m_d       ',.FALSE.)
      call inputtest(CPcon,mu,'m_u       ',.FALSE.)
      call inputtest(CPcon,ms,'m_s       ',.FALSE.)
      call inputtest(CPcon,mc,'m_c       ',.FALSE.)
      call inputtest(CPcon,tanb,'tanbeta   ',.FALSE.)
      call inputtest(CPcon,extval(1),'ReM1      ',.FALSE.)
      call inputtest(CPcon,imextval(1),'ImM1      ',.TRUE.)
      call inputtest(CPcon,extval(2),'ReM2      ',.FALSE.)
      call inputtest(CPcon,imextval(2),'ImM2      ',.TRUE.)
      call inputtest(CPcon,extval(3),'ReM3      ',.FALSE.)
      call inputtest(CPcon,imextval(3),'ImM3      ',.TRUE.)
      call inputtest(CPcon,extval(11),'ReAt      ',.FALSE.)
      call inputtest(CPcon,imextval(11),'ImAt      ',.TRUE.)
      call inputtest(CPcon,extval(12),'ReAb      ',.FALSE.)
      call inputtest(CPcon,imextval(12),'ImAb      ',.TRUE.)
      call inputtest(CPcon,extval(13),'ReAtau    ',.FALSE.)
      call inputtest(CPcon,imextval(13),'ImAtau    ',.TRUE.)
      call inputtest(CPcon,extval(61),'Relambda   ',.FALSE.)
      call inputtest(CPcon,extval(62),'Rekappa   ',.FALSE.)
      call inputtest(CPcon,imextval(61),'Imlambda   ',.TRUE.)
      call inputtest(CPcon,imextval(62),'Imkappa   ',.TRUE.)
      call inputtest(CPcon,extval(64),'ReAk      ',.FALSE.)
      call inputtest(CPcon,extval(65),'Remueff   ',.FALSE.)
      call inputtest(CPcon,imextval(65),'Immueff   ',.TRUE.)
      call inputtest(CPcon,extval(33),'ml_3      ',.FALSE.)
      call inputtest(CPcon,extval(36),'mer_3      ',.FALSE.)
      call inputtest(CPcon,extval(43),'mq_3      ',.FALSE.)
      call inputtest(CPcon,extval(46),'mur_3      ',.FALSE.)
      call inputtest(CPcon,extval(49),'mdr_3      ',.FALSE.)
      call inputtest(CPcon,cmplxv(3),'phiu      ',.TRUE.)
      
c mapping of the read in parameters
      abslambda=sqrt(extval(61)**2+imextval(61)**2)
      phil=arg(extval(61),imextval(61))
      abskappa=sqrt(extval(62)**2+imextval(62)**2)
      phik=arg(extval(62),imextval(62))
      RAl=extval(63)
      RAk=extval(64)
       if(imextval(63).ne.-123456789.or.imextval(64).ne.-123456789) then
       print*
       print*,'* Warning: Imaginary parts of Alambda and Akappa ',
     .        'given in input are ignored.'
       print*,'Will be recalculated to ensure consistency.'
      end if
      Mhpm=extval(27)
      If(Mhpm.eq.-123456789) then
      MhpmInput=.FALSE.
      elseif(RAl.eq.-123456789) then
      MhpmInput=.TRUE.
      else
      print*
      print*,'* Warning: please provide either MHpm or ReAlambda, not bo
     .th!'
      print*,'  MHpm is default input; input on ReAlambda is ignored.'

      MhpmInput=.TRUE.
      end if
      if(Mhpm.eq.-123456789.and.RAl.eq.-123456789) then

      print*
      print*,'* Warning: parameter value missing in slha file: ',
     .        'Mhpm or ReAl'
      call exit
      end if
      
      if(CPcon) then
      mueff=extval(65)
      phis=0d0
      else
      mueff=Abs(extval(65)+(0d0,1d0)*imextval(65))
      phis=arg(extval(65),imextval(65))-phil
      end if
      phiu=cmplxv(3)
      
      absM1=Abs(extval(1)+(0d0,1d0)*imextval(1))
      phiM1=arg(extval(1),imextval(1))
      absM2=Abs(extval(2)+(0d0,1d0)*imextval(2))
      phiM2=arg(extval(2),imextval(2))
      absM3=Abs(extval(3)+(0d0,1d0)*imextval(3))
      phiM3=arg(extval(3),imextval(3))
      
      
c if the input values for the first to generations aren't given explicitly
c they are set equal to those of the third generation
      ml(1)=extval(31)
      ml(2)=extval(32)
      ml(3)=extval(33)
      if(ml(2).eq.-123456789) ml(2)=ml(3)
      if(ml(1).eq.-123456789) ml(1)=ml(2)
      ml2=ml**2
      mer(1)=extval(34)
      mer(2)=extval(35)
      mer(3)=extval(36)
      if(mer(2).eq.-123456789) mer(2)=mer(3)
      if(mer(1).eq.-123456789) mer(1)=mer(2)
      mer2=mer**2
      mq(1)=extval(41)
      mq(2)=extval(42)
      mq(3)=extval(43)
      if(mq(2).eq.-123456789) mq(2)=mq(3)
      if(mq(1).eq.-123456789) mq(1)=mq(2)
      mq2=mq**2
      mur(1)=extval(44)
      mur(2)=extval(45)
      mur(3)=extval(46)
      if(mur(2).eq.-123456789) mur(2)=mur(3)
      if(mur(1).eq.-123456789) mur(1)=mur(2)
      mur2=mur**2
      mdr(1)=extval(47)
      mdr(2)=extval(48)
      mdr(3)=extval(49)
      if(mdr(2).eq.-123456789) mdr(2)=mdr(3)
      if(mdr(1).eq.-123456789) mdr(1)=mdr(2)
      mdr2=mdr**2
      
      
      absAu(3)=abs(extval(11)+(0d0,1d0)*imextval(11))
      phiAu(3)=arg(extval(11),imextval(11))
      absAd(3)=abs(extval(12)+(0d0,1d0)*imextval(12))
      phiAd(3)=arg(extval(12),imextval(12))
      absAe(3)=abs(extval(13)+(0d0,1d0)*imextval(13))
      phiAe(3)=arg(extval(13),imextval(13))
      if(extval(14).eq.-123456789) then
      absAu(2)=absAu(3)
      phiAu(2)=phiAu(3)
      else
      call inputtest(CPcon,imextval(14),'ImAc      ',.TRUE.)
      if(CPcon) imextval(14)=0d0
      absAu(2)=abs(extval(14)+(0d0,1d0)*imextval(14))
      phiAu(2)=arg(extval(14),imextval(14))
      end if
      if(extval(15).eq.-123456789) then
      absAd(2)=absAd(3)
      phiAd(2)=phiAd(3)
      else
      call inputtest(CPcon,imextval(15),'ImAs      ',.TRUE.)
      if(CPcon) imextval(15)=0d0
      absAd(2)=abs(extval(15)+(0d0,1d0)*imextval(15))
      phiAd(2)=arg(extval(15),imextval(15))
      end if
      if(extval(16).eq.-123456789) then
      absAe(2)=absAe(3)
      phiAe(2)=phiAe(3)
      else
      call inputtest(CPcon,imextval(16),'ImAmu      ',.TRUE.)
      if(CPcon) imextval(16)=0d0
      absAe(2)=abs(extval(16)+(0d0,1d0)*imextval(16))
      phiAe(2)=arg(extval(16),imextval(16))
      end if
      if(extval(17).eq.-123456789) then
      absAu(1)=absAu(2)
      phiAu(1)=phiAu(2)
      else
      call inputtest(CPcon,imextval(17),'ImAu      ',.TRUE.)
      if(CPcon) imextval(17)=0d0
      absAu(1)=abs(extval(17)+(0d0,1d0)*imextval(17))
      phiAu(1)=arg(extval(17),imextval(17))
      end if
      if(extval(18).eq.-123456789) then
      absAd(1)=absAd(2)
      phiAd(1)=phiAd(2)
      else
      call inputtest(CPcon,imextval(18),'ImAd      ',.TRUE.)
      if(CPcon) imextval(18)=0d0
      absAd(1)=abs(extval(18)+(0d0,1d0)*imextval(18))
      phiAd(1)=arg(extval(18),imextval(18))
      end if
      if(extval(19).eq.-123456789) then
      absAe(1)=absAe(2)
      phiAe(1)=phiAe(2)
      else
      call inputtest(CPcon,imextval(19),'ImAe      ',.TRUE.)
      if(CPcon) imextval(19)=0d0
      absAe(1)=abs(extval(19)+(0d0,1d0)*imextval(19))
      phiAe(1)=arg(extval(19),imextval(19))
      end if
      
c input scale can either be explicitly given in slha file or 
c is set to mur(3)*mq(3) by default
c Renormalization is set equal to inputscale
      Qinp=extval(0)
      if(Qinp.eq.-123456789) then
      Qs=Sqrt(mur(3)*mq(3))
      print*,
      print*, '* Inputscale=Renormalizatonscale in Block EXTPAR',
     .        ' not explicitly given: '
      print*, '  assumed as Q=sqrt(mur(3)*mq(3))= ',Qs
      else
      Qs=Qinp
      print*,
      print*, '* Inputscale=Renormalizatonscale in Block',
     .        ' EXTPAR given as ',Qs
      endif
      
      close(nin)
c finished reading in 
      

c checking for errors in input
      if(CPcon) then
      print*,
      print*,'* CP conservation is assumed.'
      if(phasetest(phik).OR.phasetest(phil).OR.phasetest(phiu).OR.
     .   phasetest(phis).OR.phasetest(phiAu(1)).OR.phasetest(phiAu(1))
     .   .OR.phasetest(phiAu(2)).OR.phasetest(phiAu(3)).OR.
     .   phasetest(phiAd(1)).OR.phasetest(phiAd(2)).OR.
     .   phasetest(phiAd(3)).OR.phasetest(phiAe(1)).OR.
     .   phasetest(phiAe(2)).OR.phasetest(phiAe(3)).OR.
     .   phasetest(phiM1).OR.phasetest(phiM2).OR.phasetest(phiM3)) then
      print*, '  All phases must be multiples of pi!!!'
      print*, '  Check input or turn of CP conservation...'
      print*, '  Progam terminated...'
      print*
      call exit
      end if
      else
      print*
      print*,'* General CP violation.'
      end if
      print*

c nhung change here  6/2021 
      if(modselval(7).eq.1) then
      mQrun=.TRUE.
      igauge=0
      print*, '* Applying DR renormalization in top/stop-sector, O(alpha_s,alpha_t,alpha_lambda,alpha_kappa)'
      elseif(modselval(7).eq.2) then
      mQrun=.TRUE.
      igauge=1
      print*, '* Applying DR renormalization in top/stop-sector, O(alpha_s,alpha_t,alpha_lambda,alpha_kappa,alpha)'
      elseif(modselval(7).eq.3) then
      mQrun=.FALSE.
      print*, '* Applying OS renormalization in top/stop-sector'
      else
      print*, '* Warning: Please specify which scheme to use for
     . the top/stop-sector'
      print*, '  Switch 7 in Block MODSEL:'
      print*, '  1: DRbar, O(alpha_s,alpha_t,alpha_lambda,alpha_kappa)'
      print*, '  2: DRbar, O(alpha_s,alpha_t,alpha_lambda,alpha_kappa,alpha)'
      print*, '  3: OS'
      print*, '  Default: 1 DRbar, O(alpha_s,alpha_t,alpha_lambda,alpha_kappa)'
      mQrun=.TRUE.
      igauge=0
      end if
      print*
c end nhung change here  6/2021  
     


c beginning of actual calculation
     
c calcualte tree level masses
      call treelevelmasses
     
c loop calculation
      call HMass(Mh,ZH5,Mh1l,ZH1l,MHpm,MHpm1l)
     

      print*, '* Finished'
      print*
      print*, '* Tree-level masses:'
      if(CPcon) then
      print*, '  M_A1=', Mh(1)
      print*, '  M_A2=', Mh(2)
      print*, '  M_H1=', Mh(3)
      print*, '  M_H2=', Mh(4)
      print*, '  M_H3=', Mh(5)
      print*, '  M_Hp=', Mhpm
      print*
      if(onelswitch) then
      print*, '* One-loop Higgs masses:'
      else
      print*, '* Two-loop Higgs masses:'
      end if
      print*, '  M_A1=', Mh1l(1)
      print*, '  M_A2=', Mh1l(2)
      print*, '  M_H1=', Mh1l(3)
      print*, '  M_H2=', Mh1l(4)
      print*, '  M_H3=', Mh1l(5)
      print*, '  M_Hp=', Mhpm1l
      else
      print*, '  M_H1=', Mh(1)
      print*, '  M_H2=', Mh(2)
      print*, '  M_H3=', Mh(3)
      print*, '  M_H4=', Mh(4)
      print*, '  M_H5=', Mh(5)
      print*, '  M_Hp=', Mhpm
      print*
      if(onelswitch) then
      print*, '* One-loop Higgs masses:'
      else
      print*, '* Two-loop Higgs masses:'
      end if
      print*, '  M_H1=', Mh1l(1)
      print*, '  M_H2=', Mh1l(2)
      print*, '  M_H3=', Mh1l(3)
      print*, '  M_H4=', Mh1l(4)
      print*, '  M_H5=', Mh1l(5)
      print*, '  M_Hp=', Mhpm1l
      end if
      print*

      call writeoutput(output)
      
      return
      end
 
* ---------------------------------------------------------------------
!functions necessary for the running of the top and bottom mass
!running of alpha_s
! see hep-ph/0004189


      double precision function a_sMS(Q)
      
      implicit none
      
      double precision Q,mtpole,b0,b1,b2,mbMSmb,aQ,amt
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      integer n,i
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /topoles/ mtpole,mbMSmb
      
      amt=0d0
      aQ=0d0
      a_sMS=alphaS/pi
      
      if(Q.le.mtpole) then
      n=5
      b0=(11d0-2d0/3d0*n)/4d0
      b1=(102d0-38d0/3d0*n)/16d0
      i=1
      do while((abs(aQ-a_sMS).gt.1d-12).AND.(i.le.100))
      aQ=a_sMS
      a_sMS=(alphaS/pi*b0)/
     -  (b0 + alphaS/pi*b1*log(alphaS/pi) - alphaS/pi*b1*log(aQ) - 
     -    alphaS/pi*b1*log(b0 + alphaS/pi*b1) + 
     -    alphaS/pi*b1*log(b0 + aQ*b1) + 
     -    alphaS/pi*b0**2*log(Q**2/Mz**2))
      i=i+1
      end do
      else
      n=5
      b0=(11d0-2d0/3d0*n)/4d0
      b1=(102d0-38d0/3d0*n)/16d0
      i=1
      do while((abs(amt-a_sMS).gt.1d-14).AND.(i.le.1000))
      amt=a_sMS
      a_sMS=(alphaS/pi*b0)/
     -  (b0 + alphaS/pi*b1*log(alphaS/pi) - alphaS/pi*b1*log(amt) - 
     -    alphaS/pi*b1*log(b0 + alphaS/pi*b1) + 
     -    alphaS/pi*b1*log(b0 + amt*b1) + 
     -    alphaS/pi*b0**2*log(mtpole**2/Mz**2))

      i=i+1
      end do
      amt=a_sMS
      
      n=6
      b0=(11d0-2d0/3d0*n)/4d0
      b1=(102d0-38d0/3d0*n)/16d0
      i=1
      do while((abs(aQ-a_sMS).gt.1d-14).AND.(i.le.1000))
      aQ=a_sMS
      a_sMS=(amt*b0)/
     -  (b0 + amt*b1*log(amt) - amt*b1*log(aQ) - 
     -    amt*b1*log(b0 + amt*b1) + 
     -    amt*b1*log(b0 + aQ*b1) + 
     -    amt*b0**2*log(Q**2/mtpole**2))
      
      i=i+1
      end do
      end if
      
      
      a_sMS=a_sMS*pi

      return
      end
     
      
!
! see hep-ph/0207126
      double precision function a_sDR(Q)
      
      implicit none
      
      double precision Q,a_sMS,das
      double precision mt,Mz
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      mt=173.5d0
      Mz=91.1876d0
      
      das=a_sMS(Q)/2d0/pi*(1d0/2d0)
      
      a_sDR=a_sMS(Q)/(1-das)

      
      return
      end

!converting mt_pole to mt_MS at scale Q
!hep-ph/9912391
      double precision function mtMS(Q)
      
      implicit none
      
      double precision Q,mtpole,a_sMS,mtMSmt,Un,mbMSmb
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /topoles/ mtpole,mbMSmb
      
      mtMSmt=(1d0-4d0/3d0*(a_sMS(mtpole)/pi)
     &        -9.1253d0*(a_sMS(mtpole)/pi)**2)*mtpole
      
      mtMS=Un(6,Q,mtpole)*mtMSmt
      
      return
      end
      
!hep-ph/9912391
      double precision function mbMS(Q)
      implicit none
      
      double precision Q,mtpole,mbMSmb,Un
      
      common /topoles/ mtpole,mbMSmb
      
      mbMS=Un(6,Q,mtpole)*Un(5,mtpole,mbMSmb)*mbMSmb
      
      return
      end
      
!hep-ph/9912391
      double precision function Un(n,Q2,Q1)
      implicit none
      
      double precision Q2,Q1,a_sMS,d,J
      integer n
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      d=12d0/(33d0-2d0*n)
      J=-(8982d0-504*n+40*n**2)/3d0/(33d0-2d0*n)**2
      
      Un=(a_sMS(Q2)/a_sMS(Q1))**d*(1d0+(a_sMS(Q1)-a_sMS(Q2))/4d0/pi*J)
      
      return
      end
      
!hep-ph/9701308
      subroutine MStoDR(mMS,mDR,Q)
      implicit none
      
      double precision mMS,mDR,Q,a_sMS
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      mDR=mMS*(1d0-a_sMS(Q)/3d0/pi-
     .   (a_sMS(Q))**2/144d0/pi**2*(73d0-3d0*6d0))
      
      return 
      end
      
      
      double precision function I(a,b,c)
      
      implicit none
      
      double precision a,b,c
      
      I=-(a*b*log(a/b)+b*c*log(b/c)+c*a*log(c/a))/((a-b)*(b-c)*(c-a))
      
      return
      end
      

!hep-ph/9912516
!hep-ph/9606211
!hep-ph/0211467
      subroutine mtbeff(mtDR,mbDR,Q,mteff,mbeff)
      
      implicit none
      
      double precision mtDR,mbDR,Q,mteff,mbeff
      double precision a_sMS
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                   ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      double precision absM3,phiM3
      double precision mtpole,mbMsmb
      double precision Mh(5),MHpm,Mcha(2),Mchi(5),MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
     
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)

      double complex dmtc,dmbc
      double complex B1,B0
     
      double precision pi
      parameter (pi = 2d0*Asin(1d0))

      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /gluino/ absM3,phiM3
      common /topoles/ mtpole,mbMsmb
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      
      dmtc=a_sMs(Q)/6/pi*(-2*mtDR*Realpart(B1(mtDR**2,absM3**2,MSu(5)**2,Q**2)+
     &      B1(mtDR**2,absM3**2,MSu(6)**2,Q**2))+2*absM3*Realpart(
     &      B0(mtDR**2,absM3**2,MSu(5)**2,Q**2)-
     &      B0(mtDR**2,absM3**2,MSu(6)**2,Q**2))*(
     &      exp((0d0,1d0)*(phiM3+phiu))*ZU(6,6)*CONJG(ZU(6,5))+
     &      exp(-(0d0,1d0)*(phiM3+phiu))*ZU(6,5)*CONJG(ZU(6,6))))
     
      dmbc=a_sMs(Q)/6/pi*(-2*mbDR*Realpart(B1(mbDR**2,absM3**2,MSd(5)**2,Q**2)+
     &      B1(mbDR**2,absM3**2,MSd(6)**2,Q**2))+2*absM3*Realpart(
     &      B0(mbDR**2,absM3**2,MSd(5)**2,Q**2)-
     &      B0(mbDR**2,absM3**2,MSd(6)**2,Q**2))*(
     &      exp((0d0,1d0)*phiM3)*ZD(6,6)*CONJG(ZD(6,5))+
     &      exp(-(0d0,1d0)*phiM3)*ZD(6,5)*CONJG(ZD(6,6))))
     
      mteff=mtDR+Realpart(dmtc)
      mbeff=mbDR+Realpart(dmbc)

      return
      end
* ---------------------------------------------------------------------
      
! tests if phases are a multiple of pi in the case of the CP conserving
! NMSSM
      logical function phasetest(phi)
      
      implicit none
      
      double precision phi,test
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      if(Abs(DMOD(phi,pi)).LE.pi/2) then
           test=Abs(DMOD(phi,pi))
      else 
           test=Abs(DMOD(phi,pi)-pi)
      end if
     
      if(test.GE.1d-7) then
      phasetest=.TRUE.
      else
      phasetest=.FALSE.
      end if
      
      return
      end


      double precision function arg(x,y)
      
      implicit none
      
      double precision x,y
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      arg=0d0
      
      if(x.gt.0) then
      arg=Atan(y/x)
      elseif(x.lt.0 .AND. y.ge.0) then
      arg=Atan(y/x)+pi
      elseif(x.lt.0 .AND. y.lt.0) then
      arg=Atan(y/x)-pi
      elseif(x.eq.0 .AND. y.gt.0) then
      arg=pi/2
      elseif(x.eq.0 .AND. y.lt.0) then
      arg=-pi/2
      elseif(x.eq.0 .AND. y.eq.0) then
      arg=0d0
      endif
     
      return
      end
      
      
      subroutine inputtest(CPcon,para,name,compval)
      
      implicit none
      
      double precision para,unlikely
      logical CPcon,compval
      character*10 name
      
      unlikely=-123456789d0
      
      if(CPcon) then
      
        if(compval) then
      
          if(para.ne.unlikely .AND. abs(para).ge.1d-8) then
          print*, '* Warning: if CP-conservation is assumed ',name
     &          , 'should be zero.'
          call exit
          else
          para=0d0
          end if
     
        else
      
          if(para.eq.unlikely) then
          print*
          print*, '* Warning: parameter value missing in slha file: ',
     .         name
          call exit
          end if
        
        end if
      
      else
      
        if(para.eq.unlikely) then
        print*
        print*, '* Warning: parameter value missing in slha file: ',name
        call exit
        end if
      
      end if
      
      return
      end


ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccc Tree-Level-Masses cccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
     
      subroutine treelevelmasses

      implicit none

      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      
      double precision RAk,RAl,IAk,IAl,arg
      
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision mtpole,mbMSmb,mtms,mbms,mtDR,mbDR
      
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
     
      double precision Qs
     
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double complex nZU(6,6),nZD(6,6)
      
      double precision nMSu(6),nMSd(6)
      
      double precision a_sMS,a_sDR
     
      double precision vu,vd
      
      double precision absAtDR,phiAtDR,mq2DR,mur2DR,absold,absnew
      
      double complex dmtfin,dmtdiv1,dYtfin,dYtdiv1
      
      double complex dMstfin(2,2),dMstdiv1(2,2),dAtfin,
     &               dAtdiv1,dphiAtfin,dphiAtdiv1
     
      double complex dmQLsq,dmtRsq
     
      double precision pi
c Maggie changed 14/1/2019
      double complex AT
c end Maggie changed 14/1/2019      
      parameter (pi = 2d0*Asin(1d0))
      
      logical MHpmInput,CPcon,mQrun,onelswitch
      
      integer i,j
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
     
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /topoles/ mtpole,mbMsmb
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
      common /slhapara/ RAl,RAk
      common /DRstop/ absAtDR,phiAtDR,mq2DR,mur2dR
      common /nSu/ nMSu,nZU
      common /nSd/ nMSd,nZD
      common /scale/ Qs
c nhung added for atat 22/3/2019
      integer ilevel,igauge 
      common/ilevelATAT/ilevel,igauge
      double precision g3Q, mtQ, vQ, mhQ, asDR,g1Q,g2Q
      double precision dmsq4fin,dmsur4fin 
      double complex dAt4fin
c end nhung added 22/3/2019
c nhung added for atlambda 6/2021
      integer ierror
      double precision nmh(6),nZH(6,6),nMNeu(5),nMCha
      double complex nZneu(5,5)
      common /nAtlambda/nmh,nZH,nMNeu,nZneu,nMCha   
c end nhung added for atlambda 6/2021 
 
c calculate the other SM parameters form the input
      e=sqrt(4*pi*alpha)
!       Mw=sqrt(2*Mz**2 + (sqrt(2d0)*sqrt(Gf*Mz**2*
!      &     (-(sqrt(2d0)*e**2) + 2*Gf*Mz**2)))/Gf)/2.
      g1=(e*Mz)/Mw
      g2=e/sqrt(1-Mw**2/Mz**2)
      v=2d0*Mw/g2
!       v=1/sqrt(Gf*sqrt(2d0))

      

c calcuate other parameters of the Higgs sector from input
      beta=dAtan(tanb)
      vu=v*Sin(beta)
      vd=v*Cos(beta)
      vs=sqrt(2d0)*mueff/abslambda
     
c Bug fixed Kathrin 21/09/2014
        if(MHpmInput) then
        RAl=(-4*abskappa*abslambda*vs**2*Cos(phik + 3*Phis) + 
     -      (4*MHpm**2 + (2*abslambda**2 - g2**2)*v**2)
     -      *Cos(phil + Phis + Phiu)*Sin(2*beta))/
     -      (4.*Sqrt(2d0)*abslambda*vs)
        else
        MHpm=Sqrt(((-2*abslambda**2 + g2**2)*v**2 + 
     -        4*abslambda*vs*(Sqrt(2d0)*RAl+
     -        abskappa*vs*Cos(phik + 3*Phis))/Sin(2*beta)/
     -        Cos(phil + Phis + Phiu))/4.)

        end if
c End bug-fix Kathrin 21/09/2014
        
        IAk=(3*abslambda*v**2*Cos(beta)/Cos(phik + 3*phis)*Sin(beta)*
     -      Sin(phik - phil + 2*phis - phiu))/(Sqrt(2d0)*vs) - 
     -      RAk*Tan(phik + 3*phis)
        IAl=(abskappa*vs/Cos(phil + phis + phiu)*
     -       Sin(phik - phil + 2*phis - phiu))/
     -       Sqrt(2d0) - RAl*Tan(phil + phis + phiu)
     
        absAlambda=Abs(RAl+(0d0,1d0)*IAl)
        phiAl=arg(RAl,IAl)
        absAkappa=Abs(RAk+(0d0,1d0)*IAk)
        phiAk=arg(RAk,IAk)

    
c calculate the Yukawa couplings
      mt=mtpole
      mb=(1d0+4d0/3d0*a_sMS(mbMSmb)/pi+
     &    8.2364d0*(a_sMS(mbMSmb)/pi)**2)*mbMSmb
      Ye(1) = me*sqrt(2d0)/v/Cos(beta)
      Ye(2) = mmu*sqrt(2d0)/v/Cos(beta)
      Ye(3) = mtau*sqrt(2d0)/v/Cos(beta)
      Yu(1) = mu*sqrt(2d0)/v/Sin(beta)
      Yu(2) = mc*sqrt(2d0)/v/Sin(beta)
      Yu(3) = mt*sqrt(2d0)/v/Sin(beta)
      Yd(1) = md*sqrt(2d0)/v/Cos(beta)
      Yd(2) = ms*sqrt(2d0)/v/Cos(beta)
      Yd(3) = mb*sqrt(2d0)/v/Cos(beta)
      
     
 
      call higgstreelevel(v,beta,g1,g2,vs,abslambda,phil,abskappa,phik,
     &         absAkappa,phiAk,absAlambda,phiAl,phiu,phis,Mh,ZH5,CPcon)

      
      do i=1,6,1
         do j=1,6,1
           if((i.ne.1).and.(j.ne.1)) then
                 ZH(i,j)=ZH5(i-1,j-1)
              else if ((i.eq.1).and.(j.eq.1)) then
                 ZH(i,j)=1
              else
                 ZH(i,j)=0
           end if
         end do
      end do
      


      call charginomasses(absM2,phiM2,g2,v,vs,abslambda,phil,
     &            phiu,phis,beta,UP,UM,MCha)
    

      call neutralinomasses(absM1,phiM1,absM2,phiM2,g1,g2,v,vs,
     &          abslambda,phil,abskappa,phik,phiu,phis,beta,NN,MChi,
     &          CPcon)
    
       
      call sleptonmasses(g1,g2,v,beta,abslambda,vs,phil,phis,phiu,
     &                   phiAe,absAe,ml2,mer2,Ye,MSe,ZE)
        
        
      call dsquarkmasses(g1,g2,v,beta,abslambda,vs,phil,phis,phiu,phiAd,
     &                   absAd,mq2,mdr2,Yd,MSd,ZD)
          

      call usquarkmasses(g1,g2,v,beta,abslambda,vs,phil,phis,phiu,phiAu,
     &                   absAu,mq2,mur2,Yu,MSu,ZU)
     
      call usquarkmasses(0d0,0d0,v,beta,abslambda,vs,phil,phis,phiu,phiAu,
     &                   absAu,mq2,mur2,Yu,nMSu,nZU)
    
    
      g3=Sqrt(a_sDR(Qs)*4d0*pi)
      
!running of the top and bottom quark masses
      absAtDR=absAu(3)
      phiAtDR=phiAu(3)
      mq2DR=mq2(3)
      mur2DR=mur2(3)
c nhung add for atat correction 22/3/2019
      if(igauge.eq.1)then
      call RGEevolutiong1g2(MZ,Qs,g1Q,g2Q,g3Q,mtQ,vQ,mhQ)
      call conversionMStoDRg1g2(mtQ,g1Q,g2Q,g3Q,mtDR,asDR)
      else
      call RGEevolution(MZ, Qs, g3Q, mtQ, vQ, mhQ)
      call conversionMStoDR(mtQ, g3Q, mtDR, asDR)     
      endif
      g3=Sqrt(asDR*4d0*pi)   
c end nhung add for atat correction 22/3/2019

      if(mQrun) then
c nhung change here 22/3/2019
      call MStoDR(mbMS(Qs),mbDR,Qs)
      Yu(3) = mtDR*sqrt(2d0)/v/Sin(beta)
      Yd(3) = mbDR*sqrt(2d0)/v/Cos(beta)
      
      call dsquarkmasses(0d0,0d0,v,beta,abslambda,vs,phil,phis,phiu,phiAd,
     &                   absAd,mq2,mdr2,Yd,nMSd,nZD)
          

      call usquarkmasses(0d0,0d0,v,beta,abslambda,vs,phil,phis,phiu,phiAu,
     &                   absAu,mq2,mur2,Yu,nMSu,nZU)

100   continue

      if(ilevel.lt.4)then
      call mtbeffat(mtDR,mbDR,g3,mhQ,vQ,Qs,mt,mb)
      else
      ierror=0
      call higgstreeha(v,beta,0d0,0d0,vs,abslambda,phil,abskappa,phik,
     &         absAkappa,phiAk,absAlambda,phiAl,phiu,phis,nMh,nZH,CPcon,ierror)
      if (ierror.eq.1)then
         ilevel=3
         go to 100
      endif
      call neutralinomasses(0d0,0d0,0d0,0d0,0d0,0d0,v,vs,
     &          abslambda,phil,abskappa,phik,phiu,phis,beta,nZNeu,nMneu,
     &          CPcon)
       
      nMCha=Abs(mueff)

      call mtbeffatlambda(mtDR,mbDR,g3,mhQ,vQ,Qs,mt,mb)
      endif
      Yu(3) = mt*sqrt(2d0)/v/Sin(beta)
      Yd(3) = mb*sqrt(2d0)/v/Cos(beta)

c end nhung change here 22/3/2019
      else
      
c if OS renormalization is chosen for the stop/top sector
c At, mq2(3) and mur2(3) have to be converted to OS values

      call qcdcouplings
      call topsector(dmtfin,dmtdiv1,dYtfin,dYtdiv1)
      call stopsector(dmtfin,dmtdiv1,dMstfin,dMstdiv1,
     &                dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1)

c nhung changed for atlambda correction 6/2021
101   continue
      if(ilevel.lt.4)then
      call MSQ_MSU_AT_OStoDR(dmsq4fin,dmsur4fin,dAt4fin)
      else
      call MSQ_MSU_AT_OStoDR_atlamda(dmsq4fin,dmsur4fin,dAt4fin,ierror)
         if (ierror .eq.1) then
          ilevel=3
         goto 101
         endif
      endif
c end nhung changed for atlambda correction 6/2021   
  
      absnew=1d9
      absold=absAtDR
      i=1
      do while(Abs(absold-absnew).ge.1d-3 .AND. i.le.100)
      absold=absnew
      call qcdcouplings
      call topsector(dmtfin,dmtdiv1,dYtfin,dYtdiv1)
      call stopsector(dmtfin,dmtdiv1,dMstfin,dMstdiv1,
     &                dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1)

c nhung changed for atlambda correction 6/2021
      if(ilevel.lt.4)then
      call MSQ_MSU_AT_OStoDR(dmsq4fin,dmsur4fin,dAt4fin)
      else
      call MSQ_MSU_AT_OStoDR_atlamda(dmsq4fin,dmsur4fin,dAt4fin,ierror)
      endif
c end nhung changed for atlambda correction 6/2021   

c Maggie changed 14/1/2019
c Mistake found by Nhung      
c      absAu(3)=absAtDR-Realpart(dAtfin)
c      phiAu(3)=phiAtDR-Realpart(dphiAtfin)

c nhung change here for atat 22/3/2019
      AT= absAtDR*(Cos(phiAtDR) + (0,1D0)*sin(phiAtDR))-
     &    dAtfin*(Cos(phiAtDR) + (0,1D0)*sin(phiAtDR))-
     &    absAtDR*dphiAtfin*((0,1D0)*Cos(phiAtDR)-sin(phiAtDR))-
     &    dAt4fin
c end nhung change here for atat 22/3/2019

      absAu(3)=Abs(AT)
      phiAu(3)=arg(DBLE(AT),DIMAG(AT))      
c end Maggie changed 14/1/2019
      
      dmQLsq=Abs(nZU(5,5))**2*dMstfin(1,1)+Abs(nZU(5,6))**2*dMstfin(2,2)
     &      -nZU(6,6)*CONJG(nZU(5,6))*dMstfin(1,2)
     &      -nZU(5,6)*CONJG(nZU(6,6))*dMstfin(2,1)
     &      -2*mt*dmtfin
     
      dmtRsq=Abs(nZU(6,5))**2*dMstfin(1,1)+Abs(nZU(6,6))**2*dMstfin(2,2)
     &      -nZU(5,5)*CONJG(nZU(6,5))*dMstfin(2,1)
     &      -nZU(6,5)*CONJG(nZU(5,5))*dMstfin(1,2)
     &      -2*mt*dmtfin
     
c nhung change for atat correction 22/3/2019    
      mq2(3)=mq2DR-Realpart(dmQLsq)-dmsq4fin
      mur2(3)=mur2DR-Realpart(dmtRsq)-dmsur4fin
c end nhung change for atat correction 22/3/2019
      
      mt=mtpole
      Yu(3) = mt*sqrt(2d0)/v/Sin(beta)
      call usquarkmasses(g1,g2,v,beta,abslambda,vs,phil,phis,phiu,phiAu,
     &                   absAu,mq2,mur2,Yu,MSu,ZU)
      call usquarkmasses(0d0,0d0,v,beta,abslambda,vs,phil,phis,phiu,phiAu,
     &                   absAu,mq2,mur2,Yu,nMSu,nZU)
      absnew=absAu(3)
      i=i+1
      end do
      
      if(i.eq.101) then
      print*, '*Warning: Conversion form AtDR to AtOS
     . does not converge properly'
      print*, '  Using simplified conversion; i.e. no iteration'
      absAu(3)=absAtDR
      mur2(3)=mur2DR
      mq2(3)=mq2DR
      call qcdcouplings
      call topsector(dmtfin,dmtdiv1,dYtfin,dYtdiv1)
      call stopsector(dmtfin,dmtdiv1,dMstfin,dMstdiv1,
     &                dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1)
c nhung changed for atlambda correction 6/2021
      if(ilevel.lt.4)then
      call MSQ_MSU_AT_OStoDR(dmsq4fin,dmsur4fin,dAt4fin)
      else
      call MSQ_MSU_AT_OStoDR_atlamda(dmsq4fin,dmsur4fin,dAt4fin,ierror)
      endif
c end nhung changed for atlambda correction 6/2021  

c Maggie changed 14/1/2019
c Mistake found by Nhung      
c      absAu(3)=absAtDR-Realpart(dAtfin)
c      phiAu(3)=phiAtDR-Realpart(dphiAtfin)

c nhung change here for atat 22/3/2019
      AT= absAtDR*(Cos(phiAtDR) + (0,1D0)*sin(phiAtDR))-
     &    dAtfin*(Cos(phiAtDR) + (0,1D0)*sin(phiAtDR))-
     &    absAtDR*dphiAtfin*((0,1D0)*Cos(phiAtDR)-sin(phiAtDR))-
     &    dAt4fin
c end nhung change here for atat 22/3/2019

      absAu(3)=Abs(AT)
      phiAu(3)=arg(DBLE(AT),DIMAG(AT))      
c end Maggie changed 14/1/2019
      
      dmQLsq=Abs(nZU(5,5))**2*dMstfin(1,1)+Abs(nZU(5,6))**2*dMstfin(2,2)
     &      -nZU(6,6)*CONJG(nZU(5,6))*dMstfin(1,2)
     &      -nZU(5,6)*CONJG(nZU(6,6))*dMstfin(2,1)
     &      -2*mt*dmtfin
     
      dmtRsq=Abs(nZU(6,5))**2*dMstfin(1,1)+Abs(nZU(6,6))**2*dMstfin(2,2)
     &      -nZU(5,5)*CONJG(nZU(6,5))*dMstfin(2,1)
     &      -nZU(6,5)*CONJG(nZU(5,5))*dMstfin(1,2)
     &      -2*mt*dmtfin
     
c nhung change for atat correction 22/3/2019    
      mq2(3)=mq2DR-Realpart(dmQLsq)-dmsq4fin
      mur2(3)=mur2DR-Realpart(dmtRsq)-dmsur4fin
c end nhung change for atat correction 22/3/2019 

      end if
      
      end if
      
      call dsquarkmasses(g1,g2,v,beta,abslambda,vs,phil,phis,phiu,phiAd,
     &                   absAd,mq2,mdr2,Yd,MSd,ZD)
          

      call usquarkmasses(g1,g2,v,beta,abslambda,vs,phil,phis,phiu,phiAu,
     &                   absAu,mq2,mur2,Yu,MSu,ZU)
      
      call usquarkmasses(0d0,0d0,v,beta,abslambda,vs,phil,phis,phiu,phiAu,
     &                   absAu,mq2,mur2,Yu,nMSu,nZU)
     
      nMSd(5)=sqrt(mq2(3))
c set the right-handed bottom to some unlikely high number, because it does not 
c contribute to avoid problems with mass degeneracy
      nMSd(6)=123.45678d0
      
      do i=5,6
      do j=5,6
      if (i.eq.j) then
      nZD(i,j)=1d0
      else
      nZD(i,j)=0d0
      end if
      end do
      end do
      
      call sneutrinomasses(g1,g2,v,beta,ml2,MSv)
     
       
      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccc Higgs Masses at tree-levelcccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine higgstreelevel(v,beta,g1,g2,vs,abslambda,phil,
     &                          abskappa,phik,absAkappa,phiAk,
     &                          absAlambda,phiAl,phiu,phis,Mh,ZH,CPcon)
     
      implicit none
      
      double precision v,beta,g1,g2,vs,abslambda,phil,abskappa,phik,
     &                 absAkappa,phiAk,absAlambda,phiAl,phiu,phis,Mh(5)
     &                 ,ZH(5,5),Mh2(5)
    
     
      double precision phi,phiIl,phiIk
      double complex MhMatrix(5,5),U(5,5)
      
      integer i,j
      
      logical CPcon
      
      double complex Meven(3,3),Modd(2,2),Ueven(3,3),Uodd(2,2)
      double precision Mhh2(3),MAh2(2)
      
      logical mainrun,calcmasseserror
      
      COMMON/calcerror/mainrun,calcmasseserror
    
      phi=-phik + phil - 2*phis + phiu
      phiIl=phis + phiAl + phil + phiu
      phiIk=3*phis + phiAk + phik 
      
      MhMatrix(1,1)=abslambda*vS*(abskappa*vS*Cos(phi) + Sqrt(2d0)*
     &               absAlambda*Cos(phiIl))/Sin(2*beta)
      MhMatrix(1,2)=-(abskappa*abslambda*v*vS*Cos(phi)) + 
     &                (absAlambda*abslambda*v*Cos(phiIl))/Sqrt(2d0)
      MhMatrix(1,3)=(abslambda*vS*Cos(beta)*(abskappa*vS*Sin(phi) 
     &                 + Sqrt(2d0)*absAlambda*Sin(phiIl)))/2.
      MhMatrix(1,4)=(abslambda*vS*Sin(beta)*(abskappa*vS*Sin(phi) 
     &                 + Sqrt(2d0)*absAlambda*Sin(phiIl)))/2.
      MhMatrix(1,5)=abskappa*abslambda*v*vS*Sin(phi) + 
     &                (absAlambda*abslambda*v*Sin(phiIl))/Sqrt(2d0)
      MhMatrix(2,1)=MhMatrix(1,2)
      MhMatrix(2,2)=(-3*Sqrt(2d0)*absAkappa*abskappa*vS**2*Cos(phiIk) + 
     &              abslambda*v**2*Cos(beta)*(4*abskappa*vS*Cos(phi) +
     &              Sqrt(2d0)*absAlambda*Cos(phiIl))*Sin(beta))/(2.*vS)
      MhMatrix(2,3)=(abslambda*v*Sin(beta)*(-2*abskappa*vS*Sin(phi) + 
     &               Sqrt(2d0)*absAlambda*Sin(phiIl)))/2.
      MhMatrix(2,4)=(abslambda*v*Cos(beta)*(-2*abskappa*vS*Sin(phi) + 
     &               Sqrt(2d0)*absAlambda*Sin(phiIl)))/2.
      MhMatrix(2,5)=-(abskappa*(abslambda*v**2*Cos(beta)*Sin(beta)*
     &                 Sin(phi) + Sqrt(2d0)*absAkappa*vS*Sin(phiIk)))
      MhMatrix(3,1)=MhMatrix(1,3)
      MhMatrix(3,2)=MhMatrix(2,3)
      MhMatrix(3,3)=((g1**2 + g2**2)*v**2*Cos(beta)**2 + 2*abslambda*vS
     &              *(abskappa*vS*Cos(phi) + Sqrt(2d0)*absAlambda*
     &               Cos(phiIl))*Tan(beta))/4.
      MhMatrix(3,4)=(-2*abslambda*vS*(abskappa*vS*Cos(phi) + Sqrt(2d0)*
     &              absAlambda*Cos(phiIl)) + (4*abslambda**2 - g1**2 -
     &               g2**2)*v**2*Cos(beta)*Sin(beta))/4.
      MhMatrix(3,5)=(abslambda*v*(2*abslambda*vS*Cos(beta) - 
     &              (2*abskappa*vS*Cos(phi) + Sqrt(2d0)*absAlambda*
     &               Cos(phiIl))*Sin(beta)))/2.
      MhMatrix(4,1)=MhMatrix(1,4)
      MhMatrix(4,2)=MhMatrix(2,4)
      MhMatrix(4,3)=MhMatrix(3,4)
      MhMatrix(4,4)=(2*abslambda*vS*(abskappa*vS*Cos(phi) + Sqrt(2d0)*
     &              absAlambda*Cos(phiIl))/Tan(beta) + (g1**2 + g2**2)
     &               *v**2*Sin(beta)**2)/4.
      MhMatrix(4,5)=(abslambda*v*(-(Cos(beta)*(2*abskappa*vS*Cos(phi) + 
     &               Sqrt(2d0)*absAlambda*Cos(phiIl))) + 
     &                2*abslambda*vS*Sin(beta)))/2.
      MhMatrix(5,1)=MhMatrix(1,5)
      MhMatrix(5,2)=MhMatrix(2,5)
      MhMatrix(5,3)=MhMatrix(3,5)
      MhMatrix(5,4)=MhMatrix(4,5)
      MhMatrix(5,5)=2*abskappa**2*vS**2 + (absAkappa*abskappa*vS**2*
     &              Cos(phiIk) + absAlambda*abslambda*v**2*Cos(beta)*
     &              Cos(phiIl)*Sin(beta))/(Sqrt(2d0)*vS)
     

      if(CPcon) then
        
         do i=1,3,1
           do j=1,3,1
               Meven(i,j)=MhMatrix(i+2,j+2)
           end do
         end do
        
         do i=1,2,1
           do j=1,2,1
               Modd(i,j)=MhMatrix(i,j)
           end do
         end do
        
         call HEigensystem(3,Meven,3,Mhh2,Ueven,3,1)
         call HEigensystem(2,Modd,2,MAh2,Uodd,2,1)
         
         do i=1,2,1
            if(MAh2(i).le.0) then
              if(mainrun) then
              print*, '* Warning!! Negative Eigenvalues, MA^2<0!!'
              print*, "MA_1^2=",MAh2(i)
              print*, '  Program terminated...'
              print*
              call exit
              else
              calcmasseserror=.TRUE.
              end if
            end if
         end do
         
         do i=1,3,1
            if(Mhh2(i).le.0) then
              if(mainrun) then
              print*, '* Warning!! Negative Eigenvalues, Mh^2<0!!'
              print*, "MH_1^2=",Mhh2(i)
              print*, '  Program terminated...'
              print*
              call exit
              else
              calcmasseserror=.TRUE.
              end if
            end if
         end do
        
         do i=1,2,1
          Mh(i)=Sqrt(MAh2(i))
          Mh(i+2)=Sqrt(Mhh2(i))
         end do
          Mh(5)=Sqrt(Mhh2(3))
        
         do i=1,5,1
           do j=1,5,1
              if((i.le.2).and.(j.le.2)) then
                  ZH(i,j)=realpart(Uodd(i,j))
                else if((i.ge.3).and.(j.ge.3)) then
                  ZH(i,j)=realpart(Ueven(i-2,j-2))
                else
                  ZH(i,j)=0d0
              end if
           end do
         end do
      
      else

        call HEigensystem(5,MhMatrix,5,Mh2, U,5,1)
        
         do i=1,5,1
            if(Mh2(i).le.0) then
              if(mainrun) then
              print*, '* Warning!! Negative Eigenvalues, Mh^2<0!!'
              print*, "MH_1^2=",Mh2(i)
              print*, '  Program terminated...'
              print*
              call exit
              else
              calcmasseserror=.TRUE.
              end if
            end if
         end do

        Mh=sqrt(Mh2)
        ZH=realpart(U)
      end if
      
      return
      end
      
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccc Charginos at tree-levelccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine charginomasses(absM2,phiM2,g2,v,vs,abslambda,phil,
     &            phiu,phis,beta,UP,UM,MCha)
     
      implicit none
      double precision absM2,phiM2,g2,v,vs,abslambda,phil,
     &            phiu,phis,beta,MCha(2)
     
      double complex UP(2,2),UM(2,2),MatrixCha(2,2)
      
      MatrixCha(1,1)=absM2*(Cos(phiM2) + (0,1)*Sin(phiM2))
      MatrixCha(1,2)=(g2*v*Sin(beta)*(Cos(Phiu) - 
     &                (0,1)*Sin(Phiu)))/Sqrt(2d0)
      MatrixCha(2,1)=(g2*v*Cos(beta))/Sqrt(2d0)
      MatrixCha(2,2)=(abslambda*vS*(Cos(Phil) + (0,1)*Sin(Phil))*
     &                (Cos(Phis) + (0,1)*Sin(Phis)))/Sqrt(2d0)
     
      
      call SVD(2,2,MatrixCha,2,MCha,UM,2,UP,2,1)
     
      return
      end
      
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccc Neutralinos at tree-levelccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine neutralinomasses(absM1,phiM1,absM2,phiM2,g1,g2,v,vs,
     &       abslambda,phil,abskappa,phik,phiu,phis,beta,NN,MChi,CPcon)
     
      implicit none 
      double precision absM1,phiM1,absM2,phiM2,g1,g2,v,vs,
     &          abslambda,phil,abskappa,phik,phiu,phis,beta,MChi(5)
     
      double complex NN(5,5),MatrixChi(5,5)
      
      logical CPcon
      
      integer i,j
      
      MatrixChi(1,1)=absM1*(Cos(phiM1) + (0,1)*Sin(phiM1))
      MatrixChi(1,2)=0
      MatrixChi(1,3)=-(g1*v*Cos(beta))/2.
      MatrixChi(1,4)=(g1*v*Sin(beta)*(Cos(phiu) - (0,1)*Sin(phiu)))/2.
      MatrixChi(1,5)=0
      MatrixChi(2,2)=absM2*(Cos(phiM2) + (0,1)*Sin(phiM2))
      MatrixChi(2,3)=(g2*v*Cos(beta))/2.
      MatrixChi(2,4)=-(g2*v*Sin(beta)*(Cos(phiu) - (0,1)*Sin(phiu)))/2.
      MatrixChi(2,5)=0
      MatrixChi(3,3)=0
      MatrixChi(3,4)=-((abslambda*vS*(Cos(phil) + (0,1)*Sin(phil))*
     &                (Cos(phis) + (0,1)*Sin(phis)))/Sqrt(2d0))
      MatrixChi(3,5)=-((abslambda*v*Sin(beta)*(Cos(phil) + (0,1)*
     &        Sin(phil))*(Cos(phiu) + (0,1)*Sin(phiu)))/Sqrt(2d0))
      MatrixChi(4,4)=0
      MatrixChi(4,5)=-((abslambda*v*Cos(beta)*(Cos(phil) + 
     &                (0,1)*Sin(phil)))/Sqrt(2d0))
      MatrixChi(5,5)=Sqrt(2d0)*abskappa*vS*(Cos(phik) + (0,1)*
     &                 Sin(phik))*(Cos(phis) + (0,1)*Sin(phis))
     
      call TakagiFactor(5,MatrixChi,5,MChi,NN,5,1)
      
      if(CPcon) then
         do i=1,5,1
           if(abs(realpart(NN(i,1))).le.1d-15) then
             do j=1,5,1
                NN(i,j)=(0,1)*NN(i,j)
             end do
             MChi(i)=-MChi(i)
           end if
         end do
      end if
      
      return
      end
      
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccc Sleptons at tree-levelccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine sleptonmasses(g1,g2,v,beta,abslambda,vs,phil,phis,
     &                         phiu,phiAe,absAe,ml2,mer2,Ye,MSe,ZE)
      
      implicit none
      
      double precision g1,g2,v,beta,abslambda,vs,phil,
     &                 phis,phiu,phiAe(3),absAe(3),ml2(3),mer2(3),
     &                 Ye(3),MSe(6)
      
      double complex MatrixSetemp(3,2,2),ZEtemp(3,2,2),ZE(6,6)
      double precision MSetemp(3,2)
      
      integer i,j
      
      logical mainrun,calcmasseserror
      
      COMMON/calcerror/mainrun,calcmasseserror
          
      do i=1,3,1
         MatrixSetemp(i,1,1)=(8*ml2(i) + 2*v**2*Ye(i)**2 + v**2*
     &                     Cos(2*beta)*(g1**2 - g2**2 + 2*Ye(i)**2))/8.
         MatrixSetemp(i,1,2)= (v*(-(abslambda*vS*Sin(beta)*(Cos(phil +
     &                  phis + phiu) - (0,1)*Sin(phil + phis +phiu))) + 
     &                  Sqrt(2d0)*absAe(i)*Cos(beta)*(Cos(phiAe(i)) + 
     &                  (0,1)*Sin(phiAe(i))))*Ye(i))/2.
         MatrixSetemp(i,2,1)=(v*(-(abslambda*vS*Sin(beta)*(Cos(phil + 
     &                 phis + phiu) + (0,1)*Sin(phil + phis + phiu))) + 
     &                 Sqrt(2d0)*absAe(i)*Cos(beta)*(Cos(phiAe(i)) - 
     &                  (0,1)*Sin(phiAe(i))))*Ye(i))/2.
         MatrixSetemp(i,2,2)=(4*mer2(i) + v**2*Ye(i)**2 - v**2*
     &                   Cos(2*beta)*(g1**2 - Ye(i)**2))/4.
      end do
      
          
      call HEigensystem(2,MatrixSetemp(1,1:2,1:2),2,MSetemp(1,1:2),
     &                  ZEtemp(1,1:2,1:2),2,1)
      call HEigensystem(2,MatrixSetemp(2,1:2,1:2),2,MSetemp(2,1:2),
     &                  ZEtemp(2,1:2,1:2),2,1)
      call HEigensystem(2,MatrixSetemp(3,1:2,1:2),2,MSetemp(3,1:2),
     &                  ZEtemp(3,1:2,1:2),2,1)
        
     
      MSe(1:2)=sqrt(MSetemp(1,1:2))
      MSe(3:4)=sqrt(MSetemp(2,1:2))
      MSe(5:6)=sqrt(MSetemp(3,1:2))
     
      do i=1,6,1
         do j=1,6,1
          ZE(i,j)=(0.d0,0.d0)
         end do
      end do
    
      ZE(1:2,1:2)=CONJG(ZEtemp(1,1:2,1:2))
      ZE(3:4,3:4)=CONJG(ZEtemp(2,1:2,1:2))
      ZE(5:6,5:6)=CONJG(ZEtemp(3,1:2,1:2))
      
      do i=1,3
      do j=1,2
            if(MSetemp(i,j).le.0) then
              if(mainrun) then
              print*, '* Warning!! Eigenvalues of selectron mass'
     .                ,' matrix are negative!!'
              print*, '  Program terminated...'
              print*
              call exit
              else
              calcmasseserror=.TRUE.
              end if
            end if
      end do
      end do

      return
      end
          
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccc d-type squarks at tree-levelccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine dsquarkmasses(g1,g2,v,beta,abslambda,vs, phil,phis,
     &                         phiu,phiAd,absAd,mq2,mdr2,Yd,MSd,ZD)
      
      implicit none
      
      double precision g1,g2,v,beta,abslambda,vs,phil,
     &                 phis,phiu,phiAd(3),absAd(3),mq2(3),mdr2(3),
     &                 Yd(3),MSd(6)
      
      double complex MatrixSdtemp(3,2,2),ZDtemp(3,2,2),ZD(6,6)
      double precision MSdtemp(3,2)
      
      integer i,j
      
      logical mainrun,calcmasseserror
      
      COMMON/calcerror/mainrun,calcmasseserror

          
      do i=1,3,1
         MatrixSdtemp(i,1,1)=(-((g1**2 + 3*g2**2)*v**2*Cos(2*beta)) +
     &                  24*mq2(i) + 12*v**2*Cos(beta)**2*Yd(i)**2)/24.
         MatrixSdtemp(i,1,2)= (v*(-(abslambda*vS*Sin(beta)*(Cos(phil +
     &                  phis + phiu) - (0,1)*Sin(phil + phis +phiu))) + 
     &                  Sqrt(2d0)*absAd(i)*Cos(beta)*(Cos(phiAd(i)) + 
     &                  (0,1)*Sin(phiAd(i))))*Yd(i))/2.
         MatrixSdtemp(i,2,1)=(v*(-(abslambda*vS*Sin(beta)*(Cos(phil + 
     &                 phis + phiu) + (0,1)*Sin(phil + phis + phiu))) + 
     &                 Sqrt(2d0)*absAd(i)*Cos(beta)*(Cos(phiAd(i)) - 
     &                  (0,1)*Sin(phiAd(i))))*Yd(i))/2.
         MatrixSdtemp(i,2,2)=-(g1**2*v**2*Cos(2*beta))/12. + mdr2(i) +
     &                  (v**2*Cos(beta)**2*Yd(i)**2)/2.
      end do
   
          
      call HEigensystem(2,MatrixSdtemp(1,1:2,1:2),2,MSdtemp(1,1:2),
     &                  ZDtemp(1,1:2,1:2),2,1)
      call HEigensystem(2,MatrixSdtemp(2,1:2,1:2),2,MSdtemp(2,1:2),
     &                  ZDtemp(2,1:2,1:2),2,1)
      call HEigensystem(2,MatrixSdtemp(3,1:2,1:2),2,MSdtemp(3,1:2),
     &                  ZDtemp(3,1:2,1:2),2,1)
     
      MSd(1:2)=sqrt(MSdtemp(1,1:2))
      MSd(3:4)=sqrt(MSdtemp(2,1:2))
      MSd(5:6)=sqrt(MSdtemp(3,1:2))
      
      
      do i=1,6,1
         do j=1,6,1
          ZD(i,j)=(0.d0,0.d0)
         end do
      end do
      
      ZD(1:2,1:2)=CONJG(ZDtemp(1,1:2,1:2))
      ZD(3:4,3:4)=CONJG(ZDtemp(2,1:2,1:2))
      ZD(5:6,5:6)=CONJG(ZDtemp(3,1:2,1:2))
     
      do i=1,3
      do j=1,2
            if(MSdtemp(i,j).le.0) then
              if(mainrun) then
              print*, '* Warning!! Eigenvalues of mass matrix for down',
     .                'type squarks are negative!!'
              print*, '  Program terminated...'
              print*
              call exit
              else 
              calcmasseserror=.TRUE.
              end if
            end if
      end do
      end do
      
      return
      end
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccc u-type squarks at tree-levelccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine usquarkmasses(g1,g2,v,beta,abslambda,vs,phil,phis,
     &                         phiu,phiAu,absAu,mq2,mur2,Yu,MSu,ZU)
      
      implicit none
      
      double precision g1,g2,v,beta,abslambda,vs,phil,
     &                 phis,phiu,phiAu(3),absAu(3),mq2(3),mur2(3),
     &                 Yu(3),MSu(6)
      
      double complex MatrixSutemp(3,2,2),ZUtemp(3,2,2),ZU(6,6)
      double precision MSutemp(3,2)
      
      integer i,j
      
      logical mainrun,calcmasseserror
      
      COMMON/calcerror/mainrun,calcmasseserror

          
      do i=1,3,1
         MatrixSutemp(i,1,1)=(24*mq2(i) + v**2*(-((g1**2 - 3*g2**2)*
     &                   Cos(2*beta)) + 12*Sin(beta)**2*Yu(i)**2))/24.
         MatrixSutemp(i,1,2)= -(v*(abslambda*vs*Cos(beta)*(Cos(phil  
     &                   + phis) - (0,1)*Sin(phil + phis)) -Sqrt(2d0)*
     &                   absAu(i)*Sin(beta)*(Cos(phiu + phiAu(i)) +
     &                    (0,1)*Sin(phiu + phiAu(i))))*Yu(i))/2.
         MatrixSutemp(i,2,1)= -(v*(abslambda*vs*Cos(beta)*(Cos(phil  
     &                   + phis) + (0,1)*Sin(phil + phis)) -Sqrt(2d0)*
     &                   absAu(i)*Sin(beta)*(Cos(phiu + phiAu(i)) -
     &                   (0,1)*Sin(phiu + phiAu(i))))*Yu(i))/2.
         MatrixSutemp(i,2,2)=(g1**2*v**2*Cos(2*beta))/6. + mur2(i) + 
     &                     (v**2*Sin(beta)**2*Yu(i)**2)/2.
      end do
     
      
          
      call HEigensystem(2,MatrixSutemp(1,1:2,1:2),2,MSutemp(1,1:2),
     &                  ZUtemp(1,1:2,1:2),2,1)
      call HEigensystem(2,MatrixSutemp(2,1:2,1:2),2,MSutemp(2,1:2),
     &                  ZUtemp(2,1:2,1:2),2,1)
      call HEigensystem(2,MatrixSutemp(3,1:2,1:2),2,MSutemp(3,1:2),
     &                  ZUtemp(3,1:2,1:2),2,1)
     
      MSu(1:2)=sqrt(MSutemp(1,1:2))
      MSu(3:4)=sqrt(MSutemp(2,1:2))
      MSu(5:6)=sqrt(MSutemp(3,1:2))
      
      
      do i=1,6,1
         do j=1,6,1
          ZU(i,j)=(0.d0,0.d0)
         end do
      end do
      
      ZU(1:2,1:2)=CONJG(ZUtemp(1,1:2,1:2))
      ZU(3:4,3:4)=CONJG(ZUtemp(2,1:2,1:2))
      ZU(5:6,5:6)=CONJG(ZUtemp(3,1:2,1:2))
      
      do i=1,3
      do j=1,2
            if(MSutemp(i,j).le.0) then
              if(mainrun) then
              print*, '* Warning!! Eigenvalues of mass matrix for up',
     .                'type squarks are negative!!'
              print*, '  Program terminated...'
              print*
              call exit
              else 
              calcmasseserror=.TRUE.
              end if
            end if
      end do
      end do

      return
      end
      
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccc sneutrinos at tree-levelccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine sneutrinomasses(g1,g2,v,beta,ml2,MSv)
      
      double precision g1,g2,v,beta,ml2(3),MSv(3),MSv2(3)
      
      integer i
      
      logical mainrun,calcmasseserror
      
      COMMON/calcerror/mainrun,calcmasseserror
      
      do i=1,3,1
        MSv2(i)=((g1**2 + g2**2)*v**2*Cos(2*beta) + 8*ml2(i))/8.
      end do
      
      MSv=Sqrt(MSv2)
      
      do i=1,3,1
            if(MSv2(i).le.0) then
              if(mainrun) then
              print*, '* Warning!! Eigenvalues of mass matrix for ',
     .                'sneutrinos are negative!!'
              print*, '  Program terminated...'
              print*
              call exit
              else 
              calcmasseserror=.TRUE.
              end if
            end if
      end do
      
      return
      end



ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccc Calculation of the Higgs Masses ccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
     
      subroutine HMass(Mh,ZH5,Mhlo,ZHlo,MHpm,MHpmlo)

      implicit none
      
      double precision Mh(5),dM(5,5),ZH5(5,5),Mhlo(5),ZHlo(5,5),
     &                 MHpm,MHpmlo,ZHloiter(5,5),MHpmlo2

      double precision ZHlotemp(5,5)
      double complex ZHtreelotemp(5,5)
      double complex M1l(5,5),p2,Mhlotemp(5),ZHtreelo(5,5)
      double complex Mlo(5,5),ESelf2loop(5,5)
      
      double precision dMW2,dMZ2,dZe,dthd,dthu,dths,dtad,dtas,dMHpm2,
     &                 dMchaH2 
     
      double precision selfHpm,k2,value(10)

      integer i,j,k,l,n,m
      
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      
      logical CPcon,MHpmInput,onelswitch,mQrun
      
      logical mainrun,calcmasseserror
      
      double precision dMHpm22l,selfcha2loop
      
      double precision configerror(120),lineerror,minerror
      integer config(120,5),minconfig

c MMM changed 21 Aug 2018
      double complex ZHfac(5,5)
c end MMM changed 21 Aug 2018      
      
      COMMON/calcerror/mainrun,calcmasseserror
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
      common /MHpm2l/ dMHpm22l,selfcha2loop
      common /ZHtreeto2l/ ZHtreelo
c MMM changed 21 Aug 2018
      common /Zfactor/ ZHfac
c end MMM changed 21 Aug 2018

c nhung added for atat 22/3/2019
      integer ilevel,igauge 
      common/ilevelATAT/ilevel,igauge
      double precision selfCHatat,dMHp2atat,ESelfATAT(5,5)
      common /MHpmatat/selfCHatat,dMHp2atat
      double complex MCH2C,selfHpmC
c end nhung added 22/3/2019
      
c nhung added for atlambda 6/2021
      double precision regfac
      common/ATlambda/regfac
c end nhung added 6/2021

c calculate the coupling fo the Higgs sector
      call couplings
      
           
c setting the ID's for the loop functions
      bb0=1
      bb1=2
      bb00=3
      dbb0=4
      dbb1=5
      dbb00=6

c determine counterterms via self energies and tadpoles
      call Counterterms(dMW2,dMZ2,dZe,dMHpm2,dthd,dthu,dths,dtad,dtas)

c calcualte counterterm mass matrix
      call counterM(dMW2,dMZ2,dZe,dMHpm2,dthd,dthu,dths,dtad,dtas,dM,
     &              dMchaH2)
     
c p2=0 approximation (to obtain a unitary rotation matrix)
      p2=0d0
      call MassMatrix1l(p2,Mh,dM,M1l,ZH5)
     
      if(onelswitch) then
      Mlo=M1l
      ESelf2loop=0d0
      else
      call SelfCollect2loop(ESelf2loop)
c nhung added for atlambda correction 6/2021
      if(ilevel.eq.3)then
      call ReHSelf_atat(ESelfATAT)
      elseif(ilevel.eq.4)then
      call ReHSelf_atlambda(ESelfATAT,regfac)
      else
      EselfATAT=0D0
      endif
      do i=1,5
       do j=1,5
      ESelf2loop(i,j)=ESelf2loop(i,j)+ESelfATAT(i,j)
       enddo
      enddo 
c end nhung added for atlambda correction 6/2021
      Mlo=M1l-ESelf2loop
      end if
      
      if(CPcon) then
            call SEigensystem(2,Mlo(1:2,1:2),2,Mhlotemp(1:2),
     .                        ZHtreelo(1:2,1:2),2,1)
            call SEigensystem(3,Mlo(3:5,3:5),3,Mhlotemp(3:5),
     .                        ZHtreelo(3:5,3:5),3,1)
             ZHtreelo(1:2,3:5)=0d0
             ZHtreelo(3:5,1:2)=0d0
      else
            call SEigensystem(5,Mlo,5,Mhlotemp,ZHtreelo,5,1)
      end if
     
      
      do i=1,5
      do j=1,5
            ZHlo(i,j)=0d0
      do k=1,5
            ZHlo(i,j) = ZHlo(i,j) + REALPART(ZHtreelo(i,k)*ZH5(k,j))
      end do
      end do
      end do

c calculate loop masses including all imaginary parts
      do i=1,5
c MMM changed 21 Aug 2018
c         call iter(Mh,dM,ESelf2loop,ZH5,i,9,CPcon,Mhlo(i),
c     &        ZHloiter(i,1:5),onelswitch)
         call iterk(Mh,dM,ESelf2loop,ZH5,i,9,CPcon,Mhlo(i),
     &        ZHloiter(i,1:5),ZHfac(i,1:5),onelswitch)
c end MMM changed 21 Aug 2018       
      end do
     
      if(MHpmInput) then
      
        MHpmlo=MHpm
      
      else
        

c nhung change for atat correction 22/3/2019
        MHpmlo2=MHpm**2
        k2=0d0
        value=0d0
        i=1
        do while (abs(k2-MHpmlo2).ge.1d-6 .AND. i.le.100)
        k2=MHpmlo2
        if(onelswitch) then
        MHpmlo2=MHpm**2-selfHpm(k2)+dMchaH2
        else
        MHpmlo2=MHpm**2-selfHpm(k2)-selfcha2loop+dMchaH2+dMHpm22l
     &                 -selfCHatat+dMHp2atat
        end if
c end nhung change here 22/3/2019   
        !stores the last few iteration results
        !needed if convergence doubtfull
        do j=1,9
        value(j)=value(j+1)
        end do
        value(10)=Sqrt(MHpmlo2)
        i=i+1
        end do

        
        if(MHpmlo2.ge.0d0) then
          MHpmlo=Sqrt(MHpmlo2)
        else
        if(mainrun) then
          print*,'* Warning! Negative mass squared for 1loop mass
     . of charged Higgs boson'
        print*, 'MHpmlo^2=',MHpmlo2
        print*, '  Program terminated...'
        call exit
        else 
        calcmasseserror=.TRUE.
        end if
        end if
        
        if(i.eq.101) then
        print*, '* Warning! Convergence issues in the calculation',
     .          ' of charged Higgs mass.'
        print*, 'Last ten values are:',value
        end if
       
      end if


c check if the higgs mass ordering in the p2=0 approximation,
c which is used for the calculation of the mixing matrix,
c is consistent with the mass ordering obtained via the 
c iterative procedure, if not the elements of the mixing matrix
c are reordered accordingly
      n=1
      do i=1,5
      do j=1,5
        if(j.ne.i) then
        do k=1,5
          if(k.ne.i .AND. k.ne.j) then
          do l=1,5
             if(l.ne.i .AND. l.ne.j .AND. l.ne.k) then
             do m=1,5
              if(m.ne.i .AND. m.ne.j .AND. m.ne.k .AND. m.ne.l) then
              config(n,1)=i
              config(n,2)=j
              config(n,3)=k
              config(n,4)=l
              config(n,5)=m
              
              configerror(n)=lineerror(ZHloiter(1,1:5),ZHlo(i,1:5))+
     &                       lineerror(ZHloiter(2,1:5),ZHlo(j,1:5))+
     &                       lineerror(ZHloiter(3,1:5),ZHlo(k,1:5))+
     &                       lineerror(ZHloiter(4,1:5),ZHlo(l,1:5))+
     &                       lineerror(ZHloiter(5,1:5),ZHlo(m,1:5))
              n=n+1
              end if
             end do
             end if
          end do
          end if
        end do
        end if
      end do
      end do
      
      minerror=configerror(1)
      minconfig=1
      do i=2,120
      if(configerror(i).le.minerror) then
      minconfig=i
      minerror=configerror(i)
      end if
      end do
      
      do i=1,5
      do j=1,5
      ZHlotemp(i,j)=ZHlo(config(minconfig,i),j)
      ZHtreelotemp(i,j)=ZHtreelo(config(minconfig,i),j)
      end do
      end do
      ZHlo=ZHlotemp
      ZHtreelo=ZHtreelotemp
      
      return
      end
      
      
      
      double precision function lineerror(ZHloiter,ZHp20)
      
      implicit none
      double precision ZHloiter(5),ZHp20(5)
      integer i
      
      lineerror=0d0
      
      do i=1,5
      lineerror=lineerror+Abs(Abs(ZHloiter(i))**2-Abs(ZHp20(i))**2)
      end do
      
      return
      end



ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccc One-Loop Mass Matrix cccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      
      subroutine MassMatrix1l(p2,Mh,dM,M1l,ZH5)
      
      implicit none

      
      double precision Mh(5),dM(5,5),ZH5(5,5)
      double complex M1l(5,5),p2
      double complex HSelfren
      
      integer i,j
      
      do i=1,5,1
      do j=1,5,1
        if(i.EQ.j) then
           M1l(i,j)=Mh(i)**2-HSelfren(i,j,p2,dM,ZH5)
        else
           M1l(i,j)=-HSelfren(i,j,p2,dM,ZH5)
        end if
      end do
      end do
      
      return
      end



ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccc Renormalized Self-Energy cccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      double complex function HSelfren(i,j,p2,dM,ZH5)
      
      implicit none

      double complex p2
      integer i,j,k,l
      
      double precision dM(5,5),ZH5(5,5)
      double complex HSelf,DHSelf
     
      HSelfren=0d0
      
      do k=1,5,1
      do l=1,5,1
           HSelfren= HSelfren - ZH5(i,k)*dM(k,l)*ZH5(j,l)
      end do
      end do
     
c expansion around the imaginary part of p2
      HSelfren = HSelfren + HSelf(i+1,j+1,REALPART(p2))  
     .         + DHSelf(i+1,j+1,REALPART(p2))*(0d0,1d0)*IMAGPART(p2)
      
      return
      end
      
      

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccc iteration procedure  cccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine iter(Mh,dM,ESelf2loop,ZH5,k,prec,CPcon,Mhlo,
     &                ZHiter,onelswitch)
      
      implicit none
      
      double precision Mhlo,ZHiter(5)
      
      double precision Mh(5),dM(5,5),ZH5(5,5)
      double complex Mlo(5,5),Mhlotemp(5),ZHtreelo(5,5),p2,
     &               M1l(5,5),ESelf2loop(5,5)
      double precision diff,accur,value(10)
      
      integer k,i,prec,j,m
      logical CPcon
      
      logical mainrun,calcmasseserror,onelswitch
      
      COMMON/calcerror/mainrun,calcmasseserror
      
      diff=100d0
      accur=10d0**(-prec)
      
      i=1
      Mhlotemp(k)=Mh(k)**2
      
      do while((diff.GE.accur).AND.(i.LE.100))
      p2=Mhlotemp(k)

         call MassMatrix1l(p2,Mh,dM,M1l,ZH5)
         
         if(onelswitch) then
         Mlo=M1l
         else
         Mlo=M1l-ESelf2loop
         end if
        
        if(CPcon) then
          if(k.LE.2) then
             call SEigensystem(2,Mlo(1:2,1:2),2,Mhlotemp(1:2),
     .                        ZHtreelo(1:2,1:2),2,1)
          else
             call SEigensystem(3,Mlo(3:5,3:5),3,Mhlotemp(3:5),
     .                        ZHtreelo(3:5,3:5),3,1)
          end if
        else
          call SEigensystem(5,Mlo,5,Mhlotemp,ZHtreelo,5,1)
        end if
        
        !stores the last few iteration results
        !needed if convergence doubtfull
        do j=1,9
        value(j)=value(j+1)
        end do
        value(10)=Sqrt(Realpart(Mhlotemp(k)))
        
        diff=Abs(Sqrt(Realpart(Mhlotemp(k)))-Sqrt(Realpart(p2)))
        i = i + 1

      end do
      
      if(i.eq.101) then
        print*, '* Warning! Convergence issues in the calculation',
     .          ' of mass of the',k,'th Higgs boson.'
        print*, 'Last ten values are:',value
        end if
      
      if(Realpart(Mhlotemp(k)).ge.0d0) then
        Mhlo=Sqrt(Realpart(Mhlotemp(k)))
      else
        if(mainrun) then
        print*,'* Warning! Negative eigenvalue for loop Higgs
     . mass matrix'
        print*, 'MHloop(i)^2=',Realpart(Mhlotemp(k)),',i=',k
        print*, '  Program terminated...'
        call exit
        else 
        calcmasseserror=.TRUE.
        end if
      end if

      ZHiter=0d0
      do m=1,5
      do j=1,5
            ZHiter(m) = ZHiter(m) + REALPART(ZHtreelo(k,j)*ZH5(j,m))
      end do
      end do
     
      return
      end



ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccc Higgs-Self-Energy cccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
     
      double complex function HSelf(i,j,p2)
      
      implicit none
      
      integer i,j
      double precision p2

      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
    
      
      double complex  ChHpWmA(6,2),ChHmWpA(2,6),ChHpWmZ(6,2),
     &                ChHmWpZ(2,6),ChhWmWp(6,6),ChhAZ(6,6),ChhZZ(6,6),
     &                ChHpWm(6,2),ChHmWp(2,6),ChhZ(6,6),ChWmWp(6),
     &                ChZZ(6),ChSdmSdp(6,6,6),ChSemSep(6,6,6),
     &                ChSupSum(6,6,6),ChSvSvconj(6),ChHmHp(2,2,6),
     &                ChhSemSep(6,6,6,6),ChhSvSvconj(6,6),
     &                ChhHmHp(2,6,2,6),ChhSupSum(6,6,6,6),
     &                ChhSdmSdp(6,6,6,6),CLhChapCham(2,2,6),
     &                CRhChapCham(2,2,6),CLhFepFem(3,3,6),
     &                CRhFepFem(3,3,6),CLhFdpFdm(3,3,6),
     &                CRhFdpFdm(3,3,6),CLhFumFup(3,3,6),
     &                CRhFumFup(3,3,6),ChU3U3(6),ChU2U2(6),ChU4U4(6),
     &                Chhh(6,6,6),Chhhh(6,6,6,6),CLhChiChi(5,5,6),
     &                CRhChiChi(5,5,6),CHmHpSvSv(3,2,3,2),
     &                CHmHpSdmSdp(6,2,6,2),CHmHpSupSum(6,2,6,2),
     &                CHmHpSemSep(6,2,6,2),CHmSepSv(3,6,2),
     &                CHpSemSv(6,3,2),CHmSupSdp(6,6,2),
     &                CHpSumSdm(6,6,2),CLHpChiCha(5,2,2),
     &                CRHpChiCha(5,2,2),CLHmChiCha(2,5,2),
     &                CRHmChiCha(2,5,2)
     
      double precision MassFd(3),MassFe(3),MassFu(3),MassHpm(2),
     &                 Masshh(6)
      
      double complex part1,part2,part3a,part3b,part4,part5a,part5b
      
      integer ind3,ind4
      
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      double complex B0i
      double precision A0
     
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
     
     
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /coupl/ ChHpWmA,ChHmWpA,ChHpWmZ,ChHmWpZ,ChhWmWp,
     &               ChhAZ,ChhZZ,ChHpWm,ChHmWp,ChhZ,ChWmWp,ChZZ,
     &               ChSdmSdp,ChSemSep,ChSupSum,ChSvSvconj,ChHmHp,
     &               ChhSemSep,ChhSvSvconj,ChhHmHp,ChhSupSum,ChhSdmSdp,
     &               CLhChapCham,CRhChapCham,CLhFepFem,CRhFepFem,
     &               CLhFdpFdm,CRhFdpFdm,CLhFumFup,CRhFumFup,ChU3U3,
     &               ChU2U2,ChU4U4,Chhh,Chhhh,CLhChiChi,CRhChiChi,
     &               CHmHpSvSv,CHmHpSdmSdp,CHmHpSupSum,CHmHpSemSep,
     &               CHmSepSv,CHpSemSv,CHmSupSdp,CHpSumSdm,CRHpChiCha,
     &               CLHpChiCha,CRHmChiCha,CLHmChiCha
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
      
!contributions from Vector bosons and ghosts
      part1=
     -     ((0d0,-4d0)*A0(MW**2)*ChhWmWp(i,j) - 
     -     (0d0,2d0)*A0(MZ**2)*ChhZZ(i,j) + 
     -     B0i(bb0,p2,MW**2,MW**2)*
     -     (ChU3U3(i)*ChU3U3(j) + ChU4U4(i)*ChU4U4(j) - 
     -     4d0*ChWmWp(i)*ChWmWp(j)) + 
     -     B0i(bb0,p2,MZ**2,MZ**2)*
     -     (ChU2U2(i)*ChU2U2(j) - 2d0*ChZZ(i)*ChZZ(j)))/
     -     (16d0*Pi**2)
     
!contributions from SM-Fermions and sneutrinos
       part2=0d0
       MassFd(1)=md
       MassFd(2)=ms
       MassFd(3)=mb
       MassFe(1)=me
       MassFe(2)=mmu
       MassFe(3)=mtau
       MassFu(1)=mu
       MassFu(2)=mc
       MassFu(3)=mt
       
       
       do ind3=1,3,1
       part2 = part2 +
     -    ((0,1)*A0(MSv(ind3)**2)*ChhSvSvconj(i,j) - 
     -    B0i(bb0,p2,MSv(ind3)**2,MSv(ind3)**2)*ChSvSvconj(i)*
     -    ChSvSvconj(j) + 
     -    6*A0(MassFd(ind3)**2)*
     -    (CLhFdpFdm(ind3,ind3,j)*CRhFdpFdm(ind3,ind3,i) + 
     -    CLhFdpFdm(ind3,ind3,i)*CRhFdpFdm(ind3,ind3,j)) + 
     -    6*p2*B0i(bb1,p2,MassFd(ind3)**2,MassFd(ind3)**2)*
     -    (CLhFdpFdm(ind3,ind3,j)*CRhFdpFdm(ind3,ind3,i) + 
     -    CLhFdpFdm(ind3,ind3,i)*CRhFdpFdm(ind3,ind3,j)) + 
     -    2*A0(MassFe(ind3)**2)*
     -    (CLhFepFem(ind3,ind3,j)*CRhFepFem(ind3,ind3,i) + 
     -    CLhFepFem(ind3,ind3,i)*CRhFepFem(ind3,ind3,j)) + 
     -    2*p2*B0i(bb1,p2,MassFe(ind3)**2,MassFe(ind3)**2)*
     -    (CLhFepFem(ind3,ind3,j)*CRhFepFem(ind3,ind3,i) + 
     -    CLhFepFem(ind3,ind3,i)*CRhFepFem(ind3,ind3,j)) + 
     -    6*A0(MassFu(ind3)**2)*
     -    (CLhFumFup(ind3,ind3,j)*CRhFumFup(ind3,ind3,i) + 
     -    CLhFumFup(ind3,ind3,i)*CRhFumFup(ind3,ind3,j)) + 
     -    6*p2*B0i(bb1,p2,MassFu(ind3)**2,MassFu(ind3)**2)*
     -    (CLhFumFup(ind3,ind3,j)*CRhFumFup(ind3,ind3,i) + 
     -    CLhFumFup(ind3,ind3,i)*CRhFumFup(ind3,ind3,j)) + 
     -    6*B0i(bb0,p2,MassFd(ind3)**2,MassFd(ind3)**2)*
     -    (CLhFdpFdm(ind3,ind3,i) + CRhFdpFdm(ind3,ind3,i))*
     -    (CLhFdpFdm(ind3,ind3,j) + CRhFdpFdm(ind3,ind3,j))*
     -    MassFd(ind3)**2 + 
     -    2*B0i(bb0,p2,MassFe(ind3)**2,MassFe(ind3)**2)*
     -    (CLhFepFem(ind3,ind3,i) + CRhFepFem(ind3,ind3,i))*
     -    (CLhFepFem(ind3,ind3,j) + CRhFepFem(ind3,ind3,j))*
     -    MassFe(ind3)**2 + 
     -    6*B0i(bb0,p2,MassFu(ind3)**2,MassFu(ind3)**2)*
     -    (CLhFumFup(ind3,ind3,i) + CRhFumFup(ind3,ind3,i))*
     -    (CLhFumFup(ind3,ind3,j) + CRhFumFup(ind3,ind3,j))*
     -    MassFu(ind3)**2)/(16.*Pi**2)
       end do
      
! charged Higgs and charged Goldstone contribution + Chargino contributions
      part3a=0d0
      part3b=0d0
      MassHpm(1)=Mw
      MassHpm(2)=Mhpm
      
      do ind3=1,2,1
      part3a = part3a + 
     -     ((0,1)*A0(MassHpm(ind3)**2)*ChhHmHp(ind3,i,ind3,j) + 
     -     (ChHmWp(ind3,j)*ChHpWm(i,ind3) + 
     -     ChHmWp(ind3,i)*ChHpWm(j,ind3))*
     -     (A0(MW**2) + 2*p2* B0i(bb1,p2,MW**2,MassHpm(ind3)**2) + 
     -     B0i(bb0,p2,MW**2,MassHpm(ind3)**2)*
     -     (3*p2 + MassHpm(ind3)**2)))/(16.*Pi**2)
     
      do ind4=1,2,1
      part3b = part3b +
     -     (-(B0i(bb0,p2,MassHpm(ind3)**2,MassHpm(ind4)**2)*
     -     ChHmHp(ind3,ind4,i)*ChHmHp(ind4,ind3,j)) + 
     -     2*(A0(MCha(ind4)**2)*(CLhChapCham(ind4,ind3,i)*
     -     CRhChapCham(ind3,ind4,j) + CLhChapCham(ind3,ind4,j)*
     -     CRhChapCham(ind4,ind3,i)) + 
     -     p2*B0i(bb1,p2,MCha(ind3)**2,MCha(ind4)**2)*
     -     (CLhChapCham(ind4,ind3,i)*CRhChapCham(ind3,ind4,j) + 
     -     CLhChapCham(ind3,ind4,j)*CRhChapCham(ind4,ind3,i)) + 
     -     B0i(bb0,p2,MCha(ind3)**2,MCha(ind4)**2)*MCha(ind3)*
     -     (CLhChapCham(ind4,ind3,i)*(CRhChapCham(ind3,ind4,j)* 
     -     MCha(ind3) + CLhChapCham(ind3,ind4,j)*MCha(ind4)) + 
     -     CRhChapCham(ind4,ind3,i)*(CLhChapCham(ind3,ind4,j)* 
     -     MCha(ind3) + CRhChapCham(ind3,ind4,j)*MCha(ind4)))))/
     -     (16.*Pi**2)
      end do
      end do
      
      
! Neutralino Contributions
      part4=0d0
      
      do ind3=1,5,1
      do ind4=1,5,1
      part4 = part4 + 
     -     ((A0(MChi(ind4)**2) + 
     -     p2*B0i(bb1,p2,MChi(ind3)**2,MChi(ind4)**2))*
     -     (CLhChiChi(ind4,ind3,i)*CRhChiChi(ind3,ind4,j) + 
     -     CLhChiChi(ind3,ind4,j)*CRhChiChi(ind4,ind3,i)) + 
     -     B0i(bb0,p2,MChi(ind3)**2,MChi(ind4)**2)*
     -     (CLhChiChi(ind4,ind3,i)*CRhChiChi(ind3,ind4,j) + 
     -     CLhChiChi(ind3,ind4,j)*CRhChiChi(ind4,ind3,i))*
     -     MChi(ind3)**2 + B0i(bb0,p2,MChi(ind3)**2,MChi(ind4)**2)*
     -     (CLhChiChi(ind3,ind4,j)*CLhChiChi(ind4,ind3,i) + 
     -     CRhChiChi(ind3,ind4,j)*CRhChiChi(ind4,ind3,i))*
     -     MChi(ind3)*MChi(ind4))/(16.*Pi**2)
      end do
      end do
      
      
! Contributions form Higgs and Sfermions
      part5a=0d0
      part5b=0d0
      
      Masshh(1)=Mz
      Masshh(2:6)=Mh(1:5)
      
      do ind3=1,6,1
      part5a = part5a +
     -     ((0,0.03125)*(A0(Masshh(ind3)**2)*Chhhh(i,j,ind3,ind3) + 
     -     6*A0(MSd(ind3)**2)*ChhSdmSdp(ind3,i,ind3,j) + 
     -     2*A0(MSe(ind3)**2)*ChhSemSep(ind3,i,ind3,j) + 
     -     6*A0(MSu(ind3)**2)*ChhSupSum(ind3,i,ind3,j) + 
     -     (0,2)*ChhZ(i,ind3)*ChhZ(j,ind3)*(A0(MZ**2) + 
     -     2*p2*B0i(bb1,p2,MZ**2,Masshh(ind3)**2) + 
     -     B0i(bb0,p2,MZ**2,Masshh(ind3)**2)*
     -     (3*p2 + Masshh(ind3)**2))))/Pi**2
     
      do ind4=1,6,1
      part5b = part5b -
     -     (B0i(bb0,p2,Masshh(ind3)**2,Masshh(ind4)**2)*
     -     Chhh(i,ind3,ind4)*Chhh(j,ind3,ind4) + 
     -     6*B0i(bb0,p2,MSd(ind3)**2,MSd(ind4)**2)*
     -     ChSdmSdp(ind3,ind4,i)*ChSdmSdp(ind4,ind3,j) + 
     -     2*B0i(bb0,p2,MSe(ind3)**2,MSe(ind4)**2)*
     -     ChSemSep(ind3,ind4,i)*ChSemSep(ind4,ind3,j) + 
     -     6*B0i(bb0,p2,MSu(ind3)**2,MSu(ind4)**2)*
     -     ChSupSum(ind3,ind4,i)*ChSupSum(ind4,ind3,j))/(32.*Pi**2)
      
      end do
      end do

      HSelf=part1+part2+part3a+part3b+part4+part5a+part5b
      
      return
      end
      
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccDerivative of Higgs-Self-Energy with respect to p2 cccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
     
      double complex function DHSelf(i,j,p2)
     
      
      integer i,j
      double precision p2

      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
    
      
      double complex  ChHpWmA(6,2),ChHmWpA(2,6),ChHpWmZ(6,2),
     &                ChHmWpZ(2,6),ChhWmWp(6,6),ChhAZ(6,6),ChhZZ(6,6),
     &                ChHpWm(6,2),ChHmWp(2,6),ChhZ(6,6),ChWmWp(6),
     &                ChZZ(6),ChSdmSdp(6,6,6),ChSemSep(6,6,6),
     &                ChSupSum(6,6,6),ChSvSvconj(6),ChHmHp(2,2,6),
     &                ChhSemSep(6,6,6,6),ChhSvSvconj(6,6),
     &                ChhHmHp(2,6,2,6),ChhSupSum(6,6,6,6),
     &                ChhSdmSdp(6,6,6,6),CLhChapCham(2,2,6),
     &                CRhChapCham(2,2,6),CLhFepFem(3,3,6),
     &                CRhFepFem(3,3,6),CLhFdpFdm(3,3,6),
     &                CRhFdpFdm(3,3,6),CLhFumFup(3,3,6),
     &                CRhFumFup(3,3,6),ChU3U3(6),ChU2U2(6),ChU4U4(6),
     &                Chhh(6,6,6),Chhhh(6,6,6,6),CLhChiChi(5,5,6),
     &                CRhChiChi(5,5,6),CHmHpSvSv(3,2,3,2),
     &                CHmHpSdmSdp(6,2,6,2),CHmHpSupSum(6,2,6,2),
     &                CHmHpSemSep(6,2,6,2),CHmSepSv(3,6,2),
     &                CHpSemSv(6,3,2),CHmSupSdp(6,6,2),
     &                CHpSumSdm(6,6,2),CLHpChiCha(5,2,2),
     &                CRHpChiCha(5,2,2),CLHmChiCha(2,5,2),
     &                CRHmChiCha(2,5,2)
     
      double precision MassFd(3),MassFe(3),MassFu(3),MassHpm(2),
     &                 Masshh(6)
      
      double complex part1,part2,part3a,part3b,part4,part5a,part5b
      
      integer ind3,ind4
      
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      double complex B0i
     
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
     
     
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /coupl/ ChHpWmA,ChHmWpA,ChHpWmZ,ChHmWpZ,ChhWmWp,
     &               ChhAZ,ChhZZ,ChHpWm,ChHmWp,ChhZ,ChWmWp,ChZZ,
     &               ChSdmSdp,ChSemSep,ChSupSum,ChSvSvconj,ChHmHp,
     &               ChhSemSep,ChhSvSvconj,ChhHmHp,ChhSupSum,ChhSdmSdp,
     &               CLhChapCham,CRhChapCham,CLhFepFem,CRhFepFem,
     &               CLhFdpFdm,CRhFdpFdm,CLhFumFup,CRhFumFup,ChU3U3,
     &               ChU2U2,ChU4U4,Chhh,Chhhh,CLhChiChi,CRhChiChi,
     &               CHmHpSvSv,CHmHpSdmSdp,CHmHpSupSum,CHmHpSemSep,
     &               CHmSepSv,CHpSemSv,CHmSupSdp,CHpSumSdm,CRHpChiCha,
     &               CLHpChiCha,CRHmChiCha,CLHmChiCha
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
      
!contributions from Vector bosons and ghosts
      part1= (B0i(dbb0,p2,MW**2,MW**2)*
     -       (ChU3U3(i)*ChU3U3(j) + ChU4U4(i)*ChU4U4(j) - 4*ChWmWp(i)
     -       *ChWmWp(j)) + B0i(dbb0,p2,MZ**2,MZ**2)*(ChU2U2(i)*ChU2U2(j)
     -       - 2*ChZZ(i)*ChZZ(j)))/(16.*Pi**2)
     
     
!contributions from SM-Fermions and sneutrinos
       part2=0d0
       MassFd(1)=md
       MassFd(2)=ms
       MassFd(3)=mb
       MassFe(1)=me
       MassFe(2)=mmu
       MassFe(3)=mtau
       MassFu(1)=mu
       MassFu(2)=mc
       MassFu(3)=mt
       
       
       do ind3=1,3,1
       part2 = part2 +
     -        (-(B0i(dbb0,p2,MSv(ind3)**2,MSv(ind3)**2)*
     -        ChSvSvconj(i)*ChSvSvconj(j)) + 
     -        6*B0i(bb1,p2,MassFd(ind3)**2,MassFd(ind3)**2)*
     -        (CLhFdpFdm(ind3,ind3,j)*CRhFdpFdm(ind3,ind3,i) + 
     -        CLhFdpFdm(ind3,ind3,i)*CRhFdpFdm(ind3,ind3,j)) + 
     -        6*p2*B0i(dbb1,p2,MassFd(ind3)**2,MassFd(ind3)**2)*
     -        (CLhFdpFdm(ind3,ind3,j)*CRhFdpFdm(ind3,ind3,i) + 
     -        CLhFdpFdm(ind3,ind3,i)*CRhFdpFdm(ind3,ind3,j)) + 
     -        2*B0i(bb1,p2,MassFe(ind3)**2,MassFe(ind3)**2)*
     -        (CLhFepFem(ind3,ind3,j)*CRhFepFem(ind3,ind3,i) + 
     -        CLhFepFem(ind3,ind3,i)*CRhFepFem(ind3,ind3,j)) + 
     -        2*p2*B0i(dbb1,p2,MassFe(ind3)**2,MassFe(ind3)**2)*
     -        (CLhFepFem(ind3,ind3,j)*CRhFepFem(ind3,ind3,i) + 
     -        CLhFepFem(ind3,ind3,i)*CRhFepFem(ind3,ind3,j)) + 
     -        6*B0i(bb1,p2,MassFu(ind3)**2,MassFu(ind3)**2)*
     -        (CLhFumFup(ind3,ind3,j)*CRhFumFup(ind3,ind3,i) + 
     -        CLhFumFup(ind3,ind3,i)*CRhFumFup(ind3,ind3,j)) + 
     -        6*p2*B0i(dbb1,p2,MassFu(ind3)**2,MassFu(ind3)**2)*
     -        (CLhFumFup(ind3,ind3,j)*CRhFumFup(ind3,ind3,i) + 
     -        CLhFumFup(ind3,ind3,i)*CRhFumFup(ind3,ind3,j)) + 
     -        6*B0i(dbb0,p2,MassFd(ind3)**2,MassFd(ind3)**2)*
     -        (CLhFdpFdm(ind3,ind3,i) + CRhFdpFdm(ind3,ind3,i))*
     -        (CLhFdpFdm(ind3,ind3,j) + CRhFdpFdm(ind3,ind3,j))*
     -        MassFd(ind3)**2 + 
     -        2*B0i(dbb0,p2,MassFe(ind3)**2,MassFe(ind3)**2)*
     -        (CLhFepFem(ind3,ind3,i) + CRhFepFem(ind3,ind3,i))*
     -        (CLhFepFem(ind3,ind3,j) + CRhFepFem(ind3,ind3,j))*
     -        MassFe(ind3)**2 + 
     -        6*B0i(dbb0,p2,MassFu(ind3)**2,MassFu(ind3)**2)*
     -        (CLhFumFup(ind3,ind3,i) + CRhFumFup(ind3,ind3,i))*
     -        (CLhFumFup(ind3,ind3,j) + CRhFumFup(ind3,ind3,j))*
     -        MassFu(ind3)**2)/(16.*Pi**2)
     
       end do
      
! charged Higgs and charged Goldstone contribution + Chargino contributions
      part3a=0d0
      part3b=0d0
      MassHpm(1)=Mw
      MassHpm(2)=Mhpm
      
      do ind3=1,2,1
      part3a = part3a +
     -        ((ChHmWp(ind3,j)*ChHpWm(i,ind3) + ChHmWp(ind3,i)*
     -        ChHpWm(j,ind3))*(3*B0i(bb0,p2,MW**2,MassHpm(ind3)**2) + 
     -        2*B0i(bb1,p2,MW**2,MassHpm(ind3)**2) + 
     -        3*p2*B0i(dbb0,p2,MW**2,MassHpm(ind3)**2) + 
     -        2*p2*B0i(dbb1,p2,MW**2,MassHpm(ind3)**2) + 
     -        B0i(dbb0,p2,MW**2,MassHpm(ind3)**2)*MassHpm(ind3)**2)
     -        )/(16.*Pi**2)
    
     
      do ind4=1,2,1
      part3b = part3b + 
     -        (-(B0i(dbb0,p2,MassHpm(ind3)**2,MassHpm(ind4)**2)*
     -        ChHmHp(ind3,ind4,i)*ChHmHp(ind4,ind3,j)) + 
     -        2*(B0i(bb1,p2,MCha(ind3)**2,MCha(ind4)**2)*
     -        (CLhChapCham(ind4,ind3,i)*CRhChapCham(ind3,ind4,j) + 
     -        CLhChapCham(ind3,ind4,j)*CRhChapCham(ind4,ind3,i)) + 
     -        p2*B0i(dbb1,p2,MCha(ind3)**2,MCha(ind4)**2)*
     -        (CLhChapCham(ind4,ind3,i)*CRhChapCham(ind3,ind4,j) + 
     -        CLhChapCham(ind3,ind4,j)*CRhChapCham(ind4,ind3,i)) + 
     -        B0i(dbb0,p2,MCha(ind3)**2,MCha(ind4)**2)*
     -        (CRhChapCham(ind3,ind4,j)*(CRhChapCham(ind4,ind3,i)
     -        *MCha(ind3)*MCha(ind4) + CLhChapCham(ind4,ind3,i)*
     -        MCha(ind3)**2) + CLhChapCham(ind3,ind4,j)*
     -        (CLhChapCham(ind4,ind3,i)*MCha(ind3)*MCha(ind4) + 
     -        CRhChapCham(ind4,ind3,i)*MCha(ind3)**2))))/(16.*Pi**2)
     
      end do
      end do
      
      
! Neutralino Contributions
      part4=0d0
      
      do ind3=1,5,1
      do ind4=1,5,1
      part4 = part4 + 
     -        ((B0i(bb1,p2,MChi(ind3)**2,MChi(ind4)**2) + 
     -        p2*B0i(dbb1,p2,MChi(ind3)**2,MChi(ind4)**2))*
     -        (CLhChiChi(ind4,ind3,i)*CRhChiChi(ind3,ind4,j) + 
     -        CLhChiChi(ind3,ind4,j)*CRhChiChi(ind4,ind3,i)) + 
     -        B0i(dbb0,p2,MChi(ind3)**2,MChi(ind4)**2)*
     -        (CLhChiChi(ind4,ind3,i)*CRhChiChi(ind3,ind4,j) + 
     -        CLhChiChi(ind3,ind4,j)*CRhChiChi(ind4,ind3,i))*
     -        MChi(ind3)**2 + B0i(dbb0,p2,MChi(ind3)**2,MChi(ind4)**2)*
     -        (CLhChiChi(ind3,ind4,j)*CLhChiChi(ind4,ind3,i) + 
     -        CRhChiChi(ind3,ind4,j)*CRhChiChi(ind4,ind3,i))*
     -        MChi(ind3)*MChi(ind4))/(16.*Pi**2)
      end do
      end do
      
      
! Contributions form Higgs and Sfermions
      part5a=0d0
      part5b=0d0
      
      Masshh(1)=Mz
      Masshh(2:6)=Mh(1:5)
      
      do ind3=1,6,1
      part5a = part5a -
     -        (ChhZ(i,ind3)*ChhZ(j,ind3)*
     -        (3*B0i(bb0,p2,MZ**2,Masshh(ind3)**2) + 
     -        2*B0i(bb1,p2,MZ**2,Masshh(ind3)**2) + 
     -        2*p2*B0i(dbb1,p2,MZ**2,Masshh(ind3)**2) + 
     -        B0i(dbb0,p2,MZ**2,Masshh(ind3)**2)*
     -        (3*p2 + Masshh(ind3)**2)))/(16.*Pi**2)
        
      do ind4=1,6,1
      part5b = part5b -
     -        (B0i(dbb0,p2,Masshh(ind3)**2,Masshh(ind4)**2)*
     -        Chhh(i,ind3,ind4)*Chhh(j,ind3,ind4) + 
     -        6*B0i(dbb0,p2,MSd(ind3)**2,MSd(ind4)**2)*
     -        ChSdmSdp(ind3,ind4,i)*ChSdmSdp(ind4,ind3,j) + 
     -        2*B0i(dbb0,p2,MSe(ind3)**2,MSe(ind4)**2)*
     -        ChSemSep(ind3,ind4,i)*ChSemSep(ind4,ind3,j) + 
     -        6*B0i(dbb0,p2,MSu(ind3)**2,MSu(ind4)**2)*
     -        ChSupSum(ind3,ind4,i)*ChSupSum(ind4,ind3,j))/
     -        (32.*Pi**2)
     
     
      
      end do
      end do
      
      DHSelf=part1+part2+part3a+part3b+part4+part5a+part5b
      
      return
      end
      

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccc Counterterm MassMatrix ccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
     
      subroutine counterM(dMW2,dMZ2,dZe,dMHpm2,dthd,dthu,dths,dtad,
     &                    dtas,dM,dMchaH2)

      implicit none

      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
     
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
     
      double precision phi,phiIl,phiIk
   
      double precision dMW2,dMZ2,dZe,dMHpm2,dthu,dthd,dths,dtad,dtas,
     &                 dMchaH2 
      
      double precision dM(5,5),dg1,dg2,dv,dTlambda,dphiTl,dImTlambda

c MMM changed 10/8/2013      
      double precision dImTkappa,absTkappa,absTlambda,dReTkappa,
     &     dReTlambda
c end MMM changed 10/8/2013
      
      double precision ImTkappaDR,ImTlambdaDR,ReTkappaDR,ReTlambdaDR,
     &                 ImAkappaDR,ImAlambdaDR,ReAkappaDR,ReAlambdaDR 
      
      logical MHpmInput,CPcon,mQrun,onelswitch
     
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
     
c MMM changed 22 Aug 2018      
      double precision phiI,phi3ks,sqrt2,ImAlambda,ReAlambda,dImAkappa,
     &     dImAlambda,dReAlambda
c end MMM changed 22 Aug 2018          
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
      common /DRpara/ ReAlambdaDR,ReAkappaDR,ImAlambdaDR,ImAkappaDR
     
      
      phi=-phik + phil - 2*phis + phiu
      phiIl=phis + phiAl + phil + phiu
      phiIk=3*phis + phiAk + phik
c MMM changed 22 Aug 2018 - Nhung added this
      phiI=phil + phis + phiu
      phi3ks=phik + 3*phis
      sqrt2=Sqrt(2d0)
      ReAlambda = absAlambda* cos(phiAl)
      ImAlambda = absAlambda* sin(phiAl)
c end MMM changed 22 Aug 2018 - end Nhung add      
      
      absTlambda=abslambda*absAlambda
      absTkappa=abskappa*absAkappa
      
      dg1 = (e**2*(dMZ2*MW**2 - (dMW2 - 2*dZe*MW**2)*MZ**2))
     &      /(2.*g1*MW**4)
      dg2 = (e**2*(-(dMZ2*MW**2) + MZ**2*(dMW2 + 2*dZe*
     &       (-MW**2 + MZ**2))))/(2.*g2*(MW**2 - MZ**2)**2)
      dv = (2*(dMZ2*MW**4 + MZ**2*(2*dZe*MW**2*(MW**2 - MZ**2) 
     &       + dMW2*(-2*MW**2 + MZ**2))))/(e**2*MZ**4*v)
      
      if(MHpmInput) then
      dTlambda=(2*absAlambda*1/Sin(beta)*(4*Sqrt(2d0)*absAlambda*
     -         abslambda*dthu*vS*Cos(beta)**3*Sin(beta) - 
     -         2*Sqrt(2d0)*absAlambda*abslambda*v*(2*dMHpm2 - v*
     -         (-2*abslambda**2*dv + dv*g2**2 + dg2*g2*v))*vS*
     -         Cos(beta)*Sin(beta)**2+4*Sqrt(2d0)*absAlambda*abslambda
     -         *dthd*vS*Sin(beta)**4 + dtad/Cos(phiIl)*
     -         (4*abskappa*abslambda*vS**2*Cos(phi) + 
     -         (-4*MHpm**2 + (-2*abslambda**2 + g2**2)*v**2)*
     -         Sin(2*beta))*Tan(phiIl)))/(v*vS**2*(-8*absAlambda**2*
     -         abslambda*Cos(phiIl) + abskappa/Cos(phiIl)*
     -         (-4*abskappa*abslambda*vS**2*Cos(phi) + 
     -         (4*MHpm**2 + (2*abslambda**2 - g2**2)*v**2)*
     -         Sin(2*beta))*Sin(phi)*Tan(phiIl)))
      dphiTl= (2*(-4*Sqrt(2d0)*absAlambda*dtad/Sin(beta) + 
     -        abskappa*vS/Cos(phiIl)*(4*dthu*Cos(beta)**3 + 
     -        4*dthd*Sin(beta)**3 + v*(-2*dMHpm2 +v*(-2*abslambda**2*dv
     -        + dv*g2**2 + dg2*g2*v))*Sin(2*beta))*Sin(phi)))/
     -        (v*vS*(-8*absAlambda**2*abslambda*Cos(phiIl) + 
     -        abskappa/Cos(phiIl)*(-4*abskappa*abslambda*vS**2*Cos(phi)
     -         + (4*MHpm**2 + (2*abslambda**2 - g2**2)*v**2)*
     -        Sin(2*beta))*Sin(phi)*Tan(phiIl)))
     
      dReTlambda=dTlambda*Cos(phiAl+phil)
     -           - dphiTl*absTlambda*Sin(phiAl+phil)
      dImTlambda=dTlambda*Sin(phiAl+phil)
     -           + dphiTl*absTlambda*Cos(phiAl+phil)
      
      dReTkappa=0d0
      dImTkappa=(1/Cos(3*phis)*(-2*Sqrt(2d0)*dtas*vs + 
     -          Sqrt(2d0)*v*Cos(beta)*(2*dtad - 
     -          5*abskappa*abslambda*dv*vs**2*Sin(beta)*Sin(phi)) + 
     -          absTlambda*dv*v*vs*Sin(2*beta)*Sin(phiIl)) - 
     -          2*dReTkappa*vs**3*Tan(3*phis))/(2.*vs**3)

c MMM changed 22 Aug 2018 - Nhung changed this
      dReAlambda=-((dthu*sqrt2*Cos(beta)**3*Cos(phiI))/(abslambda*v*vS)) + 
     -  (dMHpm2*sqrt2*Cos(beta)*Cos(phiI)*Sin(beta))/(abslambda*vS) - 
     -  (dMW2*sqrt2*Cos(beta)*Cos(phiI)*Sin(beta))/(abslambda*vS) + 
     -  (abslambda*dv*sqrt2*v*Cos(beta)*Cos(phiI)*Sin(beta))/vS - 
     -  (dthd*sqrt2*Cos(phiI)*Sin(beta)**3)/(abslambda*v*vS) + 
     -  (dtad*sqrt2*Sin(phiI))/(abslambda*v*vS*sin(beta))

      dImAlambda=(dtad*sqrt2)/(abslambda*v*vS*cos(phiI)*sin(beta)) 
     -  - dReAlambda*Tan(phiI)
      dReTlambda=(dReAlambda*Cos(phil)-dImAlambda*Sin(phil))*abslambda
      dImTlambda=(dReAlambda*Sin(phil)+dImAlambda*Cos(phil))*abslambda
      dImAkappa=-((dtas*sqrt2)/(abskappa*vS**2*cos(phi3ks))) + 
     -  (abslambda*dImAlambda*v**2*Cos(beta)*Cos(phiI)*Sin(beta))/
     -  (abskappa*vS**2*cos(phi3ks)) + 
     -  (abslambda*dReAlambda*v**2*Cos(beta)*Sin(beta)*Sin(phiI))/
     -  (abskappa*vS**2*cos(phi3ks)) + (dv*
     -  (2*abslambda*ImAlambda*sqrt2*v*Cos(beta)*Cos(phiI)*Sin(beta)+ 
     -  4*abskappa*abslambda*v*vS*Cos(beta)*Sin(beta)*Sin(-phi) + 
     -  2*abslambda*ReAlambda*sqrt2*v*Cos(beta)*Sin(beta)*Sin(phiI)))/
     -  (abskappa*Sqrt2*vS**2*cos(phi3ks))
      dReTkappa= -dImAkappa*abskappa* Sin(phik)
      dImTkappa=  dImAkappa*abskappa* Cos(phik)
c end MMM changed 22 Aug 2018 - end Nhung change
      
      dMchaH2=dMHpm2
     
      else

      dImTlambda=(1/Cos(phis + phiu)*(2*dtad/Sin(beta) - 
     -           dv*vs*(abskappa*abslambda*vs*Sin(phi) + 
     -           Sqrt(2d0)*absTlambda*Sin(phiIl))))/
     -           (Sqrt(2d0)*v*vs)
      dImTkappa=(1/Cos(3*phis)*(-2*Sqrt(2d0)*dtas*vs + 
     -         Sqrt(2d0)*v*Cos(beta)*(2*dtad - 
     -         5*abskappa*abslambda*dv*vs**2*Sin(beta)*Sin(phi)) + 
     -         absTlambda*dv*v*vs*Sin(2*beta)*Sin(phiIl)))/(2.*vs**3)
      dReTkappa=0d0
      dReTlambda=0d0

c MMM changed 22 Aug 2018 - Nhung changed this
      dImAlambda =(dtad*sqrt(2d0))/(abslambda*v*vS*cos(phiI)*sin(beta)) + 
     -  dv*(-((-(abskappa*abslambda*vS**2*Sin(beta)*Sin(-phi)) + 
     -  absAlambda*abslambda*sqrt(2d0)*vS*Cos(phiAl)*Sin(beta)*Sin(phiI))/
     -  (abslambda*sqrt(2d0)*v*vS*cos(phiI)*sin(beta))) + 
     -  (-(abskappa*abslambda*v*vS**2*Sin(beta)*Sin(-phi)) + 
     -  absAlambda*abslambda*sqrt(2d0)*v*vS*Cos(phiAl)*Sin(beta)*Sin(phiI))/
     -  (abslambda*sqrt(2d0)*v**2*vS*cos(phiI)*sin(beta)))
      dReAlambda=0D0
      dReTlambda= -dImAlambda*abslambda* Sin(phil)
      dImTlambda=  dImAlambda*abslambda* Cos(phil)
      dImAkappa=-((dtas*sqrt2)/(abskappa*vS**2*cos(phi3ks))) + 
     -  (abslambda*dImAlambda*v**2*Cos(beta)*Cos(phiI)*Sin(beta))/
     -   (abskappa*vS**2*cos(phi3ks)) + 
     -  (abslambda*dReAlambda*v**2*Cos(beta)*Sin(beta)*Sin(phiI))/
     -   (abskappa*vS**2*cos(phi3ks)) + 
     -  (dv*(2*abslambda*ImAlambda*sqrt2*v*Cos(beta)*Cos(phiI)*Sin(beta) + 
     -       4*abskappa*abslambda*v*vS*Cos(beta)*Sin(beta)*Sin(-phi) + 
     -       2*abslambda*ReAlambda*sqrt2*v*Cos(beta)*Sin(beta)*Sin(phiI)))/
     -   (abskappa*Sqrt2*vS**2*cos(phi3ks))
      dReTkappa= -dImAkappa*abskappa* Sin(phik)
      dImTkappa=  dImAkappa*abskappa* Cos(phik)
c end MMM changed 22 Aug 2018 - end Nhung change
      
      dMchaH2=(1/Sin(2*beta)*(3*dthu*Cos(beta) + dthu*Cos(3*beta) + 
     -        3*dthd*Sin(beta) - 2*abslambda**2*dv*v**2*Sin(2*beta) + 
     -        dv*g2**2*v**2*Sin(2*beta) + dg2*g2*v**3*Sin(2*beta) - 
     -        dthd*Sin(3*beta) - 2*Sqrt(2d0)*dImTlambda*v*vS*
     -        Sin(Phis + Phiu)))/(2.*v)
    
      end if
      
      
c MMM changed 10/8/2013      
      dM(1,1)=(1/Sin(2*beta)*(2*dthu*Cos(beta)**3 + 2*dthd*Sin(beta)**3
     .     + Sqrt(2d0)*v*vS*(dReTlambda*Cos(phis + phiu) - 
     .     dImTlambda*Sin(phis + phiu))))/v
      dM(1,2)=-(abskappa*abslambda*dv*vS*Cos(phi)) + 
     .        (absTlambda*dv*Cos(phiIl) + dReTlambda*v*Cos(phis + phiu)
     .     - dImTlambda*v*Sin(phis + phiu))/Sqrt(2d0)
c end MMM changed 10/8/2013
      dM(1,3)=(vS*Cos(beta)*(dImTlambda*Cos(phis + phiu) + 
     -        dReTlambda*Sin(phis + phiu)))/Sqrt(2d0)
      dM(1,4)=(vS*Sin(beta)*(dImTlambda*Cos(phis + phiu) + 
     -        dReTlambda*Sin(phis + phiu)))/Sqrt(2d0)
c MMM changed 10/8/2013
      dM(1,5)=abskappa*abslambda*dv*vS*Sin(phi) + 
     .        (dImTlambda*v*Cos(phis + phiu) + absTlambda*dv*Sin(phiIl)
     .     + dReTlambda*v*Sin(phis + phiu))/Sqrt(2d0)
c end MMM changed 10/8/2013
      dM(2,1)=dM(1,2)
c MMM changed 10/8/2013
      dM(2,2)=(4*dths + 6*Sqrt(2d0)*vS**2*(-(dReTkappa*Cos(3*phis)) + 
     -        dImTkappa*Sin(3*phis)) + v*Sin(2*beta)*
     -        (8*abskappa*abslambda*dv*vS*Cos(phi) + Sqrt(2d0)*
     -        (2*absTlambda*dv*Cos(phiIl) + dReTlambda*v*
     .        Cos(phis + phiu) - 
     -        dImTlambda*v*Sin(phis + phiu))))/(4.*vS)
c end MMM changed 10/8/2013
      dM(2,3)=-(abskappa*abslambda*dv*vS*Sin(beta)*Sin(phi)) + 
     -        (Sin(beta)*(dImTlambda*v*Cos(phis + phiu)
     -         + absTlambda*dv*Sin(phiIl) + 
     -         dReTlambda*v*Sin(phis + phiu)))/Sqrt(2d0)
      dM(2,4)=-(abskappa*abslambda*dv*vS*Cos(beta)*Sin(phi)) + 
     -        (Cos(beta)*(dImTlambda*v*Cos(phis + phiu) + 
     -        absTlambda*dv*Sin(phiIl) + 
     -        dReTlambda*v*Sin(phis + phiu)))/Sqrt(2d0)
c MMM changed 10/8/2013
      dM(2,5)=-(abskappa*abslambda*dv*v*Sin(2*beta)*Sin(phi)) - 
     -        Sqrt(2d0)*vS*(dImTkappa*Cos(3*phis)  
     .        + dReTkappa*Sin(3*phis))
c end MMM changed 10/8/2013
      dM(3,1)=dM(1,3)
      dM(3,2)=dM(2,3)
      dM(3,3)=(v**2*(dv*(g1**2 + g2**2) + (dg1*g1 + dg2*g2)*v)*
     -        Cos(beta)**2 + 2*dthd/Cos(beta) + Sqrt(2d0)*v*vS*
     -        (dReTlambda*Cos(phis + phiu) - 
     -        dImTlambda*Sin(phis + phiu))*Tan(beta))/(2.*v)
      dM(3,4)=-(v*(-4*abslambda**2*dv + dv*(g1**2 + g2**2) + 
     -         (dg1*g1 + dg2*g2)*v)*Sin(2*beta))/4. +
     -         (vS*(-(dReTlambda*Cos(phis + phiu)) + 
     -          dImTlambda*Sin(phis + phiu)))/Sqrt(2d0)
      dM(3,5)=abslambda**2*dv*vS*Cos(beta) - 
     -       (Sin(beta)*(2*abskappa*abslambda*dv*vS*Cos(phi) + 
     -       Sqrt(2d0)*(absTlambda*dv*Cos(phiIl) + 
     -       dReTlambda*v*Cos(phis + phiu) - 
     -       dImTlambda*v*Sin(phis + phiu))))/2.
      dM(4,1)=dM(1,4)
      dM(4,2)=dM(2,4)
      dM(4,3)=dM(3,4)
c MMM changed 10/8/2013
      dM(4,4)=(2*dthu*1/Sin(beta) + v*(v*(dv*(g1**2 + g2**2) + 
     -        (dg1*g1 + dg2*g2)*v)*Sin(beta)**2 
     .        + Sqrt(2d0)*vS/Tan(beta)*
     -        (dReTlambda*Cos(phis + phiu) - 
     -         dImTlambda*Sin(phis + phiu))))/(2.*v)
      dM(4,5)=-(abskappa*abslambda*dv*vS*Cos(beta)*Cos(phi)) + 
     -        abslambda**2*dv*vS*Sin(beta) - (Cos(beta)*
     -       (absTlambda*dv*Cos(phiIl) + dReTlambda*v*Cos(phis + phiu) 
     .       - dImTlambda*v*Sin(phis + phiu)))/Sqrt(2d0)
c end MMM changed 10/8/2013
      dM(5,1)=dM(1,5)
      dM(5,2)=dM(2,5)
      dM(5,3)=dM(3,5)
      dM(5,4)=dM(4,5)
      dM(5,5)=(4*dths + Sqrt(2d0)*(2*vS**2*(dReTkappa*Cos(3*phis) - 
     -        dImTkappa*Sin(3*phis)) + 
     -        v*Sin(2*beta)*(2*absTlambda*dv*Cos(phiIl) + 
     -        dReTlambda*v*Cos(phis + phiu) - 
     -        dImTlambda*v*Sin(phis + phiu))))/(4.*vS)
      
      ReTkappaDR=absTkappa*Cos(phiAk+phik)+dReTkappa
      ImTkappaDR=absTkappa*Sin(phiAk+phik)+dImTkappa
      ReTlambdaDR=absTlambda*Cos(phiAl+phil)+dReTlambda
      ImTlambdaDR=absTlambda*Sin(phiAl+phil)+dImTlambda
      
      ReAkappaDR=(ImTkappaDR*Sin(phik)+ReTkappaDR*cos(phik))/abskappa
      ImAkappaDR=(ImTkappaDR*Cos(phik)-ReTkappaDR*Sin(phik))/abskappa
c MMM changed 10/8/2013
      ReAlambdaDR=(ImTlambdaDR*Sin(phil)
     .     +ReTlambdaDR*cos(phil))/abslambda
      ImAlambdaDR=(ImTlambdaDR*Cos(phil)
     .     -ReTlambdaDR*Sin(phil))/abslambda
c end MMM changed 10/8/2013      

      return
      end
      
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccc Counterterms  cccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
     
      subroutine Counterterms(dMW2,dMZ2,dZe,dMHpm2,
     .                        dthd,dthu,dths,dtad,dtas)

      implicit none

      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3,SW,CW,ThetaW
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
     
      double precision ZH5(5,5),ZH(6,6)
      
      double precision Masshh(6),MassFu(3),MassFe(3),MassFd(3),
     &                 MassHpm(2)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double precision dthd,dthu,dths,dtad,dtas,dtG,dtaud
      double complex tadpole(6)
      
      double precision ReB0i
      
      double precision dMW2,dMZ2,dZe,dMHpm2
      double complex parta,partb,partc,partd,partcha,partchi,part1sq,
     .                 part2sq,parthiggs1,parthiggs2,partrest,addrun,
     .                 partF,partsq,EAZ,DEAA,part3sq,part4sq
      double complex part1,part2,part3,part4,part5
     
      double complex  ChHpWmA(6,2),ChHmWpA(2,6),ChHpWmZ(6,2),
     &                ChHmWpZ(2,6),ChhWmWp(6,6),ChhAZ(6,6),ChhZZ(6,6),
     &                ChHpWm(6,2),ChHmWp(2,6),ChhZ(6,6),ChWmWp(6),
     &                ChZZ(6),ChSdmSdp(6,6,6),ChSemSep(6,6,6),
     &                ChSupSum(6,6,6),ChSvSvconj(6),ChHmHp(2,2,6),
     &                ChhSemSep(6,6,6,6),ChhSvSvconj(6,6),
     &                ChhHmHp(2,6,2,6),ChhSupSum(6,6,6,6),
     &                ChhSdmSdp(6,6,6,6),CLhChapCham(2,2,6),
     &                CRhChapCham(2,2,6),CLhFepFem(3,3,6),
     &                CRhFepFem(3,3,6),CLhFdpFdm(3,3,6),
     &                CRhFdpFdm(3,3,6),CLhFumFup(3,3,6),
     &                CRhFumFup(3,3,6),ChU3U3(6),ChU2U2(6),ChU4U4(6),
     &                Chhh(6,6,6),Chhhh(6,6,6,6),CLhChiChi(5,5,6),
     &                CRhChiChi(5,5,6),CHmHpSvSv(3,2,3,2),
     &                CHmHpSdmSdp(6,2,6,2),CHmHpSupSum(6,2,6,2),
     &                CHmHpSemSep(6,2,6,2),CHmSepSv(3,6,2),
     &                CHpSemSv(6,3,2),CHmSupSdp(6,6,2),
     &                CHpSumSdm(6,6,2),CLHpChiCha(5,2,2),
     &                CRHpChiCha(5,2,2),CLHmChiCha(2,5,2),
     &                CRHmChiCha(2,5,2)
     
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      double complex B0i
      double precision A0
      
      integer i,j,ind1,ind2,ind3
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
     
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /coupl/ ChHpWmA,ChHmWpA,ChHpWmZ,ChHmWpZ,ChhWmWp,
     &               ChhAZ,ChhZZ,ChHpWm,ChHmWp,ChhZ,ChWmWp,ChZZ,
     &               ChSdmSdp,ChSemSep,ChSupSum,ChSvSvconj,ChHmHp,
     &               ChhSemSep,ChhSvSvconj,ChhHmHp,ChhSupSum,ChhSdmSdp,
     &               CLhChapCham,CRhChapCham,CLhFepFem,CRhFepFem,
     &               CLhFdpFdm,CRhFdpFdm,CLhFumFup,CRhFumFup,ChU3U3,
     &               ChU2U2,ChU4U4,Chhh,Chhhh,CLhChiChi,CRhChiChi,
     &               CHmHpSvSv,CHmHpSdmSdp,CHmHpSupSum,CHmHpSemSep,
     &               CHmSepSv,CHpSemSv,CHmSupSdp,CHpSumSdm,CRHpChiCha,
     &               CLHpChiCha,CRHmChiCha,CLHmChiCha
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
     
      
      SW=Sqrt(1-MW**2/MZ**2)
      CW=MW/MZ
      ThetaW=ACos(MW/MZ)
     
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccc dMW2   cccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c calculates W sefl energy split into parta-partd
      
      parta=0d0
      partb=0d0
      partc=0d0
      partd=0d0
      
         parta= 
     -           -(4*e**2*(4*MW**2*ReB0i(bb0,MW**2,0d0,MW**2) - 
     -          MZ**2*SW**2*ReB0i(bb0,MW**2,MZ**2,MW**2) + 
     -          8*ReB0i(bb00,MW**2,0d0,MW**2) + 
     -          2*MW**2*ReB0i(bb1,MW**2,0d0,MW**2)) + 
     -          (g2**2*(12*MZ**2*A0(mb**2) + 12*MZ**2*A0(md**2) + 
     -          4*MZ**2*A0(me**2) - 2*MZ**2*A0(MHpm**2) + 
     -          4*MZ**2*A0(mmu**2) + 12*MZ**2*A0(ms**2) + 
     -          4*MZ**2*A0(mtau**2) + 8*MW**2*A0(MW**2) - 
     -          14*MZ**2*A0(MW**2) + 8*MZ**2*SW**2*A0(MW**2) - 
     -          12*MW**2*A0(MZ**2) - MZ**2*A0(MZ**2) + 
     -          12*mt**2*MZ**2*ReB0i(bb0,MW**2,mb**2,mt**2) - 
     -          12*MW**2*MZ**2*ReB0i(bb0,MW**2,mb**2,mt**2) + 
     -          12*mu**2*MZ**2*ReB0i(bb0,MW**2,md**2,mu**2) - 
     -          12*MW**2*MZ**2*ReB0i(bb0,MW**2,md**2,mu**2) - 
     -          4*MW**2*MZ**2*ReB0i(bb0,MW**2,me**2,0d0) - 
     -          4*MW**2*MZ**2*ReB0i(bb0,MW**2,mmu**2,0d0) + 
     -          12*mc**2*MZ**2*ReB0i(bb0,MW**2,ms**2,mc**2) - 
     -          12*MW**2*MZ**2*ReB0i(bb0,MW**2,ms**2,mc**2) - 
     -          4*MW**2*MZ**2*ReB0i(bb0,MW**2,mtau**2,0d0) + 
     -          12*MW**4*ReB0i(bb0,MW**2,MW**2,MZ**2) + 
     -          8*MW**2*MZ**2*ReB0i(bb0,MW**2,MW**2,MZ**2) - 
     -          24*MZ**2*ReB0i(bb00,MW**2,mb**2,mt**2) - 
     -          24*MZ**2*ReB0i(bb00,MW**2,md**2,mu**2) - 
     -          8*MZ**2*ReB0i(bb00,MW**2,me**2,0d0) - 
     -          8*MZ**2*ReB0i(bb00,MW**2,mmu**2,0d0) - 
     -          24*MZ**2*ReB0i(bb00,MW**2,ms**2,mc**2) - 
     -          8*MZ**2*ReB0i(bb00,MW**2,mtau**2,0d0) + 
     -          32*MW**2*ReB0i(bb00,MW**2,MW**2,MZ**2) + 
     -          4*MZ**2*ReB0i(bb00,MW**2,MZ**2,MW**2) - 
     -          12*MW**2*MZ**2*ReB0i(bb1,MW**2,mb**2,mt**2) - 
     -          12*MW**2*MZ**2*ReB0i(bb1,MW**2,md**2,mu**2) - 
     -          4*MW**2*MZ**2*ReB0i(bb1,MW**2,me**2,0d0) - 
     -          4*MW**2*MZ**2*ReB0i(bb1,MW**2,mmu**2,0d0) - 
     -          12*MW**2*MZ**2*ReB0i(bb1,MW**2,ms**2,mc**2) - 
     -          4*MW**2*MZ**2*ReB0i(bb1,MW**2,mtau**2,0d0) - 
     -          8*MW**4*ReB0i(bb1,MW**2,MW**2,MZ**2)))/MZ**2)/
     -          (64.*Pi**2)
     
      do ind1=1,2,1
      do ind2=1,5,1
          partb=partb+
     -         (g2**2*(2*ReB0i(bb00,MW**2,MCha(ind1)**2,MChi(ind2)**2)*
     -         (Abs(NN(ind2,3))**2*Abs(UM(ind1,2))**2 + 
     -         2*Abs(NN(ind2,2))**2*
     -         (Abs(UM(ind1,1))**2 + Abs(UP(ind1,1))**2) + 
     -         Abs(NN(ind2,4))**2*Abs(UP(ind1,2))**2 + 
     -         Sqrt(2d0)*CONJG(NN(ind2,2))*CONJG(UM(ind1,2))*
     -         NN(ind2,3)*UM(ind1,1) + 
     -         Sqrt(2d0)*CONJG(NN(ind2,3))*CONJG(UM(ind1,1))*
     -         NN(ind2,2)*UM(ind1,2) - 
     -         Sqrt(2d0)*CONJG(NN(ind2,2))*CONJG(UP(ind1,2))*
     -         NN(ind2,4)*UP(ind1,1) - 
     -         Sqrt(2d0)*CONJG(NN(ind2,4))*CONJG(UP(ind1,1))*
     -         NN(ind2,2)*UP(ind1,2)) + 
     -         ReB0i(bb0,MW**2,MCha(ind1)**2,MChi(ind2)**2)*
     -         (MW**2*Abs(NN(ind2,4))**2*Abs(UP(ind1,2))**2 - 
     -         Abs(NN(ind2,4))**2*Abs(UP(ind1,2))**2*MChi(ind2)**2 + 
     -         Abs(NN(ind2,3))**2*Abs(UM(ind1,2))**2*(MW**2  
     -         - MChi(ind2)**2) + 2*Abs(NN(ind2,2))**2*
     -         (Abs(UM(ind1,1))**2 + Abs(UP(ind1,1))**2)*
     -         (MW**2 - MChi(ind2)**2) + 2*CONJG(UM(ind1,1))*
     -         CONJG(UP(ind1,1))*MCha(ind1)* MChi(ind2)*NN(ind2,2)**2 + 
     -         Sqrt(2d0)*CONJG(UM(ind1,2))*CONJG(UP(ind1,1))*
     -         MCha(ind1)*MChi(ind2)*NN(ind2,2)*NN(ind2,3) - 
     -         Sqrt(2d0)*CONJG(UM(ind1,1))*CONJG(UP(ind1,2))*
     -         MCha(ind1)*MChi(ind2)*NN(ind2,2)*NN(ind2,4) - 
     -         CONJG(UM(ind1,2))*CONJG(UP(ind1,2))*MCha(ind1)*
     -         MChi(ind2)*NN(ind2,3)*NN(ind2,4) + 
     -         Sqrt(2d0)*MW**2*CONJG(NN(ind2,2))*CONJG(UM(ind1,2))*
     -         NN(ind2,3)*UM(ind1,1) - Sqrt(2d0)*CONJG(NN(ind2,2))*
     -         CONJG(UM(ind1,2))*MChi(ind2)**2*NN(ind2,3)*UM(ind1,1) + 
     -         Sqrt(2d0)*MW**2*CONJG(NN(ind2,3))*CONJG(UM(ind1,1))*
     -         NN(ind2,2)*UM(ind1,2) - Sqrt(2d0)*CONJG(NN(ind2,3))*
     -         CONJG(UM(ind1,1))*MChi(ind2)**2*NN(ind2,2)*UM(ind1,2) - 
     -         Sqrt(2d0)*MW**2*CONJG(NN(ind2,2))*CONJG(UP(ind1,2))*
     -         NN(ind2,4)*UP(ind1,1) + Sqrt(2d0)*CONJG(NN(ind2,2))*
     -         CONJG(UP(ind1,2))*MChi(ind2)**2*NN(ind2,4)*UP(ind1,1) + 
     -         2*CONJG(NN(ind2,2))**2*MCha(ind1)*MChi(ind2)*
     -         UM(ind1,1)*UP(ind1,1) + Sqrt(2d0)*CONJG(NN(ind2,2))*
     -         CONJG(NN(ind2,3))*MCha(ind1)*MChi(ind2)*UM(ind1,2)* 
     -         UP(ind1,1) - Sqrt(2d0)*MW**2*CONJG(NN(ind2,4))*
     -         CONJG(UP(ind1,1))*NN(ind2,2)*UP(ind1,2) + 
     -         Sqrt(2d0)*CONJG(NN(ind2,4))*CONJG(UP(ind1,1))*
     -         MChi(ind2)**2*NN(ind2,2)*UP(ind1,2) - 
     -         Sqrt(2d0)*CONJG(NN(ind2,2))*CONJG(NN(ind2,4))*
     -         MCha(ind1)*MChi(ind2)*UM(ind1,1)*UP(ind1,2) - 
     -         CONJG(NN(ind2,3))*CONJG(NN(ind2,4))*MCha(ind1)*
     -         MChi(ind2)*UM(ind1,2)*UP(ind1,2)) - (A0(MCha(ind1)**2) - 
     -         MW**2*ReB0i(bb1,MW**2,MCha(ind1)**2,MChi(ind2)**2))*
     -         (2*Abs(NN(ind2,2))**2*Abs(UM(ind1,1))**2 + 
     -         Abs(NN(ind2,3))**2*Abs(UM(ind1,2))**2 + 
     -         2*Abs(NN(ind2,2))**2*Abs(UP(ind1,1))**2 + 
     -         Abs(NN(ind2,4))**2*Abs(UP(ind1,2))**2 + 
     -         Sqrt(2d0)*(CONJG(NN(ind2,2))*
     -         (CONJG(UM(ind1,2))*NN(ind2,3)*UM(ind1,1) - 
     -         CONJG(UP(ind1,2))*NN(ind2,4)*UP(ind1,1)) + NN(ind2,2)*
     -         (CONJG(NN(ind2,3))*CONJG(UM(ind1,1))*UM(ind1,2) - 
     -         CONJG(NN(ind2,4))*CONJG(UP(ind1,1))*UP(ind1,2))))))
     -         /(16.*Pi**2)
      end do
      end do
      
      do ind1=1,5,2
         partc=partc+
     -         (g2**2*(A0(MSv(INT(INT((1 + ind1)/2.)))**2) + 
     -         A0(MSe(ind1)**2)*Abs(ZE(ind1,ind1))**2 + 
     -         A0(MSe(1 + ind1)**2)*Abs(ZE(1 + ind1,ind1))**2 + 
     -         3*(A0(MSd(ind1)**2)*Abs(ZD(ind1,ind1))**2 + 
     -         A0(MSd(1 + ind1)**2)*Abs(ZD(1 + ind1,ind1))**2 + 
     -         A0(MSu(ind1)**2)*Abs(ZU(ind1,ind1))**2 + 
     -         A0(MSu(1 + ind1)**2)*Abs(ZU(1 + ind1,ind1))**2) - 
     -         4*(3*Abs(ZD(ind1,ind1))**2*(Abs(ZU(ind1,ind1))**2*
     -         ReB0i(bb00,MW**2,MSd(ind1)**2,MSu(ind1)**2) + 
     -         Abs(ZU(1 + ind1,ind1))**2*
     -         ReB0i(bb00,MW**2,MSd(ind1)**2,MSu(1 + ind1)**2))
     -         + 3*Abs(ZD(1 + ind1,ind1))**2*
     -         (Abs(ZU(ind1,ind1))**2*
     -         ReB0i(bb00,MW**2,MSd(1 + ind1)**2,MSu(ind1)**2)
     -         + Abs(ZU(1 + ind1,ind1))**2*
     -         ReB0i(bb00,MW**2,MSd(1 + ind1)**2,MSu(1 + ind1)**2)) + 
     -         Abs(ZE(ind1,ind1))**2*
     -         ReB0i(bb00,MW**2,MSe(ind1)**2,
     -         MSv(INT((1 + ind1)/2.))**2)+ Abs(ZE(1 + ind1,ind1))**2*
     -         ReB0i(bb00,MW**2,MSe(1 + ind1)**2,
     -         MSv(INT((1 + ind1)/2.))**2))))/(32.*Pi**2)
      end do
     
      do ind1=2,6,1
         partd=partd+
     -         (g2**2*(A0(Mh(-1 + ind1)**2)*
     -         (ZH(ind1,2)**2 + ZH(ind1,4)**2 + ZH(ind1,5)**2) - 
     -         4*(-((MW**2*ReB0i(bb0,MW**2,MW**2,Mh(-1 + ind1)**2) - 
     -         ReB0i(bb00,MW**2,Mh(-1 + ind1)**2,MW**2))*
     -         (Cos(beta)*ZH(ind1,4) + Sin(beta)*ZH(ind1,5))**2)
     -         + ReB0i(bb00,MW**2,Mh(-1 + ind1)**2,MHpm**2)*
     -         (ZH(ind1,2)**2 + (Sin(beta)*ZH(ind1,4) - 
     -         Cos(beta)*ZH(ind1,5))**2))))/(64.*Pi**2)

      end do
     
      
      dMW2=Realpart(parta+partb+partc+partd)
     

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccc dMZ2   cccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c calcualtes Z self energy (split into different parts)

      partcha=0d0
      partchi=0d0
      parthiggs1=0d0
      parthiggs2=0d0
      part1sq=0d0
      part2sq=0d0
      partrest=0d0
      
      do ind1=1,2,1
      do ind2=1,2,1
         partcha=partcha+
     -         (g2**2*(-((A0(MCha(ind2)**2)*(2*MW**2*CONJG(UM(ind1,1))*
     -         (2*MW**2*CONJG(UM(ind2,1))*UM(ind1,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UM(ind2,2))*UM(ind1,2))*
     -         UM(ind2,1) + (MW**2 - MZ**2*SW**2)*CONJG(UM(ind1,2))*
     -         (2*MW**2*CONJG(UM(ind2,1))*UM(ind1,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UM(ind2,2))*UM(ind1,2))*
     -         UM(ind2,2) + (2*MW**2*CONJG(UP(ind2,1))*UP(ind1,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UP(ind2,2))*UP(ind1,2))*
     -         (2*MW**2*CONJG(UP(ind1,1))*UP(ind2,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UP(ind1,2))*UP(ind2,2)))) 
     -         /MZ**2) +(2*B0i(bb00,MZ**2,MCha(ind1)**2,MCha(ind2)**2)*
     -         (2*MW**2*CONJG(UM(ind1,1))* (2*MW**2*CONJG(UM(ind2,1))*
     -         UM(ind1,1) + (MW**2 - MZ**2*SW**2)*CONJG(UM(ind2,2))*
     -         UM(ind1,2))*UM(ind2,1) + (MW**2 - MZ**2*SW**2)*
     -         CONJG(UM(ind1,2))*(2*MW**2*CONJG(UM(ind2,1))*UM(ind1,1)+ 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UM(ind2,2))*UM(ind1,2))*
     -         UM(ind2,2) + (2*MW**2*CONJG(UP(ind2,1))*UP(ind1,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UP(ind2,2))*UP(ind1,2))*
     -         (2*MW**2*CONJG(UP(ind1,1))*UP(ind2,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UP(ind1,2))*UP(ind2,2)))
     -         )/MZ**2 - B0i(bb1,MZ**2,MCha(ind1)**2,MCha(ind2)**2)*
     -         (2*MW**2*CONJG(UM(ind1,1))*(2*MW**2*CONJG(UM(ind2,1))*
     -         UM(ind1,1) + (MW**2 - MZ**2*SW**2)*CONJG(UM(ind2,2))*
     -         UM(ind1,2))*UM(ind2,1) + (MW**2 - MZ**2*SW**2)*
     -         CONJG(UM(ind1,2))*(2*MW**2*CONJG(UM(ind2,1))*UM(ind1,1)+ 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UM(ind2,2))*UM(ind1,2))*
     -         UM(ind2,2) + (2*MW**2*CONJG(UP(ind2,1))*UP(ind1,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UP(ind2,2))*UP(ind1,2))*
     -         (2*MW**2*CONJG(UP(ind1,1))*UP(ind2,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UP(ind1,2))*UP(ind2,2))) - 
     -         (B0i(bb0,MZ**2,MCha(ind1)**2,MCha(ind2)**2)*MCha(ind1)*
     -         (-((2*MW**2*CONJG(UP(ind2,1))*UP(ind1,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UP(ind2,2))*UP(ind1,2))*
     -         (2*MW**2*CONJG(UM(ind2,1))*MCha(ind2)*UM(ind1,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UM(ind2,2))*MCha(ind2)* 
     -         UM(ind1,2) + MCha(ind1)*(-2*MW**2*CONJG(UP(ind1,1))*
     -         UP(ind2,1) + (-MW**2 + MZ**2*SW**2)*CONJG(UP(ind1,2))*
     -         UP(ind2,2)))) + 2*MW**2*CONJG(UM(ind1,1))*UM(ind2,1)*
     -         (2*MW**2*CONJG(UM(ind2,1))*MCha(ind1)*UM(ind1,1) + 
     -         (MW**2 - MZ**2*SW**2)*CONJG(UM(ind2,2))*MCha(ind1)*
     -         UM(ind1,2) + MCha(ind2)*(-2*MW**2*CONJG(UP(ind1,1))*
     -         UP(ind2,1) + (-MW**2 + MZ**2*SW**2)*CONJG(UP(ind1,2))*
     -         UP(ind2,2))) + (MW**2 - MZ**2*SW**2)*CONJG(UM(ind1,2))*
     -         UM(ind2,2)*(2*MW**2*CONJG(UM(ind2,1))*MCha(ind1)*
     -         UM(ind1,1) + (MW**2 - MZ**2*SW**2)*CONJG(UM(ind2,2))*
     -         MCha(ind1)* UM(ind1,2) + MCha(ind2)* (-2*MW**2* 
     -         CONJG(UP(ind1,1))*UP(ind2,1) + (-MW**2 + MZ**2*SW**2)*
     -         CONJG(UP(ind1,2))*UP(ind2,2)))))/MZ**2))
     -         /(32.*MW**2*Pi**2)
      end do
      end do
    
      
      Masshh(1)=Mz
      Masshh(2:6)=Mh(1:5)
      
      do ind1=1,6,1
         parthiggs1=parthiggs1+
     -         ((0,-0.03125)*A0(Masshh(ind1)**2)*ChhZZ(ind1,ind1)) 
     -         /Pi**2 -(ReB0i(bb0,MZ**2,MZ**2,Masshh(ind1)**2)
     -         *ChZZ(ind1)**2)/(16.*Pi**2)

      do ind2=1,6,1
         parthiggs2=parthiggs2-
     -         (ReB0i(bb00,MZ**2,Masshh(ind1)**2,Masshh(ind2)**2)
     -         *ChhZ(ind1,ind2)**2)/(8.*Pi**2)

      end do
      end do
     
      
      
      do ind1=1,5,2
         part1sq=part1sq+
     -         (g2**2*(A0(MSd(ind1)**2)*((2*CW*MW + MZ)**2*
     -         Abs(ZD(ind1,ind1))**2 + 4*MZ**2*SW**4*
     -         Abs(ZD(ind1,1 + ind1))**2) + A0(MSd(1 + ind1)**2)*
     -         ((2*CW*MW + MZ)**2*Abs(ZD(1 + ind1,ind1))**2 + 
     -         4*MZ**2*SW**4*Abs(ZD(1 + ind1,1 + ind1))**2) + 
     -         3*A0(MSe(ind1)**2)*((-2*CW*MW + MZ)**2*
     -         Abs(ZE(ind1,ind1))**2 + 4*MZ**2*SW**4*
     -         Abs(ZE(ind1,1 + ind1))**2) + 3*A0(MSe(1 + ind1)**2)*
     -         ((-2*CW*MW + MZ)**2*Abs(ZE(1 + ind1,ind1))**2 + 
     -         4*MZ**2*SW**4*Abs(ZE(1 + ind1,1 + ind1))**2) + 
     -         A0(MSu(ind1)**2)*((-4*CW*MW + MZ)**2*
     -         Abs(ZU(ind1,ind1))**2 + 16*MZ**2*SW**4*
     -         Abs(ZU(ind1,1 + ind1))**2) + A0(MSu(1 + ind1)**2)*
     -         ((-4*CW*MW + MZ)**2*Abs(ZU(1 + ind1,ind1))**2 + 
     -         16*MZ**2*SW**4*Abs(ZU(1 + ind1,1 + ind1))**2)))/
     -         (96.*MW**2*Pi**2)
      do ind2=ind1,ind1+1,1
      do ind3=ind1,ind1+1,1
         part2sq=part2sq+
     -         (g2**2*(-(ReB0i(bb00,MZ**2,MSd(ind2)**2,MSd(ind3)**2)*
     -         ((2*MW**2 + MZ**2)**2*Abs(ZD(ind2,ind1))**2*
     -         Abs(ZD(ind3,ind1))**2 + 4*MZ**4*SW**4*
     -         Abs(ZD(ind2,1 + ind1))**2*Abs(ZD(ind3,1 + ind1))**2 + 
     -         2*(2*MW**4 - MW**2*MZ**2 - MZ**4)*(CONJG(ZD(ind2,ind1))*
     -         CONJG(ZD(ind3,1 + ind1))*ZD(ind2,1 + ind1)*ZD(ind3,ind1)
     -         + CONJG(ZD(ind2,1 + ind1))*CONJG(ZD(ind3,ind1))*
     -         ZD(ind2,ind1)*ZD(ind3,1 + ind1)))) - 
     -         3*ReB0i(bb00,MZ**2,MSe(ind2)**2,MSe(ind3)**2)*
     -         ((-2*MW**2 + MZ**2)**2*Abs(ZE(ind2,ind1))**2*
     -         Abs(ZE(ind3,ind1))**2 + 2*(2*MZ**4*SW**4*
     -         Abs(ZE(ind2,1 + ind1))**2*Abs(ZE(ind3,1 + ind1))**2 + 
     -         (2*MW**4 - 3*MW**2*MZ**2 + MZ**4)*(CONJG(ZE(ind2,ind1))*
     -         CONJG(ZE(ind3,1 + ind1))*ZE(ind2,1 + ind1)*ZE(ind3,ind1)
     -          + CONJG(ZE(ind2,1 + ind1))*CONJG(ZE(ind3,ind1))*
     -         ZE(ind2,ind1)*ZE(ind3,1 + ind1)))) - 
     -         ReB0i(bb00,MZ**2,MSu(ind2)**2,MSu(ind3)**2)*
     -         ((-4*MW**2 + MZ**2)**2*Abs(ZU(ind2,ind1))**2*
     -         Abs(ZU(ind3,ind1))**2 + 4*(4*MZ**4*SW**4*
     -         Abs(ZU(ind2,1 + ind1))**2*Abs(ZU(ind3,1 + ind1))**2 + 
     -         (4*MW**4 - 5*MW**2*MZ**2 + MZ**4)*(CONJG(ZU(ind2,ind1))*
     -         CONJG(ZU(ind3,1 + ind1))*ZU(ind2,1 + ind1)*ZU(ind3,ind1)
     -          + CONJG(ZU(ind2,1 + ind1))*CONJG(ZU(ind3,ind1))*
     -         ZU(ind2,ind1)*ZU(ind3,1 + ind1))))))/
     -         (48.*MW**2*MZ**2*Pi**2)
      end do
      end do
      end do
     
      
      do ind1=1,5,1
      do ind2=1,5,1
         partchi=partchi-
     -         (g2**2*MZ**2*(2*(A0(MChi(ind2)**2) - 
     -         2*B0i(bb00,MZ**2,MChi(ind1)**2,MChi(ind2)**2) + 
     -         MZ**2*B0i(bb1,MZ**2,MChi(ind1)**2,MChi(ind2)**2))*
     -         (CONJG(NN(ind2,3))*NN(ind1,3) - CONJG(NN(ind2,4))*
     -         NN(ind1,4))*(CONJG(NN(ind1,3))*NN(ind2,3) - 
     -         CONJG(NN(ind1,4))*NN(ind2,4)) + MChi(ind1)*
     -         B0i(bb0,MZ**2,MChi(ind1)**2,MChi(ind2)**2)*
     -         (CONJG(NN(ind2,3))**2*MChi(ind2)*NN(ind1,3)**2 + 
     -         CONJG(NN(ind2,4))**2*MChi(ind2)*NN(ind1,4)**2 + 
     -         MChi(ind2)*(CONJG(NN(ind1,3))*NN(ind2,3) - 
     -         CONJG(NN(ind1,4))*NN(ind2,4))**2 + 2*CONJG(NN(ind2,4))*
     -         MChi(ind1)*NN(ind1,4)*(-(CONJG(NN(ind1,3))*NN(ind2,3)) + 
     -         CONJG(NN(ind1,4))*NN(ind2,4)) - 2*CONJG(NN(ind2,3))*
     -         NN(ind1,3)*(CONJG(NN(ind2,4))*MChi(ind2)*NN(ind1,4) + 
     -         MChi(ind1)*(-(CONJG(NN(ind1,3))*NN(ind2,3)) + 
     -         CONJG(NN(ind1,4))*NN(ind2,4))))))/(64.*MW**2*Pi**2)
      end do
      end do
     
      
      partrest=
     -            -(g2**2*(9*MZ**4*A0(0d0) + 
     -       (8*MW**4 - 4*MW**2*MZ**2 + 5*MZ**4)*A0(mb**2) + 
     -       (32*MW**4 - 40*MW**2*MZ**2 + 17*MZ**4)*A0(mc**2) + 
     -       8*MW**4*A0(md**2) - 4*MW**2*MZ**2*A0(md**2) + 
     -       5*MZ**4*A0(md**2) + 24*MW**4*A0(me**2) - 
     -       36*MW**2*MZ**2*A0(me**2) + 15*MZ**4*A0(me**2) - 
     -       12*MW**4*A0(MHpm**2) + 12*MW**2*MZ**2*A0(MHpm**2) - 
     -       3*MZ**4*A0(MHpm**2) + 24*MW**4*A0(mmu**2) - 
     -       36*MW**2*MZ**2*A0(mmu**2) + 15*MZ**4*A0(mmu**2) + 
     -       8*MW**4*A0(ms**2) - 4*MW**2*MZ**2*A0(ms**2) + 
     -       5*MZ**4*A0(ms**2) + 32*MW**4*A0(mt**2) - 
     -       40*MW**2*MZ**2*A0(mt**2) + 17*MZ**4*A0(mt**2) + 
     -       24*MW**4*A0(mtau**2) - 36*MW**2*MZ**2*A0(mtau**2) + 
     -       15*MZ**4*A0(mtau**2) + 32*MW**4*A0(mu**2) - 
     -       40*MW**2*MZ**2*A0(mu**2) + 17*MZ**4*A0(mu**2) - 
     -       36*MW**4*A0(MW**2) + 12*MW**2*MZ**2*A0(MW**2) - 
     -       3*MZ**4*A0(MW**2) - 3*MZ**4*A0(MSv(1)**2) - 
     -       3*MZ**4*A0(MSv(2)**2) - 3*MZ**4*A0(MSv(3)**2) + 
     -       9*mb**2*MZ**4*ReB0i(bb0,MZ**2,mb**2,mb**2) + 
     -       9*mc**2*MZ**4*ReB0i(bb0,MZ**2,mc**2,mc**2) + 
     -       9*md**2*MZ**4*ReB0i(bb0,MZ**2,md**2,md**2) + 
     -       3*me**2*MZ**4*ReB0i(bb0,MZ**2,me**2,me**2) + 
     -       3*mmu**2*MZ**4*ReB0i(bb0,MZ**2,mmu**2,mmu**2) + 
     -       9*ms**2*MZ**4*ReB0i(bb0,MZ**2,ms**2,ms**2) + 
     -       9*mt**2*MZ**4*ReB0i(bb0,MZ**2,mt**2,mt**2) + 
     -       3*mtau**2*MZ**4*ReB0i(bb0,MZ**2,mtau**2,mtau**2) + 
     -       9*mu**2*MZ**4*ReB0i(bb0,MZ**2,mu**2,mu**2) + 
     -       54*MW**4*MZ**2*ReB0i(bb0,MZ**2,MW**2,MW**2) - 
     -       12*MW**2*MZ**4*ReB0i(bb0,MZ**2,MW**2,MW**2) - 
     -       18*MZ**4*ReB0i(bb00,MZ**2,0d0,0d0) - 
     -       16*MW**4*ReB0i(bb00,MZ**2,mb**2,mb**2) + 
     -       8*MW**2*MZ**2*ReB0i(bb00,MZ**2,mb**2,mb**2) - 
     -       10*MZ**4*ReB0i(bb00,MZ**2,mb**2,mb**2) - 
     -       64*MW**4*ReB0i(bb00,MZ**2,mc**2,mc**2) + 
     -       80*MW**2*MZ**2*ReB0i(bb00,MZ**2,mc**2,mc**2) - 
     -       34*MZ**4*ReB0i(bb00,MZ**2,mc**2,mc**2) - 
     -       16*MW**4*ReB0i(bb00,MZ**2,md**2,md**2) + 
     -       8*MW**2*MZ**2*ReB0i(bb00,MZ**2,md**2,md**2) - 
     -       10*MZ**4*ReB0i(bb00,MZ**2,md**2,md**2) - 
     -       48*MW**4*ReB0i(bb00,MZ**2,me**2,me**2) + 
     -       72*MW**2*MZ**2*ReB0i(bb00,MZ**2,me**2,me**2) - 
     -       30*MZ**4*ReB0i(bb00,MZ**2,me**2,me**2) + 
     -       24*MW**4*ReB0i(bb00,MZ**2,MHpm**2,MHpm**2) - 
     -       24*MW**2*MZ**2*ReB0i(bb00,MZ**2,MHpm**2,MHpm**2) + 
     -       6*MZ**4*ReB0i(bb00,MZ**2,MHpm**2,MHpm**2) - 
     -       48*MW**4*ReB0i(bb00,MZ**2,mmu**2,mmu**2) + 
     -       72*MW**2*MZ**2*ReB0i(bb00,MZ**2,mmu**2,mmu**2) - 
     -       30*MZ**4*ReB0i(bb00,MZ**2,mmu**2,mmu**2) - 
     -       16*MW**4*ReB0i(bb00,MZ**2,ms**2,ms**2) + 
     -       8*MW**2*MZ**2*ReB0i(bb00,MZ**2,ms**2,ms**2) - 
     -       10*MZ**4*ReB0i(bb00,MZ**2,ms**2,ms**2) - 
     -       64*MW**4*ReB0i(bb00,MZ**2,mt**2,mt**2) + 
     -       80*MW**2*MZ**2*ReB0i(bb00,MZ**2,mt**2,mt**2) - 
     -       34*MZ**4*ReB0i(bb00,MZ**2,mt**2,mt**2) - 
     -       48*MW**4*ReB0i(bb00,MZ**2,mtau**2,mtau**2) + 
     -       72*MW**2*MZ**2*ReB0i(bb00,MZ**2,mtau**2,mtau**2) - 
     -       30*MZ**4*ReB0i(bb00,MZ**2,mtau**2,mtau**2) - 
     -       64*MW**4*ReB0i(bb00,MZ**2,mu**2,mu**2) + 
     -       80*MW**2*MZ**2*ReB0i(bb00,MZ**2,mu**2,mu**2) - 
     -       34*MZ**4*ReB0i(bb00,MZ**2,mu**2,mu**2) + 
     -       72*MW**4*ReB0i(bb00,MZ**2,MW**2,MW**2) - 
     -       24*MW**2*MZ**2*ReB0i(bb00,MZ**2,MW**2,MW**2) + 
     -       6*MZ**4*ReB0i(bb00,MZ**2,MW**2,MW**2) + 
     -       6*MZ**4*ReB0i(bb00,MZ**2,MSv(1)**2,MSv(1)**2) + 
     -       6*MZ**4*ReB0i(bb00,MZ**2,MSv(2)**2,MSv(2)**2) + 
     -       6*MZ**4*ReB0i(bb00,MZ**2,MSv(3)**2,MSv(3)**2) + 
     -       9*MZ**6*ReB0i(bb1,MZ**2,0d0,0d0) + 
     -       8*MW**4*MZ**2*ReB0i(bb1,MZ**2,mb**2,mb**2) - 
     -       4*MW**2*MZ**4*ReB0i(bb1,MZ**2,mb**2,mb**2) + 
     -       5*MZ**6*ReB0i(bb1,MZ**2,mb**2,mb**2) + 
     -       32*MW**4*MZ**2*ReB0i(bb1,MZ**2,mc**2,mc**2) - 
     -       40*MW**2*MZ**4*ReB0i(bb1,MZ**2,mc**2,mc**2) + 
     -       17*MZ**6*ReB0i(bb1,MZ**2,mc**2,mc**2) + 
     -       8*MW**4*MZ**2*ReB0i(bb1,MZ**2,md**2,md**2) - 
     -       4*MW**2*MZ**4*ReB0i(bb1,MZ**2,md**2,md**2) + 
     -       5*MZ**6*ReB0i(bb1,MZ**2,md**2,md**2) + 
     -       24*MW**4*MZ**2*ReB0i(bb1,MZ**2,me**2,me**2) - 
     -       36*MW**2*MZ**4*ReB0i(bb1,MZ**2,me**2,me**2) + 
     -       15*MZ**6*ReB0i(bb1,MZ**2,me**2,me**2) + 
     -       24*MW**4*MZ**2*ReB0i(bb1,MZ**2,mmu**2,mmu**2) - 
     -       36*MW**2*MZ**4*ReB0i(bb1,MZ**2,mmu**2,mmu**2) + 
     -       15*MZ**6*ReB0i(bb1,MZ**2,mmu**2,mmu**2) + 
     -       8*MW**4*MZ**2*ReB0i(bb1,MZ**2,ms**2,ms**2) - 
     -       4*MW**2*MZ**4*ReB0i(bb1,MZ**2,ms**2,ms**2) + 
     -       5*MZ**6*ReB0i(bb1,MZ**2,ms**2,ms**2) + 
     -       32*MW**4*MZ**2*ReB0i(bb1,MZ**2,mt**2,mt**2) - 
     -       40*MW**2*MZ**4*ReB0i(bb1,MZ**2,mt**2,mt**2) + 
     -       17*MZ**6*ReB0i(bb1,MZ**2,mt**2,mt**2) + 
     -       24*MW**4*MZ**2*ReB0i(bb1,MZ**2,mtau**2,mtau**2) - 
     -       36*MW**2*MZ**4*ReB0i(bb1,MZ**2,mtau**2,mtau**2) + 
     -       15*MZ**6*ReB0i(bb1,MZ**2,mtau**2,mtau**2) + 
     -       32*MW**4*MZ**2*ReB0i(bb1,MZ**2,mu**2,mu**2) - 
     -       40*MW**2*MZ**4*ReB0i(bb1,MZ**2,mu**2,mu**2) + 
     -       17*MZ**6*ReB0i(bb1,MZ**2,mu**2,mu**2) + 
     -       12*MW**4*MZ**2*ReB0i(bb1,MZ**2,MW**2,MW**2)))/
     -       (96.*MW**2*MZ**2*Pi**2)
    
      
      dMZ2=Realpart(partcha+parthiggs1+parthiggs2+part1sq+part2sq+
     .     partchi+partrest)
    
      

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccc dZe   ccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
    
c calculation of self energy photon -> Z
      
      partrest = (e**2*(-2*MW**2*B0i(bb0,0d0,MW**2,MW**2) + 
     -           (A0(MHpm**2) - 2*B0i(bb00,0d0,MHpm**2,MHpm**2))*
     -           Cos(2*ThetaW) + A0(MW**2)*(2 + 3*Cos(2*ThetaW)) - 
     -           2*B0i(bb00,0d0,MW**2,MW**2)*(2 + 3*Cos(2*ThetaW)))/
     -           Cos(ThetaW)/Sin(ThetaW))/(16.*Pi**2)
      
      MassFd(1)=md
      MassFd(2)=ms
      MassFd(3)=mb
      MassFe(1)=me
      MassFe(2)=mmu
      MassFe(3)=mtau
      MassFu(1)=mu
      MassFu(2)=mc
      MassFu(3)=mt
      
      partF=0d0
      do ind1=1,3,1
      partF = partF -
     -         (e**2*(A0(MassFd(ind1)**2) + 3*A0(MassFe(ind1)**2) + 
     -         4*A0(MassFu(ind1)**2) - 
     -         2*(B0i(bb00,0d0,MassFd(ind1)**2,MassFd(ind1)**2) + 
     -         3*B0i(bb00,0d0,MassFe(ind1)**2,MassFe(ind1)**2) + 
     -         4*B0i(bb00,0d0,MassFu(ind1)**2,MassFu(ind1)**2)))
     -         /Tan(2*ThetaW))/(12.*Pi**2) - 
     -         (e**2*(A0(MassFd(ind1)**2) - 3*A0(MassFe(ind1)**2) - 
     -         2*(A0(MassFu(ind1)**2) + 
     -         B0i(bb00,0d0,MassFd(ind1)**2,MassFd(ind1)**2) - 
     -         3*B0i(bb00,0d0,MassFe(ind1)**2,MassFe(ind1)**2) - 
     -         2*B0i(bb00,0d0,MassFu(ind1)**2,MassFu(ind1)**2)))/
     -         Sin(2*ThetaW))/(24.*Pi**2)
      end do
      
      part1sq=0d0
      part2sq=0d0
      
      do ind1=1,5,2
      do ind2=ind1,ind1+1,1
      part1sq = part1sq +  
     -          (e**2*(A0(MSd(ind2)**2)*
     -          (-2*SW**2*Abs(ZD(ind2,1 + ind1))**2 + 
     -          Abs(ZD(ind2,ind1))**2*(2 + Cos(2*ThetaW))) + 
     -          2*A0(MSu(ind2)**2)*(-4*SW**2*Abs(ZU(ind2,1 + ind1))**2+ 
     -         Abs(ZU(ind2,ind1))**2*(1 + 2*Cos(2*ThetaW))) + 
     -         6*SW*A0(MSe(ind2)**2)*(-(SW*Abs(ZE(ind2,1 + ind1))**2) + 
     -         CW*Abs(ZE(ind2,ind1))**2/Tan(2*ThetaW))))/
     -         (48.*CW*Pi**2*SW)

      end do
      end do

      
      do ind1=1,5,2
      do ind2=ind1,ind1+1,1
      do ind3=ind1,ind1+1,1
      part2sq = part2sq +
     -         (e**2*(-((B0i(bb00,0d0,MSd(ind2)**2,MSd(ind3)**2)*
     -         (Abs(ZD(ind2,ind1))**2*Abs(ZD(ind3,ind1))**2*
     -         (2 + Cos(2*ThetaW))-2*SW**2*(Abs(ZD(ind2,1 + ind1))**2*
     -         Abs(ZD(ind3,1 + ind1))**2 + CONJG(ZD(ind2,ind1))*
     -         CONJG(ZD(ind3,1+ind1))*ZD(ind2,1+ind1)*ZD(ind3,ind1))+ 
     -         CONJG(ZD(ind2,1 + ind1))*CONJG(ZD(ind3,ind1))*
     -         (2 + Cos(2*ThetaW))*ZD(ind2,ind1)*ZD(ind3,1 + ind1)))/
     -         (CW*SW)) + 6*B0i(bb00,0d0,MSe(ind2)**2,MSe(ind3)**2)*
     -         ((SW*(Abs(ZE(ind2,1+ind1))**2*Abs(ZE(ind3,1+ind1))**2+ 
     -         CONJG(ZE(ind2,ind1))*CONJG(ZE(ind3,1 + ind1))*
     -         ZE(ind2,1 + ind1)*ZE(ind3,ind1)))/CW - 1/Tan(2*ThetaW)*
     -         (Abs(ZE(ind2,ind1))**2*Abs(ZE(ind3,ind1))**2 + 
     -         CONJG(ZE(ind2,1 + ind1))*CONJG(ZE(ind3,ind1))*
     -         ZE(ind2,ind1)*ZE(ind3,1 + ind1))) - 
     -         (2*B0i(bb00,0d0,MSu(ind2)**2,MSu(ind3)**2)*
     -         (-4*SW**3*(Abs(ZU(ind2,1 + ind1))**2*
     -         Abs(ZU(ind3,1 + ind1))**2 +CONJG(ZU(ind2,ind1))*
     -         CONJG(ZU(ind3,1+ind1))*ZU(ind2,1+ind1)*ZU(ind3,ind1)) + 
     -         Sin(3*ThetaW)*(Abs(ZU(ind2,ind1))**2*
     -         Abs(ZU(ind3,ind1))**2 + CONJG(ZU(ind2,1 + ind1))*
     -         CONJG(ZU(ind3,ind1))*ZU(ind2,ind1)*ZU(ind3,1+ind1))))/
     -        (CW*SW**2)))/(24.*Pi**2)
      end do
      end do
      end do
      
      partcha=0d0
      do ind1=1,2,1
      do ind2=1,2,1
      partcha = partcha +
     -         (g2**2*(-(A0(MCha(ind2)**2)*Tan(ThetaW)*
     -         (2*(Abs(UM(ind1,1)**2*UM(ind2,1)**2)+Abs(UP(ind1,1)**2
     -         *UP(ind2,1)**2))*Cos(ThetaW)**2 +CONJG(UM(ind1,2))*
     -         CONJG(UM(ind2,1))*UM(ind1,1)*UM(ind2,2) + 
     -         CONJG(UP(ind1,1))*CONJG(UP(ind2,2))*UP(ind1,2)*
     -         UP(ind2,1) + Cos(2*ThetaW)*(Abs(UM(ind1,2)**2*
     -         UM(ind2,2)**2) + Abs(UP(ind1,2)**2*UP(ind2,2)**2) + 
     -         CONJG(UM(ind1,1))*CONJG(UM(ind2,2))*UM(ind1,2)*
     -         UM(ind2,1) + CONJG(UM(ind1,2))*CONJG(UM(ind2,1))*
     -         UM(ind1,1)*UM(ind2,2) + CONJG(UP(ind1,1))*
     -         CONJG(UP(ind2,2))*UP(ind1,2)*UP(ind2,1) + 
     -         CONJG(UP(ind1,2))*CONJG(UP(ind2,1))*UP(ind1,1)*
     -        UP(ind2,2))))+2*B0i(bb00,0d0,MCha(ind1)**2,MCha(ind2)**2)
     -         *Tan(ThetaW)*(2*(Abs(UM(ind1,1)**2*UM(ind2,1)**2) + 
     -         Abs(UP(ind1,1)**2*UP(ind2,1)**2))*Cos(ThetaW)**2
     -         + CONJG(UM(ind1,2))*CONJG(UM(ind2,1))*UM(ind1,1)*
     -         UM(ind2,2)+CONJG(UP(ind1,1))*CONJG(UP(ind2,2))*
     -         UP(ind1,2)*UP(ind2,1) + Cos(2*ThetaW)*(Abs(UM(ind1,2)**2
     -         *UM(ind2,2)**2) + Abs(UP(ind1,2)**2*UP(ind2,2)**2) + 
     -         CONJG(UM(ind1,1))*CONJG(UM(ind2,2))*UM(ind1,2)*
     -         UM(ind2,1) + CONJG(UM(ind1,2))*CONJG(UM(ind2,1))*
     -         UM(ind1,1)*UM(ind2,2) + CONJG(UP(ind1,1))*
     -         CONJG(UP(ind2,2))*UP(ind1,2)*UP(ind2,1) + 
     -         CONJG(UP(ind1,2))*CONJG(UP(ind2,1))*UP(ind1,1)*
     -         UP(ind2,2)))+B0i(bb0,0d0,MCha(ind1)**2,MCha(ind2)**2)
     -         *MCha(ind1)*(MCha(ind2)*Tan(ThetaW)*
     -         (2*CONJG(UM(ind2,1))*Cos(ThetaW)**2*UM(ind1,1)*
     -         (CONJG(UP(ind2,1))*UP(ind1,1) +CONJG(UP(ind2,2))* 
     -         UP(ind1,2))+CONJG(UM(ind2,2))*Cos(2*ThetaW)*UM(ind1,2)*
     -         (CONJG(UP(ind2,1))*UP(ind1,1) + CONJG(UP(ind2,2))*
     -         UP(ind1,2)) + (CONJG(UM(ind1,1))*UM(ind2,1) + 
     -         CONJG(UM(ind1,2))*UM(ind2,2))*(2*CONJG(UP(ind1,1))*
     -         Cos(ThetaW)**2*UP(ind2,1) +CONJG(UP(ind1,2))*
     -         Cos(2*ThetaW)*UP(ind2,2))) + MCha(ind1)*(Tan(ThetaW)*
     -         (Abs(UM(ind1,2)*UM(ind2,2))**2 +Abs(UP(ind1,2)*
     -         UP(ind2,2))**2 + CONJG(UM(ind1,1))*CONJG(UM(ind2,2))*
     -         UM(ind1,2)*UM(ind2,1) + CONJG(UP(ind1,2))*
     -         CONJG(UP(ind2,1))*UP(ind1,1)*UP(ind2,2))-Sin(2*ThetaW)*
     -         (Abs(UM(ind1,1)*UM(ind2,1))**2 + Abs(UM(ind1,2)*
     -         UM(ind2,2))**2 + Abs(UP(ind1,1)*UP(ind2,1))**2 + 
     -         Abs(UP(ind1,2)*UP(ind2,2))**2 + CONJG(UM(ind1,1))*
     -         CONJG(UM(ind2,2))*UM(ind1,2)*UM(ind2,1) + 
     -         CONJG(UM(ind1,2))*CONJG(UM(ind2,1))*UM(ind1,1)* 
     -         UM(ind2,2) +CONJG(UP(ind1,1))*CONJG(UP(ind2,2))*
     -         UP(ind1,2)*UP(ind2,1) + CONJG(UP(ind1,2))*
     -         CONJG(UP(ind2,1))*UP(ind1,1)*UP(ind2,2))))))/(16.*Pi**2)
      end do
      end do
     
      EAZ = partrest + partF + part1sq + part2sq + partcha


c calculation of derivative of photon self energy 

        partrest=-(e**2*(5*B0i(bb0,0d0,MW**2,MW**2) + 
     -           2*(B0i(bb1,0d0,MW**2,MW**2) + 
     -           2*B0i(dbb00,0d0,MHpm**2,MHpm**2) + 
     -           6*B0i(dbb00,0d0,MW**2,MW**2))))/(16.*Pi**2)
     
      partF=0d0
      do ind1=1,3,1
      partF = partF -
     -        (e**2*(B0i(bb1,0d0,MassFd(ind1)**2,MassFd(ind1)**2) + 
     -       3*B0i(bb1,0d0,MassFe(ind1)**2,MassFe(ind1)**2) + 
     -       4*B0i(bb1,0d0,MassFu(ind1)**2,MassFu(ind1)**2) - 
     -       2*B0i(dbb00,0d0,MassFd(ind1)**2,MassFd(ind1)**2) - 
     -       6*B0i(dbb00,0d0,MassFe(ind1)**2,MassFe(ind1)**2) - 
     -       8*B0i(dbb00,0d0,MassFu(ind1)**2,MassFu(ind1)**2)))/
     -      (12.*Pi**2)
      end do
      
      partsq=0d0
      do ind1=1,5,2
      do ind2=ind1,ind1+1,1
      do ind3=ind1,ind1+1,1
      partsq = partsq + 
     -         (e**2*(-(B0i(dbb00,0d0,MSd(ind2)**2,MSd(ind3)**2)*
     -         (Abs(ZD(ind2,ind1))**2*Abs(ZD(ind3,ind1))**2 + 
     -         Abs(ZD(ind2,1 + ind1))**2*Abs(ZD(ind3,1 + ind1))**2 + 
     -         CONJG(ZD(ind2,ind1))*CONJG(ZD(ind3,1 + ind1))*
     -         ZD(ind2,1 + ind1)*ZD(ind3,ind1) + 
     -         CONJG(ZD(ind2,1 + ind1))*CONJG(ZD(ind3,ind1))*
     -         ZD(ind2,ind1)*ZD(ind3,1 + ind1))) - 
     -         3*B0i(dbb00,0d0,MSe(ind2)**2,MSe(ind3)**2)*
     -         (Abs(ZE(ind2,ind1))**2*Abs(ZE(ind3,ind1))**2 + 
     -         Abs(ZE(ind2,1 + ind1))**2*Abs(ZE(ind3,1 + ind1))**2 + 
     -         CONJG(ZE(ind2,ind1))*CONJG(ZE(ind3,1 + ind1))*
     -         ZE(ind2,1 + ind1)*ZE(ind3,ind1) + 
     -         CONJG(ZE(ind2,1 + ind1))*CONJG(ZE(ind3,ind1))*
     -         ZE(ind2,ind1)*ZE(ind3,1 + ind1)) - 
     -         4*B0i(dbb00,0d0,MSu(ind2)**2,MSu(ind3)**2)*
     -         (Abs(ZU(ind2,ind1))**2*Abs(ZU(ind3,ind1))**2 + 
     -         Abs(ZU(ind2,1 + ind1))**2*Abs(ZU(ind3,1 + ind1))**2 + 
     -         CONJG(ZU(ind2,ind1))*CONJG(ZU(ind3,1 + ind1))*
     -         ZU(ind2,1 + ind1)*ZU(ind3,ind1) + 
     -         CONJG(ZU(ind2,1 + ind1))*CONJG(ZU(ind3,ind1))*
     -         ZU(ind2,ind1)*ZU(ind3,1 + ind1))))/(12.*Pi**2)
      
      end do
      end do
      end do
      
      partcha=0d0
      do ind1=1,2,1
      do ind2=1,2,1
      partcha = partcha +
     -         (e**2*(-(B0i(bb1,0d0,MCha(ind1)**2,MCha(ind2)**2)*
     -         (Abs(UM(ind1,1))**2*Abs(UM(ind2,1))**2 + 
     -         Abs(UM(ind1,2))**2*Abs(UM(ind2,2))**2 + 
     -         Abs(UP(ind1,1))**2*Abs(UP(ind2,1))**2 + 
     -         Abs(UP(ind1,2))**2*Abs(UP(ind2,2))**2 + 
     -         CONJG(UM(ind1,1))*CONJG(UM(ind2,2))*UM(ind1,2)* 
     -         UM(ind2,1)+CONJG(UM(ind1,2))*CONJG(UM(ind2,1))*
     -         UM(ind1,1)*UM(ind2,2) + CONJG(UP(ind1,1))*
     -         CONJG(UP(ind2,2))*UP(ind1,2)*UP(ind2,1) + 
     -         CONJG(UP(ind1,2))*CONJG(UP(ind2,1))*UP(ind1,1)
     -         *UP(ind2,2))) + 
     -         2*B0i(dbb00,0d0,MCha(ind1)**2,MCha(ind2)**2)*
     -         (Abs(UM(ind1,1))**2*Abs(UM(ind2,1))**2 + 
     -         Abs(UM(ind1,2))**2*Abs(UM(ind2,2))**2 + 
     -         Abs(UP(ind1,1))**2*Abs(UP(ind2,1))**2 + 
     -         Abs(UP(ind1,2))**2*Abs(UP(ind2,2))**2 + 
     -         CONJG(UM(ind1,1))*CONJG(UM(ind2,2))*UM(ind1,2)*
     -         UM(ind2,1) + CONJG(UM(ind1,2))*CONJG(UM(ind2,1))*
     -         UM(ind1,1)*UM(ind2,2) + CONJG(UP(ind1,1))*
     -         CONJG(UP(ind2,2))*UP(ind1,2)*UP(ind2,1) + 
     -         CONJG(UP(ind1,2))*CONJG(UP(ind2,1))*UP(ind1,1)*
     -         UP(ind2,2)) + 
     -         B0i(dbb0,0d0,MCha(ind1)**2,MCha(ind2)**2)*MCha(ind1)*
     -         (-(MCha(ind1)*(Abs(UM(ind1,1))**2*Abs(UM(ind2,1))**2 + 
     -         Abs(UM(ind1,2))**2*Abs(UM(ind2,2))**2 + 
     -         Abs(UP(ind1,1))**2*Abs(UP(ind2,1))**2 + 
     -         Abs(UP(ind1,2))**2*Abs(UP(ind2,2))**2 + 
     -         CONJG(UM(ind1,1))*CONJG(UM(ind2,2))*UM(ind1,2)*
     -         UM(ind2,1) + CONJG(UM(ind1,2))*CONJG(UM(ind2,1))
     -         *UM(ind1,1)*UM(ind2,2) + CONJG(UP(ind1,1))*
     -         CONJG(UP(ind2,2))*UP(ind1,2)*UP(ind2,1) + 
     -         CONJG(UP(ind1,2))*CONJG(UP(ind2,1))*UP(ind1,1)*
     -         UP(ind2,2))) + MCha(ind2)*(CONJG(UM(ind2,1))*UM(ind1,1)*
     -         (CONJG(UP(ind2,1))*UP(ind1,1) + CONJG(UP(ind2,2))*
     -         UP(ind1,2)) + CONJG(UM(ind2,2))*UM(ind1,2)*
     -         (CONJG(UP(ind2,1))*UP(ind1,1) + CONJG(UP(ind2,2))*
     -         UP(ind1,2)) + (CONJG(UM(ind1,1))*UM(ind2,1) + 
     -         CONJG(UM(ind1,2))*UM(ind2,2))*(CONJG(UP(ind1,1))*
     -        UP(ind2,1) + CONJG(UP(ind1,2))*UP(ind2,2))))))/(8.*Pi**2)
      end do
      end do
     
      DEAA=partrest + partsq + partcha + partF
      
! the additional term addrun is necessary when the running 1/alpha_em is used
      addrun=  -(e**2*(A0(mb**2) + 4*A0(mc**2) + A0(md**2) + 
     -       3*A0(me**2) + 3*A0(mmu**2) + A0(ms**2) + 
     -       3*A0(mtau**2) + 4*A0(mu**2) - 
     -       2*ReB0i(bb00,Mz**2,mb**2,mb**2) - 
     -       8*ReB0i(bb00,Mz**2,mc**2,mc**2) - 
     -       2*ReB0i(bb00,Mz**2,md**2,md**2) - 
     -       6*ReB0i(bb00,Mz**2,me**2,me**2) - 
     -       6*ReB0i(bb00,Mz**2,mmu**2,mmu**2) - 
     -       2*ReB0i(bb00,Mz**2,ms**2,ms**2) - 
     -       6*ReB0i(bb00,Mz**2,mtau**2,mtau**2) - 
     -       8*ReB0i(bb00,Mz**2,mu**2,mu**2) - 
     -       Mz**2*ReB0i(bb1,0d0,mb**2,mb**2) - 
     -       4*Mz**2*ReB0i(bb1,0d0,mc**2,mc**2) - 
     -       Mz**2*ReB0i(bb1,0d0,md**2,md**2) - 
     -       3*Mz**2*ReB0i(bb1,0d0,me**2,me**2) - 
     -       3*Mz**2*ReB0i(bb1,0d0,mmu**2,mmu**2) - 
     -       Mz**2*ReB0i(bb1,0d0,ms**2,ms**2) - 
     -       3*Mz**2*ReB0i(bb1,0d0,mtau**2,mtau**2) - 
     -       4*Mz**2*ReB0i(bb1,0d0,mu**2,mu**2) + 
     -       Mz**2*ReB0i(bb1,Mz**2,mb**2,mb**2) + 
     -       4*Mz**2*ReB0i(bb1,Mz**2,mc**2,mc**2) + 
     -       Mz**2*ReB0i(bb1,Mz**2,md**2,md**2) + 
     -       3*Mz**2*ReB0i(bb1,Mz**2,me**2,me**2) + 
     -       3*Mz**2*ReB0i(bb1,Mz**2,mmu**2,mmu**2) + 
     -       Mz**2*ReB0i(bb1,Mz**2,ms**2,ms**2) + 
     -       3*Mz**2*ReB0i(bb1,Mz**2,mtau**2,mtau**2) + 
     -       4*Mz**2*ReB0i(bb1,Mz**2,mu**2,mu**2) + 
     -       2*Mz**2*ReB0i(dbb00,0d0,mb**2,mb**2) + 
     -       8*Mz**2*ReB0i(dbb00,0d0,mc**2,mc**2) + 
     -       2*Mz**2*ReB0i(dbb00,0d0,md**2,md**2) + 
     -       6*Mz**2*ReB0i(dbb00,0d0,me**2,me**2) + 
     -       6*Mz**2*ReB0i(dbb00,0d0,mmu**2,mmu**2) + 
     -       2*Mz**2*ReB0i(dbb00,0d0,ms**2,ms**2) + 
     -       6*Mz**2*ReB0i(dbb00,0d0,mtau**2,mtau**2) + 
     -       8*Mz**2*ReB0i(dbb00,0d0,mu**2,mu**2)))/
     -      (24.*Mz**2*pi**2)

      dZe=Realpart(0.5d0*DEAA+SW/CW*EAZ/MZ**2 +addrun)



ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccc dHpm2   ccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c self energy of the charged Higgs boson


      partrest=(2*A0(MW**2)*(2*CW**2*(abslambda**2 + 4*g2**2) + 
     -         Cos(4*beta)*(abslambda**2 - g2**2+abslambda**2
     -         *Cos(2*ThetaW))) + 2*A0(MHpm**2)*(abslambda**2 + g2**2 + 
     -         (-abslambda**2 + g2**2)*Cos(4*beta)+2*abslambda**2*
     -         Cos(2*ThetaW)*Sin(2*beta)**2) + g2**2*((3*A0(MZ**2) - 
     -         2*MHpm**2*(2*B0i(bb0,MHpm**2,MHpm**2,MZ**2) + 
     -         B0i(bb1,MHpm**2,MZ**2,MHpm**2)))*(1 + Cos(4*ThetaW)) - 
     -         4*MHpm**2*(2*B0i(bb0,MHpm**2,0d0,MHpm**2) + 
     -         B0i(bb1,MHpm**2,0d0,MHpm**2))*Sin(2*ThetaW)**2))/
     -         (128.*CW**2*Pi**2)
     
      parthiggs1=0d0
      parthiggs2=0d0
      MassHpm(1)=MW
      MassHpm(2)=MHpm
      
      do ind1=1,6,1
      parthiggs1 = parthiggs1 +
     -      ((0,0.03125)*A0(Masshh(ind1)**2)*ChhHmHp(2,ind1,2,ind1))/
     -      Pi**2+(A0(MW**2)*ChHmWp(2,ind1)*ChHpWm(ind1,2))/(16.*Pi**2)
     -      +(MHpm**2*B0i(bb1,MHpm**2,MW**2,Masshh(ind1)**2)*
     -      ChHmWp(2,ind1)*ChHpWm(ind1,2))/(8.*Pi**2) + 
     -      (B0i(bb0,MHpm**2,MW**2,Masshh(ind1)**2)*ChHmWp(2,ind1)*
     -      ChHpWm(ind1,2)*(3*MHpm**2 + Masshh(ind1)**2))/(16.*Pi**2)
      do ind2=1,2,1
      parthiggs2 = parthiggs2 -
     -            (B0i(bb0,MHpm**2,Masshh(ind1)**2,MassHpm(ind2)**2)*
     -            ChHmHp(2,ind2,ind1)*ChHmHp(ind2,2,ind1))/(16.*Pi**2)
      end do
      end do
      
      partF=0d0
      do ind1=1,3,1
      partF = partF +
     -      (-3*A0(MassFd(ind1)**2)*(Sin(beta)**2*Yd(ind1)**2 
     -      + Cos(beta)**2*Yu(ind1)**2) - 
     -      3*B0i(bb0,MHpm**2,MassFd(ind1)**2,MassFu(ind1)**2)*
     -      (-((MHpm**2 - MassFu(ind1)**2)*Sin(beta)**2*
     -      Yd(ind1)**2) + 2*Cos(beta)*MassFd(ind1)*
     -      MassFu(ind1)*Sin(beta)*Yd(ind1)*Yu(ind1) + 
     -      Cos(beta)**2*(-MHpm**2 + MassFu(ind1)**2)*Yu(ind1)**2) + 
     -      ((-A0(MassFe(ind1)**2) + MHpm**2*
     -      B0i(bb1,MHpm**2,MassFe(ind1)**2, 0d0) + 
     -      B0i(bb0,MHpm**2,MassFe(ind1)**2,0d0)*MHpm**2)*Sin(beta)**2*
     -      Ye(ind1)**2 + 3*MHpm**2*
     -      B0i(bb1,MHpm**2,MassFd(ind1)**2,MassFu(ind1)**2)*
     -      (Sin(beta)**2*Yd(ind1)**2 + Cos(beta)**2*Yu(ind1)**2)))/
     -      (8.*Pi**2)
      end do
      
      partcha=0d0
      do ind1=1,2,1
      do ind2=1,5,1
      partcha = partcha +
     -       (A0(MCha(ind1)**2)*(CLHpChiCha(ind2,ind1,2)*
     -       CRHmChiCha(ind1,ind2,2) + CLHmChiCha(ind1,ind2,2)
     -       *CRHpChiCha(ind2,ind1,2)))/(8.*Pi**2) - (MHpm**2*
     -       B0i(bb1,MHpm**2,MCha(ind1)**2,MChi(ind2)**2)*
     -       (CLHpChiCha(ind2,ind1,2)*CRHmChiCha(ind1,ind2,2) + 
     -       CLHmChiCha(ind1,ind2,2)*CRHpChiCha(ind2,ind1,2)))/
     -       (8.*Pi**2) + B0i(bb0,MHpm**2,MCha(ind1)**2,MChi(ind2)**2)*
     -       (((CLHmChiCha(ind1,ind2,2)*CLHpChiCha(ind2,ind1,2) + 
     -       CRHmChiCha(ind1,ind2,2)*CRHpChiCha(ind2,ind1,2))*
     -       MCha(ind1)*MChi(ind2))/(8.*Pi**2) + 
     -       ((CLHpChiCha(ind2,ind1,2)*CRHmChiCha(ind1,ind2,2) + 
     -       CLHmChiCha(ind1,ind2,2)*CRHpChiCha(ind2,ind1,2))*
     -       (-MHpm**2 + MChi(ind2)**2))/(8.*Pi**2))
      end do
      end do
      
      
      part1sq=0d0
      part2sq=0d0
      part3sq=0d0
      part4sq=0d0
      
      do ind1=1,3,1
      part1sq = part1sq +
     -    ((0,0.0625)*A0(MSv(ind1)**2)*CHmHpSvSv(ind1,2,ind1,2))/Pi**2
      do ind2=1,6,1
      part2sq = part2sq -
     -     (B0i(bb0,MHpm**2,MSe(ind2)**2,MSv(ind1)**2)*
     -     CHmSepSv(ind1,ind2,2)*CHpSemSv(ind2,ind1,2))/(16.*Pi**2)
      end do
      end do
      
      do ind1=1,6,1
      part3sq = part3sq +
     -     ((0,0.1875)*A0(MSd(ind1)**2)*CHmHpSdmSdp(ind1,2,ind1,2))/
     -     Pi**2 + ((0,0.0625)*A0(MSe(ind1)**2)*
     -     CHmHpSemSep(ind1,2,ind1,2))/Pi**2 + 
     -     ((0,0.1875)*A0(MSu(ind1)**2)*
     -     CHmHpSupSum(ind1,2,ind1,2))/Pi**2
      do ind2=1,6,1
      part4sq = part4sq +
     -         (-3*B0i(bb0,MHpm**2,MSd(ind2)**2,MSu(ind1)**2)*
     -    CHmSupSdp(ind1,ind2,2)*CHpSumSdm(ind2,ind1,2))/(16.*Pi**2)
      end do
      end do
      


      
      dMHpm2=Realpart(partrest+parthiggs1+parthiggs2+partF+partcha+
     -       part1sq+part2sq+part3sq+part4sq)



ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccc Tadpoles   ccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


      do i=1,6,1
      
        part1=0d0
        part2=0d0
        part3=0d0
        part4=0d0
        part5=0d0
      
! ghost and vector bosons in the loop
         part1=  ((0,-0.0625)*(A0(Mw**2)*
     -           (ChU3U3(i) + ChU4U4(i) + 4*ChWmWp(i)) + 
     -           A0(Mz**2)*(ChU2U2(i) + 2*ChZZ(i))))/Pi**2
    
! charginos, charged Higgs and charged Goldstone boson in the loop
        MassHpm(1)=Mw
        MassHpm(2)=Mhpm
      
        do j=1,2,1
          part2=  part2 + ((0,0.0625)*(A0(MassHpm(j)**2)*ChHmHp(j,j,i)
     -              - 2*A0(MCha(j)**2)*(CLhChapCham(j,j,i) + 
     -              CRhChapCham(j,j,i))*MCha(j)))/Pi**2
        end do

     
     
! SM-fermions and sneutrions in the loop
        MassFd(1)=md
        MassFd(2)=ms
        MassFd(3)=mb
        MassFe(1)=me
        MassFe(2)=mmu
        MassFe(3)=mtau
        MassFu(1)=mu
        MassFu(2)=mc
        MassFu(3)=mt

        do j=1,3,1
         part3=  part3 + ((0,-0.0625)*(-(A0(MSv(j)**2)*ChSvSvconj(i)) + 
     -           2*(3*A0(MassFd(j)**2)*(CLhFdpFdm(j,j,i) + 
     -           CRhFdpFdm(j,j,i))*MassFd(j)+ A0(MassFe(j)**2)*
     -           (CLhFepFem(j,j,i) + CRhFepFem(j,j,i))*MassFe(j)
     -           + 3*A0(MassFu(j)**2)*(CLhFumFup(j,j,i) +
     -           CRhFumFup(j,j,i))*MassFu(j))))/Pi**2
        end do
        
      
! Neutralinos in the loop

        do j=1,5,1
         part4=  part4 + ((0,-0.0625)*A0(MChi(j)**2)*
     -           (CLhChiChi(j,j,i) + CRhChiChi(j,j,i))*MChi(j))/Pi**2
        end do
     
     
! Higgs bosons and squarks in the loop

        Masshh(1)=Mz
        Masshh(2:6)=Mh(1:5)

        do j=1,6,1
         part5=  part5 + ((0,0.03125)*(A0(Masshh(j)**2)*Chhh(i,j,j) + 
     -           6*A0(MSd(j)**2)*ChSdmSdp(j,j,i) + 
     -           2*A0(MSe(j)**2)*ChSemSep(j,j,i) + 
     -           6*A0(MSu(j)**2)*ChSupSum(j,j,i)))/Pi**2
        end do

     
        tadpole(i)=part1+part2+part3+part4+part5
      end do
      
      dtG=0d0
      dtaud=0d0
      dtas=0d0
      dtad=0d0
      dthd=0d0
      dthu=0d0
      dths=0d0
      
      do j=1,6,1
      dtG=dtG+Realpart(ZH(j,1)*tadpole(j))
      dtaud=dtaud+Realpart(ZH(j,2)*tadpole(j))
      dtas=dtas+Realpart(ZH(j,3)*tadpole(j))
      dthd=dthd+Realpart(ZH(j,4)*tadpole(j))
      dthu=dthu+Realpart(ZH(j,5)*tadpole(j))
      dths=dths+Realpart(ZH(j,6)*tadpole(j))
      end do
      
      dtad=-Cos(beta)*dtG+Sin(beta)*dtaud
      
      return
      end
      
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccc selfHpm(p^2) cccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      double precision function selfHpm(p2)
      
      implicit none
      
      double precision p2
      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3,ThetaW
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
    
      
      double precision Masshh(6),MassFu(3),MassFe(3),MassFd(3),
     &                 MassHpm(2)
    
     
      double complex   partcha,part1sq,
     .                 part2sq,parthiggs1,parthiggs2,partrest,
     .                 partF,part3sq,part4sq
     
      double complex  ChHpWmA(6,2),ChHmWpA(2,6),ChHpWmZ(6,2),
     &                ChHmWpZ(2,6),ChhWmWp(6,6),ChhAZ(6,6),ChhZZ(6,6),
     &                ChHpWm(6,2),ChHmWp(2,6),ChhZ(6,6),ChWmWp(6),
     &                ChZZ(6),ChSdmSdp(6,6,6),ChSemSep(6,6,6),
     &                ChSupSum(6,6,6),ChSvSvconj(6),ChHmHp(2,2,6),
     &                ChhSemSep(6,6,6,6),ChhSvSvconj(6,6),
     &                ChhHmHp(2,6,2,6),ChhSupSum(6,6,6,6),
     &                ChhSdmSdp(6,6,6,6),CLhChapCham(2,2,6),
     &                CRhChapCham(2,2,6),CLhFepFem(3,3,6),
     &                CRhFepFem(3,3,6),CLhFdpFdm(3,3,6),
     &                CRhFdpFdm(3,3,6),CLhFumFup(3,3,6),
     &                CRhFumFup(3,3,6),ChU3U3(6),ChU2U2(6),ChU4U4(6),
     &                Chhh(6,6,6),Chhhh(6,6,6,6),CLhChiChi(5,5,6),
     &                CRhChiChi(5,5,6),CHmHpSvSv(3,2,3,2),
     &                CHmHpSdmSdp(6,2,6,2),CHmHpSupSum(6,2,6,2),
     &                CHmHpSemSep(6,2,6,2),CHmSepSv(3,6,2),
     &                CHpSemSv(6,3,2),CHmSupSdp(6,6,2),
     &                CHpSumSdm(6,6,2),CLHpChiCha(5,2,2),
     &                CRHpChiCha(5,2,2),CLHmChiCha(2,5,2),
     &                CRHmChiCha(2,5,2)
     
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      double complex B0i
      double precision A0
      
      integer i,ind1,ind2
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /coupl/ ChHpWmA,ChHmWpA,ChHpWmZ,ChHmWpZ,ChhWmWp,
     &               ChhAZ,ChhZZ,ChHpWm,ChHmWp,ChhZ,ChWmWp,ChZZ,
     &               ChSdmSdp,ChSemSep,ChSupSum,ChSvSvconj,ChHmHp,
     &               ChhSemSep,ChhSvSvconj,ChhHmHp,ChhSupSum,ChhSdmSdp,
     &               CLhChapCham,CRhChapCham,CLhFepFem,CRhFepFem,
     &               CLhFdpFdm,CRhFdpFdm,CLhFumFup,CRhFumFup,ChU3U3,
     &               ChU2U2,ChU4U4,Chhh,Chhhh,CLhChiChi,CRhChiChi,
     &               CHmHpSvSv,CHmHpSdmSdp,CHmHpSupSum,CHmHpSemSep,
     &               CHmSepSv,CHpSemSv,CHmSupSdp,CHpSumSdm,CRHpChiCha,
     &               CLHpChiCha,CRHmChiCha,CLHmChiCha
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
      
      ThetaW=ACos(MW/MZ)
      
c MMM changed 11/8/2013
      partrest=
     -        (A0(Mw**2)*(2*(abslambda**2 + 4*g2**2) + 
     -        (2*abslambda**2 - g1**2 - g2**2)*Cos(4*beta)) + 
     -        A0(MHpm**2)*((g1**2 + g2**2)*(1 + Cos(4*beta)) + 
     -        4*abslambda**2*Sin(2*beta)**2) + 
     -        (-((g1**2 + g2**2)*((MHpm**2 + 3*p2)*
     -        B0i(bb0,p2,MHpm**2,Mz**2) + 2*p2*(B0i(bb1,p2,0d0,MHpm**2)
     -         + B0i(bb1,p2,Mz**2,MHpm**2)))) + (g1 - g2)*(g1 + g2)*
     -        ((MHpm**2 + 3*p2)*B0i(bb0,p2,MHpm**2,Mz**2) + 
     -        2*p2*(-B0i(bb1,p2,0d0,MHpm**2) + B0i(bb1,p2,Mz**2,MHpm**2
     -        )))*Cos(2*ThetaW)+ 6*A0(Mz**2)*(g2*Cos(ThetaW) - 
     -        g1*Sin(ThetaW))**2 - 2*(MHpm**2 + 3*p2)*
     -        B0i(bb0,p2,0d0,MHpm**2)*(g1*Cos(ThetaW) + 
     -        g2*Sin(ThetaW))**2 + 2*g1*g2*((MHpm**2 + 3*p2)*
     -        B0i(bb0,p2,MHpm**2,Mz**2) + 2*p2*(-B0i(bb1,p2,0d0,MHpm**2)
     -        + B0i(bb1,p2,Mz**2,MHpm**2)))*Sin(2*ThetaW))/2.)
     .        /(64.*Pi**2)
c end MMM changed 11/8/2013
     
      parthiggs1=0d0
      parthiggs2=0d0
      Masshh(1)=Mz
      do i=2,6
      Masshh(i)=Mh(i-1)
      end do
      MassHpm(1)=MW
      MassHpm(2)=MHpm
      
      do ind1=1,6,1
      parthiggs1 = parthiggs1 +
     -      ((0,1)*A0(Masshh(ind1)**2)*ChhHmHp(2,ind1,2,ind1) + 
     -    2*ChHmWp(2,ind1)*ChHpWm(ind1,2)*
     -     (A0(Mw**2) + 2*p2*
     -        B0i(bb1,p2,Mw**2,Masshh(ind1)**2) + 
     -       B0i(bb0,p2,Mw**2,Masshh(ind1)**2)*
     -        (3*p2 + Masshh(ind1)**2)))/(32.*Pi**2)
      do ind2=1,2,1
      parthiggs2 = parthiggs2 -
     -            (B0i(bb0,p2,Masshh(ind1)**2,MassHpm(ind2)**2)*
     -            ChHmHp(2,ind2,ind1)*ChHmHp(ind2,2,ind1))/(16.*Pi**2)
      end do
      end do
     
        MassFd(1)=md
        MassFd(2)=ms
        MassFd(3)=mb
        MassFe(1)=me
        MassFe(2)=mmu
        MassFe(3)=mtau
        MassFu(1)=mu
        MassFu(2)=mc
        MassFu(3)=mt
      
      partF=0d0
      do ind1=1,3,1
      partF = partF +
     -      (Sin(beta)**2*(-3*(A0(MassFd(ind1)**2) - 
     -      p2*B0i(bb1,p2,MassFd(ind1)**2,MassFu(ind1)**2) + 
     -      B0i(bb0,p2,MassFd(ind1)**2,MassFu(ind1)**2)*
     -      (-p2 + MassFu(ind1)**2))*Yd(ind1)**2 + 
     -      (-A0(MassFe(ind1)**2) + 
     -      p2*(B0i(bb0,p2,0d0,MassFe(ind1)**2) + 
     -      B0i(bb1,p2,MassFe(ind1)**2,0d0)))*Ye(ind1)**2) - 
     -      6*B0i(bb0,p2,MassFd(ind1)**2,MassFu(ind1)**2)*Cos(beta)*
     -      MassFd(ind1)*MassFu(ind1)*Sin(beta)*Yd(ind1)*Yu(ind1) - 
     -      3*Cos(beta)**2*(A0(MassFd(ind1)**2) - 
     -      p2*B0i(bb1,p2,MassFd(ind1)**2,MassFu(ind1)**2) + 
     -      B0i(bb0,p2,MassFd(ind1)**2,MassFu(ind1)**2)*
     -      (-p2 + MassFu(ind1)**2))*Yu(ind1)**2)/(8.*Pi**2)

      end do
      
      partcha=0d0
      do ind1=1,2,1
      do ind2=1,5,1
c MMM changed 10/8/2013
      partcha = partcha +
     -     ((A0(MCha(ind1)**2) 
     .     - p2*(B0i(bb0,p2,MCha(ind1)**2,MChi(ind2)**2) + 
     -     B0i(bb1,p2,MCha(ind1)**2,MChi(ind2)**2)))*
     -     (CLHpChiCha(ind2,ind1,2)*CRHmChiCha(ind1,ind2,2) + 
     -     CLHmChiCha(ind1,ind2,2)*CRHpChiCha(ind2,ind1,2)) + 
     -     B0i(bb0,p2,MCha(ind1)**2,MChi(ind2)**2)*
     -     (CLHmChiCha(ind1,ind2,2)*CLHpChiCha(ind2,ind1,2) + 
     -     CRHmChiCha(ind1,ind2,2)*CRHpChiCha(ind2,ind1,2))*MCha(ind1)*
     .        MChi(ind2) + 
     -     B0i(bb0,p2,MCha(ind1)**2,MChi(ind2)**2)*
     -     (CLHpChiCha(ind2,ind1,2)*CRHmChiCha(ind1,ind2,2) + 
     -     CLHmChiCha(ind1,ind2,2)*CRHpChiCha(ind2,ind1,2))*
     .        MChi(ind2)**2)/(8.*Pi**2)
c end MMM changed 10/8/2013
      end do
      end do
      
      
      part1sq=0d0
      part2sq=0d0
      part3sq=0d0
      part4sq=0d0
      
      do ind1=1,3,1
      part1sq = part1sq +
     -    ((0,0.0625)*A0(MSv(ind1)**2)*CHmHpSvSv(ind1,2,ind1,2))/Pi**2
      do ind2=1,6,1
      part2sq = part2sq -
     -     (B0i(bb0,p2,MSe(ind2)**2,MSv(ind1)**2)*
     -     CHmSepSv(ind1,ind2,2)*CHpSemSv(ind2,ind1,2))/(16.*Pi**2)
      end do
      end do
      
      do ind1=1,6,1
      part3sq = part3sq +
     -     ((0,0.1875)*A0(MSd(ind1)**2)*CHmHpSdmSdp(ind1,2,ind1,2))/
     -     Pi**2 + ((0,0.0625)*A0(MSe(ind1)**2)*
     -     CHmHpSemSep(ind1,2,ind1,2))/Pi**2 + 
     -     ((0,0.1875)*A0(MSu(ind1)**2)*
     -     CHmHpSupSum(ind1,2,ind1,2))/Pi**2
      do ind2=1,6,1
      part4sq = part4sq +
     -         (-3*B0i(bb0,p2,MSd(ind2)**2,MSu(ind1)**2)*
     -    CHmSupSdp(ind1,ind2,2)*CHpSumSdm(ind2,ind1,2))/(16.*Pi**2)
      end do
      end do
      


c MMM changed 10/8/2013      
      selfHpm=Realpart(partrest+parthiggs1+parthiggs2+partF+partcha
     .     +part1sq+part2sq+part3sq+part4sq)
c end MMM changed 10/8/2103

      
      return
      end
     

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccc Couplings cccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
     
      subroutine couplings

      implicit none


      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
     
      double precision ZH5(5,5),ZH(6,6),p(6, 6, 6, 6, 6, 6),OP
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double complex  ChHpWmA(6,2),ChHmWpA(2,6),ChHpWmZ(6,2),
     &                ChHmWpZ(2,6),ChhWmWp(6,6),ChhAZ(6,6),ChhZZ(6,6),
     &                ChHpWm(6,2),ChHmWp(2,6),ChhZ(6,6),ChWmWp(6),
     &                ChZZ(6),ChSdmSdp(6,6,6),ChSemSep(6,6,6),
     &                ChSupSum(6,6,6),ChSvSvconj(6),ChHmHp(2,2,6),
     &                ChhSemSep(6,6,6,6),ChhSvSvconj(6,6),
     &                ChhHmHp(2,6,2,6),ChhSupSum(6,6,6,6),
     &                ChhSdmSdp(6,6,6,6),CLhChapCham(2,2,6),
     &                CRhChapCham(2,2,6),CLhFepFem(3,3,6),
     &                CRhFepFem(3,3,6),CLhFdpFdm(3,3,6),
     &                CRhFdpFdm(3,3,6),CLhFumFup(3,3,6),
     &                CRhFumFup(3,3,6),ChU3U3(6),ChU2U2(6),ChU4U4(6),
     &                Chhh(6,6,6),Chhhh(6,6,6,6),CLhChiChi(5,5,6),
     &                CRhChiChi(5,5,6),CHmHpSvSv(3,2,3,2),
     &                CHmHpSdmSdp(6,2,6,2),CHmHpSupSum(6,2,6,2),
     &                CHmHpSemSep(6,2,6,2),CHmSepSv(3,6,2),
     &                CHpSemSv(6,3,2),CHmSupSdp(6,6,2),
     &                CHpSumSdm(6,6,2),CLHpChiCha(5,2,2),
     &                CRHpChiCha(5,2,2),CLHmChiCha(2,5,2),
     &                CRHmChiCha(2,5,2)
      
      double precision CThetaW,SThetaW,S2ThetaW,C2ThetaW,ZP(2,2),vu,vd
      
      double precision phi,phiIk,phiIl
      
      double complex EPhilPhisPhiu,EPhilPhis,EPhiu,EPhi,EPhiIl,
     &               Ad(3),Ae(3),Au(3),lambda,kappa,EPhis,clambda,
     &               ckappa,EPhisPhiu
     
      double precision pi
      parameter (pi = 2d0*DAsin(1d0))
      
      integer i,h1,h2,h3,h4,Hp1,Hm1,Sdm1,Sdp1,Sem1,Sep1,Sup1,Sum1,
     &        Chap1,Cham1,Fep1,Fem1,Fdp1,Fdm1,Fum1,Fup1,j,k,l,m,n,
     &        Chi1,Chi2,Sv1,Sv2
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
     
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /coupl/ ChHpWmA,ChHmWpA,ChHpWmZ,ChHmWpZ,ChhWmWp,
     &               ChhAZ,ChhZZ,ChHpWm,ChHmWp,ChhZ,ChWmWp,ChZZ,
     &               ChSdmSdp,ChSemSep,ChSupSum,ChSvSvconj,ChHmHp,
     &               ChhSemSep,ChhSvSvconj,ChhHmHp,ChhSupSum,ChhSdmSdp,
     &               CLhChapCham,CRhChapCham,CLhFepFem,CRhFepFem,
     &               CLhFdpFdm,CRhFdpFdm,CLhFumFup,CRhFumFup,ChU3U3,
     &               ChU2U2,ChU4U4,Chhh,Chhhh,CLhChiChi,CRhChiChi,
     &               CHmHpSvSv,CHmHpSdmSdp,CHmHpSupSum,CHmHpSemSep,
     &               CHmSepSv,CHpSemSv,CHmSupSdp,CHpSumSdm,CRHpChiCha,
     &               CLHpChiCha,CRHmChiCha,CLHmChiCha
      
      CThetaW=Mw/Mz
      C2ThetaW=2d0*CThetaW**2-1d0
      SThetaW=Sqrt(1d0-Mw**2/Mz**2)
      S2ThetaW=2d0*SThetaW*CThetaW
      ZP(1,1)=-Cos(beta)
      ZP(1,2)=Sin(beta)
      ZP(2,2)=Cos(beta)
      ZP(2,1)=Sin(beta)
      vu=v*Sin(beta)
      vd=v*Cos(beta)
      
      do i=1,3,1
        Ad(i)=absAd(i)*Exp((0d0,1d0)*phiAd(i))
        Ae(i)=absAe(i)*Exp((0d0,1d0)*phiAe(i))
        Au(i)=absAu(i)*Exp((0d0,1d0)*phiAu(i))
      end do
      
      EPhilPhisPhiu=(1d0,0d0)*dCos(phil+phis+phiu)+
     &               (0d0,1d0)*dSin(phil+phis+phiu)
      EPhilPhis=(1d0,0d0)*dCos(phil+phis)+
     &               (0d0,1d0)*dSin(phil+phis)
      EPhiu=(1d0,0d0)*dCos(phiu)+(0d0,1d0)*dSin(phiu)
      EPhi=(1d0,0d0)*dCos(phiu+phil-phik-2*phis)+
     &              (0d0,1d0)*dSin(phiu+phil-phik-2*phis)
      EPhiIl=(1d0,0d0)*dCos(phiu+phis+phil+phiAl)+
     &               (0d0,1d0)*dSin(phiu+phis+phil+phiAl)
      EPhis=(1d0,0d0)*dCos(phis)+(0d0,1d0)*dSin(phis)
      EPhisPhiu=EPhiu*EPhis
     
     
      phi=phil-phik+phiu-2*phis
      phiIl=phiu+phis+phiAl+phil
      phiIk=3*phis+phiAk+phik
      
      lambda=abslambda*Exp((0d0,1d0)*phil)
      kappa=abskappa*Exp((0d0,1d0)*phik)
      clambda=abslambda*Exp(-(0d0,1d0)*phil)
      ckappa=abskappa*Exp(-(0d0,1d0)*phik)
      
      do h1=1,6,1
        do Hp1=1,2,1
        ChHpWmA(h1,Hp1)=
     -          (CThetaW*g1*g2*((-(Cos(beta)*ZH(h1,1)) + 
     -          Sin(beta)*ZH(h1,2))*ZP(Hp1,1) - 
     -           (0d0,1d0)*ZH(h1,4)*ZP(Hp1,1) + 
     -          (Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2))*
     -           ZP(Hp1,2) + (0d0,1d0)*ZH(h1,5)*ZP(Hp1,2)))/2d0
        end do
      end do

      do Hm1=1,2,1
         do h1=1,6,1
            ChHmWpA(Hm1,h1)=
     -         -(CThetaW*g1*g2*(ZH(h1,2)*
     -         (Sin(beta)*ZP(Hm1,1) + Cos(beta)*ZP(Hm1,2)) + 
     -         ZH(h1,1)*(-(Cos(beta)*ZP(Hm1,1)) + 
     -         Sin(beta)*ZP(Hm1,2)) + 
     -         (0d0,1d0)*(ZH(h1,4)*ZP(Hm1,1) - ZH(h1,5)*ZP(Hm1,2))))/2d0
         end do
      end do
      
      do h1=1,6,1
        do Hp1=1,2,1
          ChHpWmZ(h1,Hp1)=
     -          (g1*g2*SThetaW*(-(ZH(h1,2)*
     -          (Sin(beta)*ZP(Hp1,1) + Cos(beta)*ZP(Hp1,2))) + 
     -          ZH(h1,1)*(Cos(beta)*ZP(Hp1,1) - 
     -          Sin(beta)*ZP(Hp1,2)) + (0d0,1d0)*(ZH(h1,4)*ZP(Hp1,1)
     -          - ZH(h1,5)*ZP(Hp1,2))))/2d0
        end do
      end do
      
      do Hm1=1,2,1
         do h1=1,6,1
           ChHmWpZ(Hm1,h1)=
     -          (g1*g2*SThetaW*(ZH(h1,2)*
     -          (Sin(beta)*ZP(Hm1,1) + Cos(beta)*ZP(Hm1,2)) + 
     -          ZH(h1,1)*(-(Cos(beta)*ZP(Hm1,1)) + 
     -          Sin(beta)*ZP(Hm1,2)) + (0d0,1d0)*
     -          (ZH(h1,4)*ZP(Hm1,1) - ZH(h1,5)*ZP(Hm1,2))))/2d0
         end do
      end do
      
      do h1=1,6,1
        do h2=1,6,1
           ChhWmWp(h1,h2)=
     -          (0,0.5)*g2**2*(ZH(h1,1)*ZH(h2,1) + ZH(h1,2)*ZH(h2,2) + 
     -          ZH(h1,4)*ZH(h2,4) + ZH(h1,5)*ZH(h2,5))
        end do
      end do
      
      do h1=1,6,1
        do h2=1,6,1
           ChhAZ(h1,h2)=0d0
        end do
      end do
      
      do h1=1,6,1
        do h2=1,6,1
          ChhZZ(h1,h2)=
     -       (0,0.5)*(CThetaW*g2 + g1*SThetaW)**2*
     -       (ZH(h1,1)*ZH(h2,1) + ZH(h1,2)*ZH(h2,2) + ZH(h1,4)*ZH(h2,4)
     -       + ZH(h1,5)*ZH(h2,5))
        end do
      end do
      
      do h1=1,6,1
        do Hp1=1,2,1
           ChHpWm(h1,Hp1)=
     -      (g2*((-(Cos(beta)*ZH(h1,1)) + Sin(beta)*ZH(h1,2))*ZP(Hp1,1) 
     -       - (0d0,1d0)*ZH(h1,4)*ZP(Hp1,1) + 
     -      (Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2) + 
     -      (0d0,1d0)*ZH(h1,5))*ZP(Hp1,2)))/2d0
        end do
      end do
      
      do Hm1=1,2,1
         do h1=1,6,1
            ChHmWp(Hm1,h1)=
     -           -(g2*((-(Cos(beta)*ZH(h1,1)) + Sin(beta)*ZH(h1,2) + 
     -           (0d0,1d0)*ZH(h1,4))*ZP(Hm1,1) + 
     -           (Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2) - 
     -           (0d0,1d0)*ZH(h1,5))*ZP(Hm1,2)))/2d0
         end do
      end do
     
      do h1=1,6,1
         do h2=1,6,1
            ChhZ(h1,h2)=
     -          ((CThetaW*g2 + g1*SThetaW)*
     -          (ZH(h1,5)*(Sin(beta)*ZH(h2,1) + Cos(beta)*ZH(h2,2)) + 
     -           ZH(h1,4)*(Cos(beta)*ZH(h2,1) - Sin(beta)*ZH(h2,2)) - 
     -          Cos(beta)*ZH(h1,1)*ZH(h2,4) + 
     -          Sin(beta)*ZH(h1,2)*ZH(h2,4) - 
     -          Sin(beta)*ZH(h1,1)*ZH(h2,5) - 
     -          Cos(beta)*ZH(h1,2)*ZH(h2,5)))/2d0
         end do
      end do
      
      do h1=1,6,1
        ChWmWp(h1)=
     -      (0,0.5)*g2**2*v*(Cos(beta)*ZH(h1,4) + Sin(beta)*ZH(h1,5))
      end do
      
      do h1=1,6,1
        ChZZ(h1)=
     -          (0,0.5)*(CThetaW*g2 + g1*SThetaW)**2*v*
     -          (Cos(beta)*ZH(h1,4) + Sin(beta)*ZH(h1,5))
      end do
      
      do Sdm1=1,6,1
         do Sdp1=1,6,1
           do h1=1,6,1
            ChSdmSdp(Sdm1,Sdp1,h1)= ((0,-12)*v*Cos(beta)*
     -          (Yd(1)**2*(CONJG(ZD(Sdm1,1))*ZD(Sdp1,1) + 
     -          CONJG(ZD(Sdm1,2))*ZD(Sdp1,2)) + 
     -          Yd(2)**2*(CONJG(ZD(Sdm1,3))*ZD(Sdp1,3) + 
     -          CONJG(ZD(Sdm1,4))*ZD(Sdp1,4)) + 
     -           Yd(3)**2*(CONJG(ZD(Sdm1,5))*ZD(Sdp1,5) + 
     -          CONJG(ZD(Sdm1,6))*ZD(Sdp1,6)))*ZH(h1,4) + 
     -          (0,6)*Sqrt(2d0)*((0,1)*
     -          (Ad(1)*CONJG(ZD(Sdm1,1))*Yd(1)*ZD(Sdp1,2) + 
     -          Ad(2)*CONJG(ZD(Sdm1,3))*Yd(2)*ZD(Sdp1,4) + 
     -          Ad(3)*CONJG(ZD(Sdm1,5))*Yd(3)*ZD(Sdp1,6))*
     -          (Cos(beta)*ZH(h1,1) - Sin(beta)*ZH(h1,2)) + 
     -          (CONJG(Ad(1))*CONJG(ZD(Sdm1,2))*Yd(1)*ZD(Sdp1,1) + 
     -          CONJG(Ad(2))*CONJG(ZD(Sdm1,4))*Yd(2)*ZD(Sdp1,3) + 
     -          CONJG(Ad(3))*CONJG(ZD(Sdm1,6))*Yd(3)*ZD(Sdp1,5))*
     -          ((0,-1)*Cos(beta)*ZH(h1,1) + 
     -          (0,1)*Sin(beta)*ZH(h1,2) - ZH(h1,4)) - 
     -          (Ad(1)*CONJG(ZD(Sdm1,1))*Yd(1)*ZD(Sdp1,2) + 
     -          Ad(2)*CONJG(ZD(Sdm1,3))*Yd(2)*ZD(Sdp1,4) + 
     -          Ad(3)*CONJG(ZD(Sdm1,5))*Yd(3)*ZD(Sdp1,6))*ZH(h1,4))
     -          + (0,3)*g2**2*v*(CONJG(ZD(Sdm1,1))*ZD(Sdp1,1) + 
     -          CONJG(ZD(Sdm1,3))*ZD(Sdp1,3) + 
     -          CONJG(ZD(Sdm1,5))*ZD(Sdp1,5))*
     -          (Cos(beta)*ZH(h1,4) - Sin(beta)*ZH(h1,5)) + 
     -          (0,1)*g1**2*v*(CONJG(ZD(Sdm1,1))*ZD(Sdp1,1) + 
     -          CONJG(ZD(Sdm1,3))*ZD(Sdp1,3) + 
     -          CONJG(ZD(Sdm1,5))*ZD(Sdp1,5) + 
     -          2*(CONJG(ZD(Sdm1,2))*ZD(Sdp1,2) + 
     -          CONJG(ZD(Sdm1,4))*ZD(Sdp1,4) + 
     -          CONJG(ZD(Sdm1,6))*ZD(Sdp1,6)))*
     -          (Cos(beta)*ZH(h1,4) - Sin(beta)*ZH(h1,5)) - 
     -          6*abslambda*EPhilPhisPhiu*
     -          (CONJG(ZD(Sdm1,2))*Yd(1)*ZD(Sdp1,1) + 
     -          CONJG(ZD(Sdm1,4))*Yd(2)*ZD(Sdp1,3) + 
     -          CONJG(ZD(Sdm1,6))*Yd(3)*ZD(Sdp1,5))*
     -          (vS*Sin(beta)*ZH(h1,1) + vS*Cos(beta)*ZH(h1,2) - 
     -          (0,1)*vS*ZH(h1,5) + 
     -          v*Sin(beta)*(ZH(h1,3) - (0,1)*ZH(h1,6))) + 
     -          (6*abslambda*(CONJG(ZD(Sdm1,1))*Yd(1)*ZD(Sdp1,2) + 
     -          CONJG(ZD(Sdm1,3))*Yd(2)*ZD(Sdp1,4) + 
     -          CONJG(ZD(Sdm1,5))*Yd(3)*ZD(Sdp1,6))*
     -          (vS*Sin(beta)*ZH(h1,1) + vS*Cos(beta)*ZH(h1,2) + 
     -          (0,1)*vS*ZH(h1,5) + 
     -          v*Sin(beta)*(ZH(h1,3) + (0,1)*ZH(h1,6))))/
     -          EPhilPhisPhiu)/12.
           end do
         end do
      end do

      
      do Sem1=1,6,1
         do Sep1=1,6,1
           do h1=1,6,1
            ChSemSep(Sem1,Sep1,h1)=((0,-4)*v*Cos(beta)*
     -          (Ye(1)**2*(CONJG(ZE(Sem1,1))*ZE(Sep1,1) + 
     -          CONJG(ZE(Sem1,2))*ZE(Sep1,2)) + 
     -          Ye(2)**2*(CONJG(ZE(Sem1,3))*ZE(Sep1,3) + 
     -          CONJG(ZE(Sem1,4))*ZE(Sep1,4)) + 
     -          Ye(3)**2*(CONJG(ZE(Sem1,5))*ZE(Sep1,5) + 
     -          CONJG(ZE(Sem1,6))*ZE(Sep1,6)))*ZH(h1,4) + 
     -          (0,2)*Sqrt(2d0)*((0,1)*
     -          (Ae(1)*CONJG(ZE(Sem1,1))*Ye(1)*ZE(Sep1,2) + 
     -          Ae(2)*CONJG(ZE(Sem1,3))*Ye(2)*ZE(Sep1,4) + 
     -          Ae(3)*CONJG(ZE(Sem1,5))*Ye(3)*ZE(Sep1,6))*
     -          (Cos(beta)*ZH(h1,1) - Sin(beta)*ZH(h1,2)) + 
     -          (CONJG(Ae(1))*CONJG(ZE(Sem1,2))*Ye(1)*ZE(Sep1,1) + 
     -          CONJG(Ae(2))*CONJG(ZE(Sem1,4))*Ye(2)*ZE(Sep1,3) + 
     -          CONJG(Ae(3))*CONJG(ZE(Sem1,6))*Ye(3)*ZE(Sep1,5))
     -          *((0,-1)*Cos(beta)*ZH(h1,1) + 
     -          (0,1)*Sin(beta)*ZH(h1,2) - ZH(h1,4)) - 
     -          (Ae(1)*CONJG(ZE(Sem1,1))*Ye(1)*ZE(Sep1,2) + 
     -          Ae(2)*CONJG(ZE(Sem1,3))*Ye(2)*ZE(Sep1,4) + 
     -          Ae(3)*CONJG(ZE(Sem1,5))*Ye(3)*ZE(Sep1,6))*ZH(h1,4))
     -          +(0,1)*g2**2*v*(CONJG(ZE(Sem1,1))*ZE(Sep1,1) + 
     -          CONJG(ZE(Sem1,3))*ZE(Sep1,3) + 
     -          CONJG(ZE(Sem1,5))*ZE(Sep1,5))*
     -          (Cos(beta)*ZH(h1,4) - Sin(beta)*ZH(h1,5)) - 
     -          (0,1)*g1**2*v*(CONJG(ZE(Sem1,1))*ZE(Sep1,1) + 
     -          CONJG(ZE(Sem1,3))*ZE(Sep1,3) + 
     -          CONJG(ZE(Sem1,5))*ZE(Sep1,5) - 
     -          2*(CONJG(ZE(Sem1,2))*ZE(Sep1,2) + 
     -          CONJG(ZE(Sem1,4))*ZE(Sep1,4) + 
     -          CONJG(ZE(Sem1,6))*ZE(Sep1,6)))*
     -          (Cos(beta)*ZH(h1,4) - Sin(beta)*ZH(h1,5)) - 
     -          2*abslambda*EPhilPhisPhiu*
     -          (CONJG(ZE(Sem1,2))*Ye(1)*ZE(Sep1,1) + 
     -          CONJG(ZE(Sem1,4))*Ye(2)*ZE(Sep1,3) + 
     -          CONJG(ZE(Sem1,6))*Ye(3)*ZE(Sep1,5))*
     -          (vS*Sin(beta)*ZH(h1,1) + vS*Cos(beta)*ZH(h1,2) - 
     -          (0,1)*vS*ZH(h1,5) + 
     -          v*Sin(beta)*(ZH(h1,3) - (0,1)*ZH(h1,6))) + 
     -          (2*abslambda*(CONJG(ZE(Sem1,1))*Ye(1)*ZE(Sep1,2) + 
     -          CONJG(ZE(Sem1,3))*Ye(2)*ZE(Sep1,4) + 
     -          CONJG(ZE(Sem1,5))*Ye(3)*ZE(Sep1,6))*
     -          (vS*Sin(beta)*ZH(h1,1) + vS*Cos(beta)*ZH(h1,2) + 
     -          (0,1)*vS*ZH(h1,5) + v*Sin(beta)*(ZH(h1,3) 
     -          + (0,1)*ZH(h1,6))))/EPhilPhisPhiu)/4.
           end do
         end do
      end do  
      

      do Sup1=1,6,1
         do Sum1=1,6,1
            do h1=1,6,1
            ChSupSum(Sup1,Sum1,h1)=((0,-3)*g2**2*v*(Cos(beta)*ZH(h1,4)  
     -          - Sin(beta)*ZH(h1,5))*(CONJG(ZU(Sup1,1))*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,3))*ZU(Sum1,3) + CONJG(ZU(Sup1,5)) 
     -          *ZU(Sum1,5)) + 6*abslambda*EPhilPhis*
     -          (vS*Cos(beta)*ZH(h1,1) - vS*Sin(beta)*ZH(h1,2) + 
     -          (0,1)*(vS*ZH(h1,4) + 
     -          v*Cos(beta)*((0,1)*ZH(h1,3) + ZH(h1,6))))*
     -          (CONJG(ZU(Sup1,2))*Yu(1)*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,4))*Yu(2)*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,6))*Yu(3)*ZU(Sum1,5)) - 
     -          (6*abslambda*(vS*Cos(beta)*ZH(h1,1) - 
     -          vS*Sin(beta)*ZH(h1,2) - (0,1)*vS*ZH(h1,4) - 
     -          v*Cos(beta)*(ZH(h1,3) + (0,1)*ZH(h1,6)))*
     -          (CONJG(ZU(Sup1,1))*Yu(1)*ZU(Sum1,2) + 
     -          CONJG(ZU(Sup1,3))*Yu(2)*ZU(Sum1,4) + 
     -          CONJG(ZU(Sup1,5))*Yu(3)*ZU(Sum1,6)))*CONJG(EPhilPhis)
     -          + (0,1)*g1**2*v*(Cos(beta)*ZH(h1,4) - 
     -          Sin(beta)*ZH(h1,5))*(CONJG(ZU(Sup1,1))*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,3))*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,5))*ZU(Sum1,5) - 
     -          4*(CONJG(ZU(Sup1,2))*ZU(Sum1,2) + 
     -          CONJG(ZU(Sup1,4))*ZU(Sum1,4) + 
     -          CONJG(ZU(Sup1,6))*ZU(Sum1,6))) - 
     -          (0,12)*v*Sin(beta)*ZH(h1,5)*
     -          (Yu(1)**2*(CONJG(ZU(Sup1,1))*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,2))*ZU(Sum1,2)) + 
     -          Yu(2)**2*(CONJG(ZU(Sup1,3))*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,4))*ZU(Sum1,4)) + 
     -          Yu(3)**2*(CONJG(ZU(Sup1,5))*ZU(Sum1,5) + 
     -          CONJG(ZU(Sup1,6))*ZU(Sum1,6))) - 
     -          (6*Sqrt(2d0)*(CONJG(Au(1))*CONJG(ZU(Sup1,2))*Yu(1)*
     -          (Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2) + 
     -          (0,1)*ZH(h1,5))*ZU(Sum1,1) + 
     -          CONJG(Au(2))*CONJG(ZU(Sup1,4))*Yu(2)*
     -          (Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2) + 
     -          (0,1)*ZH(h1,5))*ZU(Sum1,3) + 
     -          CONJG(Au(3))*CONJG(ZU(Sup1,6))*Yu(3)*
     -          (Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2) + 
     -          (0,1)*ZH(h1,5))*ZU(Sum1,5) - 
     -          EPhiu**2*(Sin(beta)*ZH(h1,1) + 
     -          Cos(beta)*ZH(h1,2) - (0,1)*ZH(h1,5))*
     -          (Au(1)*CONJG(ZU(Sup1,1))*Yu(1)*ZU(Sum1,2) + 
     -          Au(2)*CONJG(ZU(Sup1,3))*Yu(2)*ZU(Sum1,4) + 
     -          Au(3)*CONJG(ZU(Sup1,5))*Yu(3)*ZU(Sum1,6))))
     -          *CONJG(EPhiu))/12.
            end do
         end do
      end do


      do h1=1,6,1
         ChSvSvconj(h1)= (0,-0.25)*(g1**2 + g2**2) *v*
     -                   (Cos(beta)*ZH(h1,4) - Sin(beta)*ZH(h1,5))
      end do
     
      
      do Hm1=1,2,1
         do Hp1=1,2,1
            do h1=1,6,1
               ChHmHp(Hm1,Hp1,h1)=(absAlambda*abslambda*
     -           ((-ZH(h1,3) - (0,1)*ZH(h1,6))*ZP(Hm1,2)*ZP(Hp1,1) + 
     -          EPhiIl**2*(ZH(h1,3) - (0,1)*ZH(h1,6))*ZP(Hm1,1)*
     -          ZP(Hp1,2)))/(Sqrt(2d0)*EPhiIl) + 
     -          abskappa*abslambda*
     -          ((vS*(ZH(h1,3) - (0,1)*ZH(h1,6))*ZP(Hm1,2)*ZP(Hp1,1))/
     -          EPhi - EPhi*vS*(ZH(h1,3) + (0,1)*ZH(h1,6))*
     -          ZP(Hm1,1)*ZP(Hp1,2)) - 
     -          (0,0.25)*g1**2*v*(Cos(beta)*ZH(h1,4) - 
     -          Sin(beta)*ZH(h1,5))*
     -          (ZP(Hm1,1)*ZP(Hp1,1) - ZP(Hm1,2)*ZP(Hp1,2)) - 
     -          (0,0.25)*g2**2*v*(((Cos(beta)*ZH(h1,4) + 
     -          Sin(beta)*ZH(h1,5))*ZP(Hm1,1) + 
     -          ((0,1)*ZH(h1,2) + Sin(beta)*ZH(h1,4) + 
     -          Cos(beta)*ZH(h1,5))*ZP(Hm1,2))*ZP(Hp1,1) + 
     -          (((0,-1)*ZH(h1,2) + Sin(beta)*ZH(h1,4) + 
     -          Cos(beta)*ZH(h1,5))*ZP(Hm1,1) + 
     -          (Cos(beta)*ZH(h1,4) + Sin(beta)*ZH(h1,5))*
     -          ZP(Hm1,2))*ZP(Hp1,2)) + 
     -          (abslambda**2*(v*ZH(h1,2)*
     -          (-(ZP(Hm1,2)*ZP(Hp1,1)) + ZP(Hm1,1)*ZP(Hp1,2)) + 
     -          (0,1)*v*(Sin(beta)*ZH(h1,4) + Cos(beta)*ZH(h1,5))*
     -          (ZP(Hm1,2)*ZP(Hp1,1) + ZP(Hm1,1)*ZP(Hp1,2)) - 
     -          (0,2)*vS*ZH(h1,6)*
     -          (ZP(Hm1,1)*ZP(Hp1,1) + ZP(Hm1,2)*ZP(Hp1,2))))/2.
            end do
         end do
      end do
      

      
      do Sem1=1,6,1
        do h1=1,6,1
          do Sep1=1,6,1
             do h2=1,6,1
                ChhSemSep(Sem1,h1,Sep1,h2)=
     -          (0,-1)*(Ye(1)**2*(CONJG(ZE(Sem1,1))*ZE(Sep1,1) + 
     -          CONJG(ZE(Sem1,2))*ZE(Sep1,2)) + 
     -          Ye(2)**2*(CONJG(ZE(Sem1,3))*ZE(Sep1,3) + 
     -          CONJG(ZE(Sem1,4))*ZE(Sep1,4)) + 
     -          Ye(3)**2*(CONJG(ZE(Sem1,5))*ZE(Sep1,5) + 
     -          CONJG(ZE(Sem1,6))*ZE(Sep1,6)))*
     -          ((Cos(beta)*ZH(h1,1) - Sin(beta)*ZH(h1,2))*
     -          (Cos(beta)*ZH(h2,1) - Sin(beta)*ZH(h2,2)) + 
     -          ZH(h1,4)*ZH(h2,4)) + 
     -          (0,0.25)*g2**2*(CONJG(ZE(Sem1,1))*ZE(Sep1,1) + 
     -          CONJG(ZE(Sem1,3))*ZE(Sep1,3) + 
     -          CONJG(ZE(Sem1,5))*ZE(Sep1,5))*
     -          (-(Sin(2*beta)*(ZH(h1,2)*ZH(h2,1) + ZH(h1,1)*ZH(h2,2)))
     -          +Cos(2*beta)*(ZH(h1,1)*ZH(h2,1) - ZH(h1,2)*ZH(h2,2)) + 
     -          ZH(h1,4)*ZH(h2,4) - ZH(h1,5)*ZH(h2,5)) + 
     -          (0,0.25)*g1**2*(CONJG(ZE(Sem1,1))*ZE(Sep1,1) + 
     -          CONJG(ZE(Sem1,3))*ZE(Sep1,3) + 
     -          CONJG(ZE(Sem1,5))*ZE(Sep1,5) - 
     -          2*(CONJG(ZE(Sem1,2))*ZE(Sep1,2) + 
     -          CONJG(ZE(Sem1,4))*ZE(Sep1,4) + 
     -          CONJG(ZE(Sem1,6))*ZE(Sep1,6)))*
     -          (Sin(2*beta)*(ZH(h1,2)*ZH(h2,1) + ZH(h1,1)*ZH(h2,2)) + 
     -          Cos(2*beta)*(-(ZH(h1,1)*ZH(h2,1)) + ZH(h1,2)*ZH(h2,2))  
     -          - ZH(h1,4)*ZH(h2,4) + ZH(h1,5)*ZH(h2,5)) - 
     -          (0,0.5)*abslambda*EPhilPhisPhiu*
     -          (CONJG(ZE(Sem1,2))*Ye(1)*ZE(Sep1,1) + 
     -          CONJG(ZE(Sem1,4))*Ye(2)*ZE(Sep1,3) + 
     -          CONJG(ZE(Sem1,6))*Ye(3)*ZE(Sep1,5))*
     -          (ZH(h1,3)*(Sin(beta)*ZH(h2,1) + Cos(beta)*ZH(h2,2) - 
     -          (0,1)*ZH(h2,5)) - 
     -          (0,1)*ZH(h1,6)*(Sin(beta)*ZH(h2,1) + 
     -          Cos(beta)*ZH(h2,2) - (0,1)*ZH(h2,5)) + 
     -          (Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2) - 
     -          (0,1)*ZH(h1,5))*(ZH(h2,3) - (0,1)*ZH(h2,6))) + 
     -          (abslambda*(CONJG(ZE(Sem1,1))*Ye(1)*ZE(Sep1,2) + 
     -          CONJG(ZE(Sem1,3))*Ye(2)*ZE(Sep1,4) + 
     -          CONJG(ZE(Sem1,5))*Ye(3)*ZE(Sep1,6))*
     -          (ZH(h1,6)*(Sin(beta)*ZH(h2,1) + Cos(beta)*ZH(h2,2) + 
     -          (0,1)*ZH(h2,5)) + 
     -          ZH(h1,3)*((0,-1)*
     -          (Sin(beta)*ZH(h2,1) + Cos(beta)*ZH(h2,2)) + 
     -          ZH(h2,5)) + 
     -          ((0,-1)*(Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2)) + 
     -          ZH(h1,5))*(ZH(h2,3) + (0,1)*ZH(h2,6))))
     -          /(2.*EPhilPhisPhiu)
             end do
          end do
        end do
      end do
      


      do h1=1,6,1
        do h2=1,6,1
           ChhSvSvconj(h1,h2)=(0,0.25)*(g1**2 + g2**2)*
     -          (Sin(2*beta)*(ZH(h1,2)*ZH(h2,1) + ZH(h1,1)*ZH(h2,2)) + 
     -          Cos(2*beta)*(-(ZH(h1,1)*ZH(h2,1)) + ZH(h1,2)*ZH(h2,2))- 
     -          ZH(h1,4)*ZH(h2,4) + ZH(h1,5)*ZH(h2,5))
        end do
      end do
      

      
      do Hm1=1,2,1
        do h1=1,6,1
           do Hp1=1,2,1
             do h2=1,6,1
             ChhHmHp(Hm1,h1,Hp1,h2)= (4*abskappa*abslambda*
     -          ((((0,1)*ZH(h1,3) + ZH(h1,6))*
     -          (ZH(h2,3) - (0,1)*ZH(h2,6))*ZP(Hm1,2)*ZP(Hp1,1))/
     -          EPhi + (0,1)*EPhi*(ZH(h1,3) + (0,1)*ZH(h1,6))*
     -          (ZH(h2,3) + (0,1)*ZH(h2,6))*ZP(Hm1,1)*ZP(Hp1,2)) + 
     -          (0,1)*g1**2*(Sin(2*beta)*
     -          (ZH(h1,2)*ZH(h2,1) + ZH(h1,1)*ZH(h2,2)) + 
     -          Cos(2*beta)*(-(ZH(h1,1)*ZH(h2,1)) + 
     -          ZH(h1,2)*ZH(h2,2)) - ZH(h1,4)*ZH(h2,4) + 
     -          ZH(h1,5)*ZH(h2,5))*
     -          (ZP(Hm1,1)*ZP(Hp1,1) - ZP(Hm1,2)*ZP(Hp1,2)) + 
     -          g2**2*(-(Cos(beta)*
     -          (ZH(h1,5)*ZH(h2,1) - ZH(h1,4)*ZH(h2,2) - 
     -          ZH(h1,2)*ZH(h2,4) + ZH(h1,1)*ZH(h2,5))*
     -          (ZP(Hm1,2)*ZP(Hp1,1) - ZP(Hm1,1)*ZP(Hp1,2))) + 
     -          Sin(beta)*(ZH(h1,4)*ZH(h2,1) + ZH(h1,5)*ZH(h2,2) + 
     -          ZH(h1,1)*ZH(h2,4) + ZH(h1,2)*ZH(h2,5))*
     -          (ZP(Hm1,2)*ZP(Hp1,1) - ZP(Hm1,1)*ZP(Hp1,2)) - 
     -          (0,1)*Cos(2*beta)*
     -          (ZH(h1,2)*ZH(h2,1) + ZH(h1,1)*ZH(h2,2))*
     -          (ZP(Hm1,2)*ZP(Hp1,1) + ZP(Hm1,1)*ZP(Hp1,2)) - 
     -          (0,1)*Sin(2*beta)*
     -          (ZH(h1,1)*ZH(h2,1) - ZH(h1,2)*ZH(h2,2))*
     -          (ZP(Hm1,2)*ZP(Hp1,1) + ZP(Hm1,1)*ZP(Hp1,2)) - 
     -          (0,1)*(((ZH(h1,1)*ZH(h2,1) + ZH(h1,2)*ZH(h2,2) + 
     -          ZH(h1,4)*ZH(h2,4) + ZH(h1,5)*ZH(h2,5))*ZP(Hm1,1) + 
     -          (ZH(h1,5)*ZH(h2,4) + ZH(h1,4)*ZH(h2,5))*ZP(Hm1,2)
     -          )*ZP(Hp1,1) + 
     -          ((ZH(h1,5)*ZH(h2,4) + ZH(h1,4)*ZH(h2,5))*ZP(Hm1,1) + 
     -          (ZH(h1,1)*ZH(h2,1) + ZH(h1,2)*ZH(h2,2) + 
     -          ZH(h1,4)*ZH(h2,4) + ZH(h1,5)*ZH(h2,5))*
     -          ZP(Hm1,2))*ZP(Hp1,2))) + 2*abslambda**2*(Cos(beta)*
     -          (ZH(h1,5)*ZH(h2,1) - ZH(h1,4)*ZH(h2,2) - 
     -          ZH(h1,2)*ZH(h2,4) + ZH(h1,1)*ZH(h2,5))*
     -          (ZP(Hm1,2)*ZP(Hp1,1) - ZP(Hm1,1)*ZP(Hp1,2)) - 
     -          Sin(beta)*(ZH(h1,4)*ZH(h2,1) + ZH(h1,5)*ZH(h2,2) + 
     -          ZH(h1,1)*ZH(h2,4) + ZH(h1,2)*ZH(h2,5))*
     -          (ZP(Hm1,2)*ZP(Hp1,1) - ZP(Hm1,1)*ZP(Hp1,2)) + 
     -          (0,1)*Cos(2*beta)*
     -          (ZH(h1,2)*ZH(h2,1) + ZH(h1,1)*ZH(h2,2))*
     -          (ZP(Hm1,2)*ZP(Hp1,1) + ZP(Hm1,1)*ZP(Hp1,2)) + 
     -          (0,1)*Sin(2*beta)*
     -          (ZH(h1,1)*ZH(h2,1) - ZH(h1,2)*ZH(h2,2))*
     -          (ZP(Hm1,2)*ZP(Hp1,1) + ZP(Hm1,1)*ZP(Hp1,2)) - 
     -          (0,1)*(-((ZH(h1,5)*ZH(h2,4) + ZH(h1,4)*ZH(h2,5))*
     -          (ZP(Hm1,2)*ZP(Hp1,1) + ZP(Hm1,1)*ZP(Hp1,2))) + 
     -          2*ZH(h1,3)*ZH(h2,3)*
     -          (ZP(Hm1,1)*ZP(Hp1,1) + ZP(Hm1,2)*ZP(Hp1,2)) + 
     -          2*ZH(h1,6)*ZH(h2,6)*
     -          (ZP(Hm1,1)*ZP(Hp1,1) + ZP(Hm1,2)*ZP(Hp1,2)))))/4.
             end do
           end do
        end do
      end do
      

      
      do Sup1=1,6,1
         do h1=1,6,1
           do Sum1=1,6,1
              do h2=1,6,1
              ChhSupSum(Sup1,h1,Sum1,h2)=((0,3)*g2**2*(Sin(2*beta)*
     -          (ZH(h1,2)*ZH(h2,1) + ZH(h1,1)*ZH(h2,2)) + 
     -          Cos(2*beta)*(-(ZH(h1,1)*ZH(h2,1)) + 
     -          ZH(h1,2)*ZH(h2,2)) - ZH(h1,4)*ZH(h2,4) + 
     -          ZH(h1,5)*ZH(h2,5))*(CONJG(ZU(Sup1,1))*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,3))*ZU(Sum1,3) +
     -          CONJG(ZU(Sup1,5))*ZU(Sum1,5)) + (6*abslambda*(ZH(h1,6)*
     -          (-(Cos(beta)*ZH(h2,1)) + Sin(beta)*ZH(h2,2) + 
     -          (0,1)*ZH(h2,4)) + ZH(h1,3)*((0,1)*Cos(beta)*ZH(h2,1) - 
     -          (0,1)*Sin(beta)*ZH(h2,2) + ZH(h2,4)) + 
     -          ((0,1)*Cos(beta)*ZH(h1,1) - 
     -          (0,1)*Sin(beta)*ZH(h1,2) + ZH(h1,4))*
     -          (ZH(h2,3) + (0,1)*ZH(h2,6)))*
     -          (CONJG(ZU(Sup1,1))*Yu(1)*ZU(Sum1,2) + 
     -          CONJG(ZU(Sup1,3))*Yu(2)*ZU(Sum1,4) + 
     -          CONJG(ZU(Sup1,5))*Yu(3)*ZU(Sum1,6)))/EPhilPhis + 
     -          (0,1)*g1**2*(-(Sin(2*beta)*
     -          (ZH(h1,2)*ZH(h2,1) + ZH(h1,1)*ZH(h2,2))) + 
     -          Cos(2*beta)*(ZH(h1,1)*ZH(h2,1) - ZH(h1,2)*ZH(h2,2)) + 
     -          ZH(h1,4)*ZH(h2,4) - ZH(h1,5)*ZH(h2,5))*
     -          (CONJG(ZU(Sup1,1))*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,3))*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,5))*ZU(Sum1,5) - 
     -          4*(CONJG(ZU(Sup1,2))*ZU(Sum1,2) + 
     -          CONJG(ZU(Sup1,4))*ZU(Sum1,4) + 
     -          CONJG(ZU(Sup1,6))*ZU(Sum1,6))) - 
     -          (0,6)*(abslambda*EPhilPhis*(ZH(h1,3) - (0,1)*ZH(h1,6))*
     -          (-(Cos(beta)*ZH(h2,1)) + Sin(beta)*ZH(h2,2))*
     -          (CONJG(ZU(Sup1,2))*Yu(1)*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,4))*Yu(2)*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,6))*Yu(3)*ZU(Sum1,5)) - 
     -          (0,1)*abslambda*EPhilPhis*ZH(h1,4)*ZH(h2,3)*
     -          (CONJG(ZU(Sup1,2))*Yu(1)*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,4))*Yu(2)*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,6))*Yu(3)*ZU(Sum1,5)) - 
     -          (0,1)*abslambda*EPhilPhis*ZH(h1,3)*ZH(h2,4)*
     -          (CONJG(ZU(Sup1,2))*Yu(1)*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,4))*Yu(2)*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,6))*Yu(3)*ZU(Sum1,5)) - 
     -          abslambda*EPhilPhis*ZH(h1,6)*ZH(h2,4)*
     -          (CONJG(ZU(Sup1,2))*Yu(1)*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,4))*Yu(2)*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,6))*Yu(3)*ZU(Sum1,5)) + 
     -          abslambda*EPhilPhis*
     -          (-(Cos(beta)*ZH(h1,1)) + Sin(beta)*ZH(h1,2))*
     -          (ZH(h2,3) - (0,1)*ZH(h2,6))*
     -          (CONJG(ZU(Sup1,2))*Yu(1)*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,4))*Yu(2)*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,6))*Yu(3)*ZU(Sum1,5)) - 
     -          abslambda*EPhilPhis*ZH(h1,4)*ZH(h2,6)*
     -          (CONJG(ZU(Sup1,2))*Yu(1)*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,4))*Yu(2)*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,6))*Yu(3)*ZU(Sum1,5)) + 
     -          2*(Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2))*
     -          (Sin(beta)*ZH(h2,1) + Cos(beta)*ZH(h2,2))*
     -          (CONJG(ZU(Sup1,1))*Yu(1)**2*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,3))*Yu(2)**2*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,5))*Yu(3)**2*ZU(Sum1,5)) + 
     -          2*ZH(h1,5)*ZH(h2,5)*
     -          (CONJG(ZU(Sup1,1))*Yu(1)**2*ZU(Sum1,1) + 
     -          CONJG(ZU(Sup1,3))*Yu(2)**2*ZU(Sum1,3) + 
     -          CONJG(ZU(Sup1,5))*Yu(3)**2*ZU(Sum1,5)) + 
     -          2*(Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2))*
     -          (Sin(beta)*ZH(h2,1) + Cos(beta)*ZH(h2,2))*
     -          (CONJG(ZU(Sup1,2))*Yu(1)**2*ZU(Sum1,2) + 
     -          CONJG(ZU(Sup1,4))*Yu(2)**2*ZU(Sum1,4) + 
     -          CONJG(ZU(Sup1,6))*Yu(3)**2*ZU(Sum1,6)) + 
     -          2*ZH(h1,5)*ZH(h2,5)*
     -          (CONJG(ZU(Sup1,2))*Yu(1)**2*ZU(Sum1,2) + 
     -          CONJG(ZU(Sup1,4))*Yu(2)**2*ZU(Sum1,4) + 
     -          CONJG(ZU(Sup1,6))*Yu(3)**2*ZU(Sum1,6))))/12.
              end do
           end do
         end do
      end do



      do Sdm1=1,6,1
         do h1=1,6,1
            do Sdp1=1,6,1
              do h2=1,6,1
               ChhSdmSdp(Sdm1,h1,Sdp1,h2)=
     -          (0,-1)*(Yd(1)**2*(CONJG(ZD(Sdm1,1))*ZD(Sdp1,1) + 
     -          CONJG(ZD(Sdm1,2))*ZD(Sdp1,2)) + 
     -          Yd(2)**2*(CONJG(ZD(Sdm1,3))*ZD(Sdp1,3) + 
     -          CONJG(ZD(Sdm1,4))*ZD(Sdp1,4)) + 
     -          Yd(3)**2*(CONJG(ZD(Sdm1,5))*ZD(Sdp1,5) + 
     -          CONJG(ZD(Sdm1,6))*ZD(Sdp1,6)))*
     -          ((Cos(beta)*ZH(h1,1) - Sin(beta)*ZH(h1,2))*
     -          (Cos(beta)*ZH(h2,1) - Sin(beta)*ZH(h2,2)) + 
     -          ZH(h1,4)*ZH(h2,4)) + 
     -          (0,0.25)*g2**2*(CONJG(ZD(Sdm1,1))*ZD(Sdp1,1) + 
     -          CONJG(ZD(Sdm1,3))*ZD(Sdp1,3) + 
     -          CONJG(ZD(Sdm1,5))*ZD(Sdp1,5))*
     -          (-(Sin(2*beta)*(ZH(h1,2)*ZH(h2,1) + ZH(h1,1)*ZH(h2,2))) 
     -          + Cos(2*beta)*(ZH(h1,1)*ZH(h2,1) - ZH(h1,2)*ZH(h2,2)) + 
     -          ZH(h1,4)*ZH(h2,4) - ZH(h1,5)*ZH(h2,5)) + 
     -          (0,1)/12d0*g1**2*(CONJG(ZD(Sdm1,1))*ZD(Sdp1,1) + 
     -          CONJG(ZD(Sdm1,3))*ZD(Sdp1,3) + 
     -          CONJG(ZD(Sdm1,5))*ZD(Sdp1,5) + 
     -          2*(CONJG(ZD(Sdm1,2))*ZD(Sdp1,2) + 
     -          CONJG(ZD(Sdm1,4))*ZD(Sdp1,4) + 
     -          CONJG(ZD(Sdm1,6))*ZD(Sdp1,6)))*
     -          (-(Sin(2*beta)*(ZH(h1,2)*ZH(h2,1) + ZH(h1,1)*ZH(h2,2)))
     -          + Cos(2*beta)*(ZH(h1,1)*ZH(h2,1) - ZH(h1,2)*ZH(h2,2)) + 
     -          ZH(h1,4)*ZH(h2,4) - ZH(h1,5)*ZH(h2,5)) - 
     -          (0,0.5)*abslambda*EPhilPhisPhiu*
     -          (CONJG(ZD(Sdm1,2))*Yd(1)*ZD(Sdp1,1) + 
     -          CONJG(ZD(Sdm1,4))*Yd(2)*ZD(Sdp1,3) + 
     -          CONJG(ZD(Sdm1,6))*Yd(3)*ZD(Sdp1,5))*
     -          (ZH(h1,3)*(Sin(beta)*ZH(h2,1) + Cos(beta)*ZH(h2,2) - 
     -          (0,1)*ZH(h2,5)) - (0,1)*ZH(h1,6)*(Sin(beta)*ZH(h2,1) + 
     -          Cos(beta)*ZH(h2,2) - (0,1)*ZH(h2,5)) + 
     -          (Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2) - 
     -          (0,1)*ZH(h1,5))*(ZH(h2,3) - (0,1)*ZH(h2,6))) + 
     -          (abslambda*(CONJG(ZD(Sdm1,1))*Yd(1)*ZD(Sdp1,2) + 
     -          CONJG(ZD(Sdm1,3))*Yd(2)*ZD(Sdp1,4) + 
     -          CONJG(ZD(Sdm1,5))*Yd(3)*ZD(Sdp1,6))*
     -          (ZH(h1,6)*(Sin(beta)*ZH(h2,1) + Cos(beta)*ZH(h2,2) + 
     -          (0,1)*ZH(h2,5)) + ZH(h1,3)*((0,-1)*
     -          (Sin(beta)*ZH(h2,1) + Cos(beta)*ZH(h2,2)) + ZH(h2,5)) + 
     -          ((0,-1)*(Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2)) + 
     -          ZH(h1,5))*(ZH(h2,3) + (0,1)*ZH(h2,6))))/
     -          (2.*EPhilPhisPhiu)
               end do
            end do
         end do
      end do
    
      
      do Chap1=1,2,1
        do Cham1=1,2,1
          do h1=1,6,1
             CLhChapCham(Chap1,Cham1,h1)=(-(g2*CONJG(UM(Cham1,1))*
     -          CONJG(UP(Chap1,2))*(Sin(beta)*ZH(h1,1) + 
     -          Cos(beta)*ZH(h1,2) + (0,1)*ZH(h1,5))) + 
     -          EPhiu*CONJG(UM(Cham1,2))*
     -          (g2*CONJG(UP(Chap1,1))*(Cos(beta)*ZH(h1,1) -  
     -          Sin(beta)*ZH(h1,2) -(0,1)*ZH(h1,4)) + 
     -          abslambda*EPhilPhis*CONJG(UP(Chap1,2))*
     -          (ZH(h1,3) - (0,1)*ZH(h1,6))))/(Sqrt(2d0)*EPhiu)
     
             CRhChapCham(Chap1,Cham1,h1)=(EPhilPhis*EPhiu*g2*
     -          UM(Chap1,1)*UP(Cham1,2)*(Sin(beta)*ZH(h1,1) +  
     -          Cos(beta)*ZH(h1,2) -(0,1)*ZH(h1,5)) - 
     -          UM(Chap1,2)*(EPhilPhis*g2*UP(Cham1,1)*
     -          (Cos(beta)*ZH(h1,1) - Sin(beta)*ZH(h1,2) + 
     -          (0,1)*ZH(h1,4)) + abslambda*UP(Cham1,2)*(ZH(h1,3) +
     -          (0,1)*ZH(h1,6))))/(Sqrt(2d0)*EPhilPhis)
          end do
        end do
      end do
     
      
      do Fep1=1,3,1
        do Fem1=1,3,1
          do h1=1,6,1
             if (Fep1.eq.Fem1) then
             CLhFepFem(Fep1,Fem1,h1)=-Ye(Fep1)*(Cos(beta)*ZH(h1,1) - 
     -                 Sin(beta)*ZH(h1,2) + (0,1)*ZH(h1,4))/Sqrt(2d0)
             CRhFepFem(Fep1,Fem1,h1)=Ye(Fep1)*(Cos(beta)*ZH(h1,1) - 
     -                 Sin(beta)*ZH(h1,2) - (0,1)*ZH(h1,4))/Sqrt(2d0)
             else
             CLhFepFem(Fep1,Fem1,h1)=0d0
             CRhFepFem(Fep1,Fem1,h1)=0d0
             end if
          end do
        end do
      end do 
      

      do Fdp1=1,3,1
        do Fdm1=1,3,1
          do h1=1,6,1
             if (Fdp1.eq.Fdm1) then
             CLhFdpFdm(Fdp1,Fdm1,h1)=-Yd(Fdp1)*(Cos(beta)*ZH(h1,1) - 
     -                 Sin(beta)*ZH(h1,2) + (0,1)*ZH(h1,4))/Sqrt(2d0)
             CRhFdpFdm(Fdp1,Fdm1,h1)=Yd(Fdp1)*(Cos(beta)*ZH(h1,1) - 
     -                 Sin(beta)*ZH(h1,2) - (0,1)*ZH(h1,4))/Sqrt(2d0)
             else
             CLhFdpFdm(Fdp1,Fdm1,h1)=0d0
             CRhFdpFdm(Fdp1,Fdm1,h1)=0d0
             end if
          end do
        end do
      end do 
     
      
      do Fum1=1,3,1
        do Fup1=1,3,1
          do h1=1,6,1
             if (Fum1.eq.Fup1) then
             CLhFumFup(Fum1,Fup1,h1)=Yu(Fum1)*(Sin(beta)*ZH(h1,1) + 
     -                Cos(beta)*ZH(h1,2) - (0,1)*ZH(h1,5))/Sqrt(2d0)
             CRhFumFup(Fum1,Fup1,h1)=-Yu(Fum1)*(Sin(beta)*ZH(h1,1) + 
     -                Cos(beta)*ZH(h1,2) + (0,1)*ZH(h1,5))/Sqrt(2d0)
             else
             CLhFumFup(Fum1,Fup1,h1)=0d0
             CRhFumFup(Fum1,Fup1,h1)=0d0
             end if
          end do
        end do
      end do 
     
      
      do h1=1,6,1
        ChU3U3(h1)=-(g2*MW*(ZH(h1,1) + (0,1)*(Cos(beta)*ZH(h1,4) + 
     -               Sin(beta)*ZH(h1,5))))/2.
      end do
     
      
      do h1=1,6,1
        ChU2U2(h1)=(0,-0.5)*MZ*(CThetaW*g2 + g1*SThetaW)*
     -             (Cos(beta)*ZH(h1,4) + Sin(beta)*ZH(h1,5))
      end do
     
      
      do h1=1,6,1
        ChU4U4(h1)=(g2*MW*(ZH(h1,1) - (0,1)*(Cos(beta)*ZH(h1,4)
     -             + Sin(beta)*ZH(h1,5))))/2.
      end do
     
     
      do l=1,6,1
      do m=1,6,1
      do n=1,6,1
      do i=1,6,1
      do j=1,6,1
      do k=1,6,1
              p(l, m, n, i, j, k)=
     -        ZH(i,n)*ZH(j,m)*ZH(k,l) + ZH(i,m)*ZH(j,n)*ZH(k,l) + 
     -        ZH(i,n)*ZH(j,l)*ZH(k,m) + ZH(i,l)*ZH(j,n)*ZH(k,m) + 
     -        ZH(i,m)*ZH(j,l)*ZH(k,n) + ZH(i,l)*ZH(j,m)*ZH(k,n)
      end do
      end do
      end do
      end do
      end do
      end do


      do h1=1,6,1
       do h2=1,6,1
         do h3=1,6,1
          Chhh(h1,h2,h3)=-(0,1d0)/24d0*
     -    (24*abskappa**2*vS*(p(6,3,3,h1,h2,h3) + p(6,6,6,h1,h2,h3)) + 
     -    12*abslambda**2*v*Cos(beta)*(Cos(beta)**2*
     -    p(4,2,2,h1,h2,h3) + p(4,3,3,h1,h2,h3) + p(4,5,5,h1,h2,h3) +
     -    p(4,6,6,h1,h2,h3) + p(4,1,1,h1,h2,h3)*Sin(beta)**2) + 
     -    3*g1**2*v*(Cos(beta)*(Cos(2*beta)*
     -    (p(4,1,1,h1,h2,h3) - p(4,2,2,h1,h2,h3)) + p(4,4,4,h1,h2,h3)  
     -    - p(4,5,5,h1,h2,h3)) + (-4*Cos(beta)**2*p(4,2,1,h1,h2,h3) + 
     -    Cos(2*beta)*(-p(5,1,1,h1,h2,h3) + p(5,2,2,h1,h2,h3)) - 
     -    p(5,4,4,h1,h2,h3) + p(5,5,5,h1,h2,h3))*Sin(beta) + 
     -    4*Cos(beta)*p(5,2,1,h1,h2,h3)*Sin(beta)**2) + 3*g2**2*v*
     -    (Cos(beta)*(Cos(2*beta)*(p(4,1,1,h1,h2,h3) -  
     -    p(4,2,2,h1,h2,h3)) + p(4,4,4,h1,h2,h3) - p(4,5,5,h1,h2,h3))  
     -    + (-4*Cos(beta)**2*p(4,2,1,h1,h2,h3) + 
     -    Cos(2*beta)*(-p(5,1,1,h1,h2,h3) + p(5,2,2,h1,h2,h3)) - 
     -    p(5,4,4,h1,h2,h3) + p(5,5,5,h1,h2,h3))*Sin(beta) + 
     -    4*Cos(beta)*p(5,2,1,h1,h2,h3)*Sin(beta)**2) + 
     -    12*abslambda**2*(vS*
     -    (p(6,1,1,h1,h2,h3) + p(6,2,2,h1,h2,h3) + p(6,4,4,h1,h2,h3) 
     -    + p(6,5,5,h1,h2,h3)) + v*Sin(beta)*(Cos(beta)**2*
     -    (2*p(4,2,1,h1,h2,h3) + p(5,1,1,h1,h2,h3)) + p(5,3,3,h1,h2,h3)
     -    + p(5,4,4,h1,h2,h3) + p(5,6,6,h1,h2,h3) + p(5,2,2,h1,h2,h3)
     -    *Sin(beta)**2 - p(5,2,1,h1,h2,h3)*Sin(2*beta))) + 
     -    12*abskappa*abslambda*(v*(Cos(phi)*(Cos(beta)*
     -    (p(5,3,3,h1,h2,h3) - p(5,6,6,h1,h2,h3)) - 2*p(6,3,2,h1,h2,h3) 
     -    + (p(4,3,3,h1,h2,h3) - p(4,6,6,h1,h2,h3))*Sin(beta)) - 
     -    (p(2,3,3,h1,h2,h3) - p(2,6,6,h1,h2,h3) + 
     -    2*Cos(beta)*p(6,5,3,h1,h2,h3) + 2*p(6,4,3,h1,h2,h3)* 
     -    Sin(beta))*Sin(phi)) - vS*(2*Cos(phi)*(Cos(2*beta)* 
     -    p(6,2,1,h1,h2,h3) + p(6,5,4,h1,h2,h3) + 
     -    (p(4,3,1,h1,h2,h3) + p(5,3,2,h1,h2,h3))*Sin(beta) + 
     -    Cos(beta)*(p(4,3,2,h1,h2,h3) - p(5,3,1,h1,h2,h3) + 
     -    (p(6,1,1,h1,h2,h3) - p(6,2,2,h1,h2,h3))*Sin(beta)))
     -    + (2*Cos(2*beta)*p(3,2,1,h1,h2,h3) + 2*p(5,4,3,h1,h2,h3) + 
     -    2*Cos(beta)*(-p(6,4,2,h1,h2,h3) + p(6,5,1,h1,h2,h3)) - 
     -    2*(p(6,4,1,h1,h2,h3) + p(6,5,2,h1,h2,h3))*Sin(beta) + 
     -    (p(3,1,1,h1,h2,h3) - p(3,2,2,h1,h2,h3))*Sin(2*beta))* 
     -    Sin(phi))) - 4*Sqrt(2d0)*absAkappa*abskappa*
     -    (3*Cos(phiIk)*p(6,3,3,h1,h2,h3)- Cos(phiIk)*p(6,6,6,h1,h2,h3) 
     -    - (p(3,3,3,h1,h2,h3) - 3*p(3,6,6,h1,h2,h3))*Sin(phiIk)) + 
     -    6*Sqrt(2d0)*absAlambda*abslambda*(Cos(phiIl)*(2*Cos(beta)*
     -    (p(4,3,2,h1,h2,h3) - p(5,3,1,h1,h2,h3)) - 
     -    2*Cos(2*beta)*p(6,2,1,h1,h2,h3) - 2*p(6,5,4,h1,h2,h3) + 
     -    2*(p(4,3,1,h1,h2,h3) + p(5,3,2,h1,h2,h3))*Sin(beta) + 
     -    (-p(6,1,1,h1,h2,h3) + p(6,2,2,h1,h2,h3))*Sin(2*beta)) + 
     -    2*(Cos(2*beta)*p(3,2,1,h1,h2,h3) + p(5,4,3,h1,h2,h3) + 
     -    Cos(beta)*(p(6,4,2,h1,h2,h3) - p(6,5,1,h1,h2,h3)) + 
     -    (Cos(beta)*(p(3,1,1,h1,h2,h3) - p(3,2,2,h1,h2,h3)) + 
     -    p(6,4,1,h1,h2,h3) + p(6,5,2,h1,h2,h3))*Sin(beta))*
     -    Sin(phiIl)))
         end do
       end do
      end do
     
     
      do h1=1,6,1
      do h2=1,6,1
      do h3=1,6,1
      do h4=1,6,1
         Chhhh(h1,h2,h3,h4)=
     -   ((0,-16)*abskappa**2*(OP(3,3,3,3,h1,h2,h3,h4) + 
     -   2*OP(6,6,3,3,h1,h2,h3,h4) + OP(6,6,6,6,h1,h2,h3,h4)) - 
     -   (0,1)*g1**2*((1 + Cos(4*beta))*OP(1,1,1,1,h1,h2,h3,h4) + 
     -   (2 -6*Cos(4*beta))*OP(2,2,1,1,h1,h2,h3,h4) + (1 + Cos(4*beta))
     -   *OP(2,2,2,2,h1,h2,h3,h4) + 2*(OP(4,4,4,4,h1,h2,h3,h4) + 
     -   2*Cos(2*beta)*(OP(4,4,1,1,h1,h2,h3,h4) - 
     -   OP(4,4,2,2,h1,h2,h3,h4) - OP(5,5,1,1,h1,h2,h3,h4) + 
     -   OP(5,5,2,2,h1,h2,h3,h4)) - 2*OP(5,5,4,4,h1,h2,h3,h4) + 
     -   OP(5,5,5,5,h1,h2,h3,h4) + 4*(Cos(2*beta)*
     -   (-OP(1,1,1,2,h1,h2,h3,h4) + OP(2,2,2,1,h1,h2,h3,h4)) - 
     -   OP(4,4,2,1,h1,h2,h3,h4) + OP(5,5,2,1,h1,h2,h3,h4))* 
     -   Sin(2*beta))) - (0,1)*g2**2*((1 + Cos(4*beta))*
     -   OP(1,1,1,1,h1,h2,h3,h4) + (2 - 6*Cos(4*beta))*
     -   OP(2,2,1,1,h1,h2,h3,h4) + (1 + Cos(4*beta))*
     -   OP(2,2,2,2,h1,h2,h3,h4) + 2*(OP(4,4,4,4,h1,h2,h3,h4) + 
     -   2*Cos(2*beta)*(OP(4,4,1,1,h1,h2,h3,h4) - 
     -   OP(4,4,2,2,h1,h2,h3,h4) - OP(5,5,1,1,h1,h2,h3,h4) + 
     -   OP(5,5,2,2,h1,h2,h3,h4)) - 2*OP(5,5,4,4,h1,h2,h3,h4) + 
     -   OP(5,5,5,5,h1,h2,h3,h4) + 4*(Cos(2*beta)*
     -   (-OP(1,1,1,2,h1,h2,h3,h4) + OP(2,2,2,1,h1,h2,h3,h4)) - 
     -   OP(4,4,2,1,h1,h2,h3,h4) + OP(5,5,2,1,h1,h2,h3,h4))*
     -   Sin(2*beta))) + (0,2)*abslambda**2*((-1 + Cos(4*beta))*
     -   OP(1,1,1,1,h1,h2,h3,h4) - 2*(1 + 3*Cos(4*beta))*
     -   OP(2,2,1,1,h1,h2,h3,h4) + (-1 + Cos(4*beta))*
     -   OP(2,2,2,2,h1,h2,h3,h4) -  8*(OP(3,3,1,1,h1,h2,h3,h4) +
     -   OP(3,3,2,2,h1,h2,h3,h4) + OP(4,4,3,3,h1,h2,h3,h4) + 
     -   Cos(beta)**2*(OP(4,4,2,2,h1,h2,h3,h4) + 
     -   OP(5,5,1,1,h1,h2,h3,h4)) + OP(5,5,3,3,h1,h2,h3,h4) + 
     -   OP(5,5,4,4,h1,h2,h3,h4) + OP(6,6,1,1,h1,h2,h3,h4) + 
     -   OP(6,6,2,2,h1,h2,h3,h4) + OP(6,6,4,4,h1,h2,h3,h4) + 
     -   OP(6,6,5,5,h1,h2,h3,h4) + (OP(4,4,1,1,h1,h2,h3,h4)
     -    + OP(5,5,2,2,h1,h2,h3,h4))* Sin(beta)**2 + 
     -   (OP(4,4,2,1,h1,h2,h3,h4) - OP(5,5,2,1,h1,h2,h3,h4))*
     -   Sin(2*beta)) + 4*(-OP(1,1,1,2,h1,h2,h3,h4) +
     -   OP(2,2,2,1,h1,h2,h3,h4))*Sin(4*beta)) +
     -   16*abskappa*abslambda*((0,-1)*Cos(phi)*
     -   (2*OP(3,3,5,4,h1,h2,h3,h4) + 4*Cos(beta)*
     -   (-OP(6,4,3,2,h1,h2,h3,h4) + OP(6,5,3,1,h1,h2,h3,h4)) + 
     -   2*Cos(2*beta)*(OP(3,3,2,1,h1,h2,h3,h4) - 
     -   OP(6,6,2,1,h1,h2,h3,h4)) - 2*OP(6,6,5,4,h1,h2,h3,h4) - 
     -   4*(OP(6,4,3,1,h1,h2,h3,h4) + OP(6,5,3,2,h1,h2,h3,h4))*
     -   Sin(beta) + (OP(3,3,1,1,h1,h2,h3,h4) - 
     -   OP(3,3,2,2,h1,h2,h3,h4) - OP(6,6,1,1,h1,h2,h3,h4) + 
     -   OP(6,6,2,2,h1,h2,h3,h4))*Sin(2*beta)) + 
     -   (0,2)*(2*Cos(2*beta)*OP(6,3,2,1,h1,h2,h3,h4) + 
     -   2*OP(6,5,4,3,h1,h2,h3,h4) + Cos(beta)*(OP(3,3,4,2,h1,h2,h3,h4) 
     -   - OP(3,3,5,1,h1,h2,h3,h4) - OP(6,6,4,2,h1,h2,h3,h4) + 
     -   OP(6,6,5,1,h1,h2,h3,h4)) + 
     -   (2*Cos(beta)*(OP(1,1,6,3,h1,h2,h3,h4) - 
     -   OP(2,2,6,3,h1,h2,h3,h4)) + OP(3,3,4,1,h1,h2,h3,h4) + 
     -   OP(3,3,5,2,h1,h2,h3,h4) - OP(6,6,4,1,h1,h2,h3,h4) - 
     -   OP(6,6,5,2,h1,h2,h3,h4))*Sin(beta))*Sin(phi)))/64d0
      end do
      end do
      end do
      end do
     
      
      do Chi1=1,5,1
      do Chi2=1,5,1
      do h1=1,6,1
      CLhChiChi(Chi1,Chi2,h1)=
     -     ((Cos(beta)*ZH(h1,1) - Sin(beta)*ZH(h1,2) - (0,1)*ZH(h1,4))*
     -     (CONJG(NN(Chi1,3))*(-(g1*CONJG(NN(Chi2,1))) + 
     -     g2*CONJG(NN(Chi2,2))) + (-(g1*CONJG(NN(Chi1,1))) +
     -     g2*CONJG(NN(Chi1,2)))*CONJG(NN(Chi2,3))) + 
     -     ((Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2) + (0,1)*ZH(h1,5))*
     -     (CONJG(NN(Chi1,4))*(-(g1*CONJG(NN(Chi2,1))) + 
     -     g2*CONJG(NN(Chi2,2))) + (-(g1*CONJG(NN(Chi1,1))) +
     -     g2*CONJG(NN(Chi1,2)))*CONJG(NN(Chi2,4))))/EPhiu -
     -     Sqrt(2d0)*EPhiu*lambda*(Sin(beta)*ZH(h1,1) + 
     -     Cos(beta)*ZH(h1,2) - (0,1)*ZH(h1,5))*
     -     (CONJG(NN(Chi1,5))*CONJG(NN(Chi2,3)) + 
     -     CONJG(NN(Chi1,3))*CONJG(NN(Chi2,5))) + 
     -     Sqrt(2d0)*lambda*(Cos(beta)*ZH(h1,1) - Sin(beta)*ZH(h1,2) +
     -     (0,1)*ZH(h1,4))*(CONJG(NN(Chi1,5))*CONJG(NN(Chi2,4)) + 
     -     CONJG(NN(Chi1,4))*CONJG(NN(Chi2,5))) - 
     -     Sqrt(2d0)*EPhis*(ZH(h1,3) - (0,1)*ZH(h1,6))*
     -     (lambda*CONJG(NN(Chi1,4))*CONJG(NN(Chi2,3)) +
     -     lambda*CONJG(NN(Chi1,3))*CONJG(NN(Chi2,4)) - 
     -     2*kappa*CONJG(NN(Chi1,5))*CONJG(NN(Chi2,5))))/2.
      end do
      end do
      end do
      
      
      do Chi1=1,5,1
      do Chi2=1,5,1
      do h1=1,6,1
      CRhChiChi(Chi1,Chi2,h1)=
     -     ((Cos(beta)*ZH(h1,1) - Sin(beta)*ZH(h1,2) + (0,1)*ZH(h1,4))*
     -     (NN(Chi1,3)*(g1*NN(Chi2,1) - g2*NN(Chi2,2)) + 
     -     (g1*NN(Chi1,1) - g2*NN(Chi1,2))*NN(Chi2,3)))/2. + (EPhiu*
     -     (Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2) - (0,1)*ZH(h1,5))*
     -     (NN(Chi1,4)*(g1*NN(Chi2,1) - g2*NN(Chi2,2)) + 
     -     (g1*NN(Chi1,1) - g2*NN(Chi1,2))*NN(Chi2,4)))/2. + 
     -     (clambda*(Sin(beta)*ZH(h1,1) + Cos(beta)*ZH(h1,2) + 
     -     (0,1)*ZH(h1,5))*(NN(Chi1,5)*NN(Chi2,3) +
     -     NN(Chi1,3)*NN(Chi2,5)))/(Sqrt(2d0)*EPhiu) - 
     -     (clambda*(Cos(beta)*ZH(h1,1) - Sin(beta)*ZH(h1,2) - 
     -     (0,1)*ZH(h1,4))*(NN(Chi1,5)*NN(Chi2,4) +
     -     NN(Chi1,4)*NN(Chi2,5)))/Sqrt(2d0) + ((clambda*(ZH(h1,3) +
     -     (0,1)*ZH(h1,6))*(NN(Chi1,4)*NN(Chi2,3) + 
     -     NN(Chi1,3)*NN(Chi2,4)))/Sqrt(2d0) - 
     -     Sqrt(2d0)*ckappa*(ZH(h1,3) + (0,1)*ZH(h1,6))*
     -     NN(Chi1,5)*NN(Chi2,5))/EPhis
      end do
      end do
      end do
     
     
      do Sv1=1,3,1
      do Sv2=1,3,1
      do Hm1=1,2,1
      do Hp1=1,2,1
      
      if (Sv1.EQ.Sv2) then
      CHmHpSvSv(Sv1,Hm1,Sv2,Hp1)=
     -     (0,-0.25)*(4*Ye(Sv2)**2*ZP(Hm1,1)*ZP(Hp1,1) + 
     -     (g1**2 - g2**2)*(ZP(Hm1,1)*ZP(Hp1,1) - ZP(Hm1,2)*ZP(Hp1,2)))
      else
      CHmHpSvSv(Sv1,Hm1,Sv2,Hp1)=0d0
      end if
      
      end do
      end do
      end do
      end do
     
      
      
      do Sdm1=1,6,1
      do Sdp1=1,6,1
      do Hm1=1,2,1
      do Hp1=1,2,1
        CHmHpSdmSdp(Sdm1,Hm1,Sdp1,Hp1)=
     -       (0,1)/12d0*(((g1**2 - 3*g2**2)*
     -       (CONJG(ZD(Sdm1,1))*ZD(Sdp1,1) + CONJG(ZD(Sdm1,3))*
     -       ZD(Sdp1,3) + CONJG(ZD(Sdm1,5))*ZD(Sdp1,5)) + 
     -       2*(CONJG(ZD(Sdm1,2))*(g1**2 - 6*Yd(1)**2)*ZD(Sdp1,2) + 
     -       CONJG(ZD(Sdm1,4))*(g1**2 - 6*Yd(2)**2)*ZD(Sdp1,4) + 
     -       CONJG(ZD(Sdm1,6))*(g1**2 - 6*Yd(3)**2)*ZD(Sdp1,6)))*
     -       ZP(Hm1,1)*ZP(Hp1,1) - ((g1**2 - 3*g2**2)*
     -       (CONJG(ZD(Sdm1,1))*ZD(Sdp1,1) + CONJG(ZD(Sdm1,3))
     -       *ZD(Sdp1,3) + CONJG(ZD(Sdm1,5))*ZD(Sdp1,5)) + 
     -       2*(6*(CONJG(ZD(Sdm1,1))*Yu(1)**2*ZD(Sdp1,1) + 
     -       CONJG(ZD(Sdm1,3))*Yu(2)**2*ZD(Sdp1,3) +CONJG(ZD(Sdm1,5))
     -       *Yu(3)**2*ZD(Sdp1,5)) + g1**2*(CONJG(ZD(Sdm1,2))*
     -       ZD(Sdp1,2) + CONJG(ZD(Sdm1,4))*ZD(Sdp1,4) + 
     -       CONJG(ZD(Sdm1,6))*ZD(Sdp1,6))))*ZP(Hm1,2)*ZP(Hp1,2))
      end do
      end do
      end do
      end do
     
      
      do Sup1=1,6,1
      do Sum1=1,6,1
      do Hm1=1,2,1
      do Hp1=1,2,1
        CHmHpSupSum(Sup1,Hm1,Sum1,Hp1)=
     -       (0,1)/12d0*(-(ZP(Hm1,2)*ZP(Hp1,2)*((g1**2 + 3*g2**2)*
     -       (CONJG(ZU(Sup1,1))*ZU(Sum1,1) +CONJG(ZU(Sup1,3))*
     -       ZU(Sum1,3) + CONJG(ZU(Sup1,5))*ZU(Sum1,5)) - 
     -       4*g1**2*(CONJG(ZU(Sup1,2))*ZU(Sum1,2) +CONJG(ZU(Sup1,4))
     -       *ZU(Sum1,4) + CONJG(ZU(Sup1,6))*ZU(Sum1,6)) + 
     -       12*(CONJG(ZU(Sup1,2))*Yu(1)**2*ZU(Sum1,2) + 
     -       CONJG(ZU(Sup1,4))*Yu(2)**2*ZU(Sum1,4) + CONJG(ZU(Sup1,6))*
     -       Yu(3)**2*ZU(Sum1,6)))) + ZP(Hm1,1)*ZP(Hp1,1)*
     -       ((g1**2 + 3*g2**2)*(CONJG(ZU(Sup1,1))*ZU(Sum1,1) + 
     -       CONJG(ZU(Sup1,3))*ZU(Sum1,3) + CONJG(ZU(Sup1,5))*
     -       ZU(Sum1,5)) - 4*(3*(CONJG(ZU(Sup1,1))*Yd(1)**2*ZU(Sum1,1)+ 
     -       CONJG(ZU(Sup1,3))*Yd(2)**2*ZU(Sum1,3) + CONJG(ZU(Sup1,5))
     -       *Yd(3)**2*ZU(Sum1,5))+g1**2*(CONJG(ZU(Sup1,2))*ZU(Sum1,2)+ 
     -       CONJG(ZU(Sup1,4))*ZU(Sum1,4)+CONJG(ZU(Sup1,6))*
     -       ZU(Sum1,6)))))
      end do
      end do
      end do
      end do
     
      
      
      do Sem1=1,6,1
      do Sep1=1,6,1
      do Hm1=1,2,1
      do Hp1=1,2,1
         CHmHpSemSep(Sem1,Hm1,Sep1,Hp1)=
     -       (0,0.25)*(-(((g1**2 + g2**2)*
     -       (CONJG(ZE(Sem1,1))*ZE(Sep1,1) + CONJG(ZE(Sem1,3))*
     -       ZE(Sep1,3) + CONJG(ZE(Sem1,5))*ZE(Sep1,5)) - 
     -       2*g1**2*(CONJG(ZE(Sem1,2))*ZE(Sep1,2) + CONJG(ZE(Sem1,4))
     -       *ZE(Sep1,4) + CONJG(ZE(Sem1,6))*ZE(Sep1,6)) + 
     -       4*(CONJG(ZE(Sem1,2))*Ye(1)**2*ZE(Sep1,2)+CONJG(ZE(Sem1,4))
     -       *Ye(2)**2*ZE(Sep1,4) + CONJG(ZE(Sem1,6))*Ye(3)**2*
     -       ZE(Sep1,6)))*ZP(Hm1,1)*ZP(Hp1,1)) + ((g1**2 + g2**2)*
     -       (CONJG(ZE(Sem1,1))*ZE(Sep1,1)+CONJG(ZE(Sem1,3))*ZE(Sep1,3) 
     -       +CONJG(ZE(Sem1,5))*ZE(Sep1,5)) - 2*g1**2*(CONJG(ZE(Sem1,2))
     -       *ZE(Sep1,2) + CONJG(ZE(Sem1,4))*ZE(Sep1,4) + 
     -       CONJG(ZE(Sem1,6))*ZE(Sep1,6)))*ZP(Hm1,2)*ZP(Hp1,2))
      end do
      end do
      end do
      end do

      
      
      do Sep1=1,6,1
      do Sv1=1,3,1
      do Hm1=1,2,1
       CHmSepSv(Sv1,Sep1,Hm1)=
     -         ((0,0.25)*(4*EPhisPhiu*Ae(Sv1)*Ye(Sv1)*ZE(Sep1,2*Sv1)*
     -         ZP(Hm1,1) + Sqrt(2d0)*(2*vS*CONJG(lambda)*Ye(Sv1)*
     -         ZE(Sep1,2*Sv1)*ZP(Hm1,2)-EPhisPhiu*ZE(Sep1,-1 + 2*Sv1)*
     -         (vd*(g2**2 - 2*Ye(Sv1)**2)*ZP(Hm1,1) + 
     -         g2**2*vu*ZP(Hm1,2)))))/EPhisPhiu
      end do
      end do
      end do

      
      
      do Sem1=1,6,1
      do Sv1=1,3,1
      do Hp1=1,2,1
         CHpSemSv(Sem1,Sv1,Hp1)=
     -       (0,0.25)*(-(Sqrt(2d0)*g2**2*CONJG(ZE(Sem1,-1 + 2*Sv1))* 
     -       (vd*ZP(Hp1,1) + vu*ZP(Hp1,2))) + 2*(2*CONJG(Ae(Sv1))*
     -       CONJG(ZE(Sem1,2*Sv1))*Ye(Sv1)*ZP(Hp1,1) + 
     -       Sqrt(2d0)*(vd*CONJG(ZE(Sem1,-1 + 2*Sv1))*Ye(Sv1)**2*
     -       ZP(Hp1,1) + EPhisPhiu*lambda*vS*CONJG(ZE(Sem1,2*Sv1))*
     -       Ye(Sv1)*ZP(Hp1,2))))
      end do
      end do
      end do

       
      
      do Sup1=1,6,1
      do Sdp1=1,6,1
      do Hm1=1,2,1
         CHmSupSdp(Sup1,Sdp1,Hm1)=
     -       ((0,1)*EPhis*lambda*vS*(CONJG(ZU(Sup1,2))*Yu(1)*
     -       ZD(Sdp1,1) + CONJG(ZU(Sup1,4))*Yu(2)*ZD(Sdp1,3) + 
     -       CONJG(ZU(Sup1,6))*Yu(3)*ZD(Sdp1,5))*ZP(Hm1,1))/
     -       Sqrt(2d0) - (0,0.25)*(CONJG(ZU(Sup1,1))*
     -       (-4*Ad(1)*Yd(1)*ZD(Sdp1,2)*ZP(Hm1,1) + 
     -       Sqrt(2d0)*ZD(Sdp1,1)*(vd*(g2**2 - 2*Yd(1)**2)
     -       *ZP(Hm1,1) + vu*(g2**2 - 2*Yu(1)**2)*ZP(Hm1,2))) + 
     -       CONJG(ZU(Sup1,3))*(-4*Ad(2)*Yd(2)*ZD(Sdp1,4)*ZP(Hm1,1) + 
     -       Sqrt(2d0)*ZD(Sdp1,3)*(vd*(g2**2 - 2*Yd(2)**2)*ZP(Hm1,1) + 
     -       vu*(g2**2 - 2*Yu(2)**2)*ZP(Hm1,2))) + CONJG(ZU(Sup1,5))*
     -       (-4*Ad(3)*Yd(3)*ZD(Sdp1,6)*ZP(Hm1,1)+Sqrt(2d0)*ZD(Sdp1,5)*
     -       (vd*(g2**2 - 2*Yd(3)**2)*ZP(Hm1,1)+vu*(g2**2 - 2*Yu(3)**2)
     -       *ZP(Hm1,2))))+(((0,1)*vS*CONJG(lambda)*(CONJG(ZU(Sup1,1))
     -       *Yd(1)*ZD(Sdp1,2) + CONJG(ZU(Sup1,3))*Yd(2)*ZD(Sdp1,4) + 
     -       CONJG(ZU(Sup1,5))*Yd(3)*ZD(Sdp1,6))*ZP(Hm1,2))/Sqrt(2d0)
     -        + EPhis*(((0,1)*CONJG(ZU(Sup1,2))*Yd(1)*Yu(1)*ZD(Sdp1,2)*
     -       (vu*ZP(Hm1,1) + vd*ZP(Hm1,2)))/Sqrt(2d0) + (0,0.5)*
     -       (2*CONJG(Au(1))*CONJG(ZU(Sup1,2))*Yu(1)*ZD(Sdp1,1)* 
     -       ZP(Hm1,2) +CONJG(ZU(Sup1,4))*Yu(2)*(2*CONJG(Au(2))*
     -       ZD(Sdp1,3)*ZP(Hm1,2) + Sqrt(2d0)*Yd(2)*ZD(Sdp1,4)*
     -       (vu*ZP(Hm1,1) + vd*ZP(Hm1,2)))+CONJG(ZU(Sup1,6))*Yu(3)*
     -       (2*CONJG(Au(3))*ZD(Sdp1,5)*ZP(Hm1,2) + Sqrt(2d0)*Yd(3)*
     -       ZD(Sdp1,6)*(vu*ZP(Hm1,1) + vd*ZP(Hm1,2))))))/EPhisPhiu
      end do
      end do
      end do

      
      do Sdm1=1,6,1
      do Sum1=1,6,1
      do Hp1=1,2,1
         CHpSumSdm(Sdm1,Sum1,Hp1)=
     -       (0,-0.25)*(-4*CONJG(Ad(1))*CONJG(ZD(Sdm1,2))*Yd(1)*
     -       ZP(Hp1,1)*ZU(Sum1,1) +CONJG(ZD(Sdm1,1))*(Sqrt(2d0)*vd*
     -       (g2**2 - 2*Yd(1)**2)*ZP(Hp1,1)*ZU(Sum1,1) + ZP(Hp1,2)*
     -       (Sqrt(2d0)*vu*(g2**2 - 2*Yu(1)**2)*ZU(Sum1,1) - 4*EPhiu*
     -       Au(1)*Yu(1)*ZU(Sum1,2)))+ Sqrt(2d0)*g2**2*vd*
     -       CONJG(ZD(Sdm1,3))*ZP(Hp1,1)*ZU(Sum1,3) - 4*CONJG(Ad(2))*
     -       CONJG(ZD(Sdm1,4))*Yd(2)*ZP(Hp1,1)*ZU(Sum1,3)-2*Sqrt(2d0)*
     -       vd*CONJG(ZD(Sdm1,3))*Yd(2)**2*ZP(Hp1,1)*ZU(Sum1,3) + 
     -       Sqrt(2d0)*g2**2*vu*CONJG(ZD(Sdm1,3))*ZP(Hp1,2)*ZU(Sum1,3)- 
     -       2*Sqrt(2d0)*vu*CONJG(ZD(Sdm1,3))*Yu(2)**2*ZP(Hp1,2)*
     -       ZU(Sum1,3)+Sqrt(2d0)*g2**2*vd*CONJG(ZD(Sdm1,5))*ZP(Hp1,1)*
     -       ZU(Sum1,5)-4*CONJG(Ad(3))*CONJG(ZD(Sdm1,6))*Yd(3)*
     -       ZP(Hp1,1)*ZU(Sum1,5)-2*Sqrt(2d0)*vd*CONJG(ZD(Sdm1,5))*
     -       Yd(3)**2*ZP(Hp1,1)*ZU(Sum1,5)+Sqrt(2d0)*g2**2*vu*
     -       CONJG(ZD(Sdm1,5))*ZP(Hp1,2)*ZU(Sum1,5)-2*Sqrt(2d0)*vu*
     -       CONJG(ZD(Sdm1,5))*Yu(3)**2*ZP(Hp1,2)*ZU(Sum1,5)) + 
     -       ((0,1)*vS*CONJG(lambda)*ZP(Hp1,1)*(CONJG(ZD(Sdm1,1))*Yu(1)
     -       *ZU(Sum1,2) + CONJG(ZD(Sdm1,3))*Yu(2)*ZU(Sum1,4) + 
     -       CONJG(ZD(Sdm1,5))*Yu(3)*ZU(Sum1,6)))/(Sqrt(2d0)*EPhis) + 
     -       EPhisPhiu*(((0,1)*lambda*vS*ZP(Hp1,2)*(CONJG(ZD(Sdm1,2))*
     -       Yd(1)*ZU(Sum1,1) + CONJG(ZD(Sdm1,4))*Yd(2)*ZU(Sum1,3) + 
     -       CONJG(ZD(Sdm1,6))*Yd(3)*ZU(Sum1,5)))/Sqrt(2d0) + 
     -       ((0,0.5)*(Sqrt(2d0)*CONJG(ZD(Sdm1,2))*Yd(1)*Yu(1)*
     -       (vu*ZP(Hp1,1) + vd*ZP(Hp1,2))*ZU(Sum1,2) + 2*Au(2)*
     -       CONJG(ZD(Sdm1,3))*Yu(2)*ZP(Hp1,2)*ZU(Sum1,4) + 
     -       Sqrt(2d0)*CONJG(ZD(Sdm1,4))*Yd(2)*Yu(2)*(vu*ZP(Hp1,1) 
     -       + vd*ZP(Hp1,2))*ZU(Sum1,4)+Sqrt(2d0)*vu*CONJG(ZD(Sdm1,6))
     -       *Yd(3)*Yu(3)*ZP(Hp1,1)*ZU(Sum1,6)+2*Au(3)*
     -       CONJG(ZD(Sdm1,5))*Yu(3)*ZP(Hp1,2)*ZU(Sum1,6) + 
     -       Sqrt(2d0)*vd*CONJG(ZD(Sdm1,6))*Yd(3)*Yu(3)*
     -       ZP(Hp1,2)*ZU(Sum1,6)))/EPhis)
      end do
      end do
      end do
     
      
      do Chi1=1,5,1
      do Cham1=1,2,1
      do Hp1=1,2,1
        CLHpChiCha(Chi1,Cham1,Hp1)=
     -      (0,1)*(-(g2*CONJG(NN(Chi1,3))*CONJG(UM(Cham1,1))*ZP(Hp1,1))
     -      + CONJG(UM(Cham1,2))*((g1*CONJG(NN(Chi1,1))*ZP(Hp1,1))/
     -      Sqrt(2d0) + (g2*CONJG(NN(Chi1,2))*ZP(Hp1,1))/Sqrt(2d0) - 
     -      EPhiu*lambda*CONJG(NN(Chi1,5))*ZP(Hp1,2)))
     
        CRHpChiCha(Chi1,Cham1,Hp1)=
     -      (0,-0.5)*(2*CONJG(lambda)*NN(Chi1,5)*UP(Cham1,2)*ZP(Hp1,1) 
     -      + EPhiu*(2*g2*NN(Chi1,4)*UP(Cham1,1)+Sqrt(2d0)*
     -      (g1*NN(Chi1,1) + g2*NN(Chi1,2))*UP(Cham1,2))*ZP(Hp1,2))
      end do
      end do
      end do
      
      
      do Chi1=1,5,1
      do Chap1=1,2,1
      do Hm1=1,2,1
        CLHmChiCha(Chap1,Chi1,Hm1)=
     -       ((0,-0.5)*(2*EPhiu*lambda*CONJG(NN(Chi1,5))*
     -       CONJG(UP(Chap1,2))*ZP(Hm1,1) + 
     -       (2*g2*CONJG(NN(Chi1,4))*CONJG(UP(Chap1,1)) + 
     -       Sqrt(2d0)*(g1*CONJG(NN(Chi1,1)) +g2*CONJG(NN(Chi1,2)))*
     -       CONJG(UP(Chap1,2)))*ZP(Hm1,2)))/EPhiu

        CRHmChiCha(Chap1,Chi1,Hm1)=
     -      ((0,0.5)*(-2*CONJG(lambda)*ZP(Hm1,2)*NN(Chi1,5)*UM(Chap1,2)
     -      +EPhiu*ZP(Hm1,1)*(-2*g2*NN(Chi1,3)*UM(Chap1,1) + Sqrt(2d0)*
     -      (g1*NN(Chi1,1) + g2*NN(Chi1,2))*UM(Chap1,2))))/EPhiu
      end do
      end do
      end do
     
      
      return
      end
      
      
* ---------------------------------------------------------------------
      
      double precision function OP(i,j,k,l,h1,h2,h3,h4)
      
      implicit none
      
      integer i,j,k,l,h1,h2,h3,h4
      
      double precision ZH5(5,5),ZH(6,6)
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      
        OP=
     -  ZH(h1,l)*ZH(h2,k)*ZH(h3,j)*ZH(h4,i) + 
     -  ZH(h1,k)*ZH(h2,l)*ZH(h3,j)*ZH(h4,i) + 
     -  ZH(h1,l)*ZH(h2,j)*ZH(h3,k)*ZH(h4,i) + 
     -  ZH(h1,j)*ZH(h2,l)*ZH(h3,k)*ZH(h4,i) + 
     -  ZH(h1,k)*ZH(h2,j)*ZH(h3,l)*ZH(h4,i) + 
     -  ZH(h1,j)*ZH(h2,k)*ZH(h3,l)*ZH(h4,i) + 
     -  ZH(h1,l)*ZH(h2,k)*ZH(h3,i)*ZH(h4,j) + 
     -  ZH(h1,k)*ZH(h2,l)*ZH(h3,i)*ZH(h4,j) + 
     -  ZH(h1,l)*ZH(h2,i)*ZH(h3,k)*ZH(h4,j) + 
     -  ZH(h1,i)*ZH(h2,l)*ZH(h3,k)*ZH(h4,j) + 
     -  ZH(h1,k)*ZH(h2,i)*ZH(h3,l)*ZH(h4,j) + 
     -  ZH(h1,i)*ZH(h2,k)*ZH(h3,l)*ZH(h4,j) + 
     -  ZH(h1,l)*ZH(h2,j)*ZH(h3,i)*ZH(h4,k) + 
     -  ZH(h1,j)*ZH(h2,l)*ZH(h3,i)*ZH(h4,k) + 
     -  ZH(h1,l)*ZH(h2,i)*ZH(h3,j)*ZH(h4,k) + 
     -  ZH(h1,i)*ZH(h2,l)*ZH(h3,j)*ZH(h4,k) + 
     -  ZH(h1,j)*ZH(h2,i)*ZH(h3,l)*ZH(h4,k) + 
     -  ZH(h1,i)*ZH(h2,j)*ZH(h3,l)*ZH(h4,k) + 
     -  ZH(h1,k)*ZH(h2,j)*ZH(h3,i)*ZH(h4,l) + 
     -  ZH(h1,j)*ZH(h2,k)*ZH(h3,i)*ZH(h4,l) + 
     -  ZH(h1,k)*ZH(h2,i)*ZH(h3,j)*ZH(h4,l) + 
     -  ZH(h1,i)*ZH(h2,k)*ZH(h3,j)*ZH(h4,l) + 
     -  ZH(h1,j)*ZH(h2,i)*ZH(h3,k)*ZH(h4,l) + 
     -  ZH(h1,i)*ZH(h2,j)*ZH(h3,k)*ZH(h4,l)
     
      return
      end
      
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccc running values for RGEs ccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine runpara(g1Qs,g2Qs,g3Qs,YtQs,YbQs,MHd2,MHu2,ms2)
      
      implicit none
      
      double precision MHd2,MHu2,ms2
      double precision g1Qs,g2Qs,g3Qs,YtQs,YbQs
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      double precision dMW2,dMZ2,dZe,dMHpm2,dthd,dthu,dths,dtad,dtas
      double precision MzDR,MwDR,eDR,cwDR,vDR,a_sDR
      double precision Mh(5),MHpm,Mcha(2),Mchi(5),MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                   ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      double precision Qs
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
     
      double precision ImAlambdaDR,ImAkappaDR,ReAkappaDR,ReAlambdaDR
      
      double precision absAlambdaDR,absAkappaDR,phiAlDR,phiAkDR
      
      double precision mtDR,mbDR,mtQs,mbQs,YuDR(3),YdDR(3),mbMSmb,
     &                 mtms,mbms
      double precision absAtDR,phiAtDR,mq2DR,mur2dR
      double precision absAuDR(3),phiAuDR(3),mq2temp(3),mur2temp(3)
      
      double precision arg
     
      double precision vu,vd,dv2loop
      
      double complex dmtfin,dmtdiv1,dYtfin,dYtdiv1
      double complex dMstfin(2,2),dMstdiv1(2,2),
     &               dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
     
      logical MHpmInput,CPcon,mQrun,onelswitch
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /DRpara/ ReAlambdaDR,ReAkappaDR,ImAlambdaDR,ImAkappaDR
      common /DRstop/ absAtDR,phiAtDR,mq2DR,mur2dR
      common/ dv2loop /dv2loop
      common/qcdcount/ dmtfin,dmtdiv1,dMstfin,dMstdiv1,dYtfin,dYtdiv1,
     &                 dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
      common /scale/ Qs
      
      call Counterterms(dMW2,dMZ2,dZe,dMHpm2,dthd,dthu,dths,dtad,dtas)
      
      MwDR=Sqrt(Mw**2+dMW2)
      MzDR=Sqrt(Mz**2+dMZ2)
      eDR=e+e*dZe

      cwDR=MwDR/MzDR
      g1Qs=eDR/cwDR
      g2Qs=eDR/Sqrt(1-cwDR**2)
      
      vDR=2*MwDR*Sqrt(1-cwDR**2)/eDR+dv2loop
      
      g3Qs=Sqrt(a_sDR(Qs)*4d0*pi)
     
      If(mQrun) then
      YtQs=mt*sqrt(2d0)/v/Sin(beta)
      YbQs=mb*sqrt(2d0)/v/Cos(beta)
      else
      call MStoDR(mtMS(Qs),mtDR,Qs)
      call MStoDR(mbMS(Qs),mbDR,Qs)
      YuDR(3) = mtDR*sqrt(2d0)/v/Sin(beta)
      YdDR(3) = mbDR*sqrt(2d0)/v/Cos(beta)
      
      absAuDR=absAu
      absAuDR(3)=absAtDR
      phiAuDR=phiAu
      phiAuDR(3)=phiAtDR
      mq2temp=mq2
      mq2temp(3)=mq2DR
      mur2temp=mur2
      mur2temp(3)=mur2DR
      call dsquarkmasses(g1,g2,v,beta,abslambda,vs,phil,phis,phiu,phiAd,
     &                   absAd,mq2temp,mdr2,YdDR,MSd,ZD)
          

      call usquarkmasses(g1,g2,v,beta,abslambda,vs,phil,phis,phiu,
     &                   phiAuDR,absAuDR,mq2temp,mur2temp,YuDR,MSu,ZU)
      call mtbeff(mtDR,mbDR,Qs,mtQS,mbQS)
      
      YtQs=mtQs*sqrt(2d0)/v/Sin(beta)
      YbQs=mbQs*sqrt(2d0)/v/Cos(beta)
      
      call dsquarkmasses(g1,g2,v,beta,abslambda,vs,phil,phis,phiu,phiAd,
     &                   absAd,mq2,mdr2,Yd,MSd,ZD)
          

      call usquarkmasses(g1,g2,v,beta,abslambda,vs,phil,phis,phiu,phiAu,
     &                   absAu,mq2,mur2,Yu,MSu,ZU)
      end if
      
      vu=vDR*Sin(beta)
      vd=vDR*Cos(beta)
      
      absAlambdaDR=Sqrt(ImAlambdaDR**2+ReAlambdaDR**2)
      phiAlDR=arg(ReAlambdaDR,ImAlambdaDR)
      absAkappaDR=Sqrt(ImAkappaDR**2+ReAkappaDR**2)
      phiAkDR=arg(ReAkappaDR,ImAkappaDR)
      
      MHd2=(-((g1Qs**2 + g2Qs**2)*vd**3) - 4*abslambda**2*vd*vs**2 + 
     .     (-4*abslambda**2 + g1Qs**2 + g2Qs**2)*vd*vu**2 + 
     .     4*abslambda*vs*vu*(abskappa*vs*Cos(phik-phil+2*Phis-Phiu)
     .     +Sqrt(2d0)*absAlambdaDR*Cos(phiAlDR+phil+Phis+Phiu)))/(8.*vd)
     
      MHu2=(-((g1Qs**2 + g2Qs**2)*vu**3) - 4*abslambda**2*vu*vs**2 + 
     .     (-4*abslambda**2 + g1Qs**2 + g2Qs**2)*vu*vd**2 + 
     .     4*abslambda*vs*vd*(abskappa*vs*Cos(phik-phil+2*Phis-Phiu)
     .     +Sqrt(2d0)*absAlambdaDR*Cos(phiAlDR+phil+Phis+Phiu)))/(8.*vu)
     
c MMM changed 10/8/2013
      ms2= -(vs*(2*abskappa**2*vs**2+abslambda**2*(vd**2+vu**2))
     .     +Sqrt(2d0)*absAkappaDR*abskappa*vs**2*
     .     Cos(phiAkDR+phik+3*Phis)- 
     .     abslambda*vd*vu*(2*abskappa*vs*Cos(phik-phil+2*Phis-Phiu)+
     .     Sqrt(2d0)*absAlambdaDR*Cos(phiAlDR+phil+Phis+Phiu)))/(2.*vs)
c end MMM changed 10/8/2013     
      
      return
      end
      
      
      double precision function a_sDRNM(Q)
      
      implicit none
      
      double precision Q,mtpole,b0,b1,mbMSmb,aQ
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      integer i
      double precision asDRMz
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /topoles/ mtpole,mbMSmb
      
      b1=0d0
      b0=3d0/4d0
     
      aQ=0d0
      asDRMz=1d0/(1d0/alphaS-1d0/4d0/pi)
      a_sDRNM=asDRMz/pi
    
      i=1
      do while((abs(aQ-a_sDRNM).gt.1d-12).AND.(i.le.100))
      aQ=a_sDRNM
      a_sDRNM=(alphaS/pi*b0)/
     -  (b0 + alphaS/pi*b1*log(alphaS/pi) - alphaS/pi*b1*log(aQ) - 
     -    alphaS/pi*b1*log(b0 + alphaS/pi*b1) + 
     -    alphaS/pi*b1*log(b0 + aQ*b1) + 
     -    alphaS/pi*b0**2*log(Q**2/Mz**2))
      i=i+1
      end do
      
      a_sDRNM=a_sDRNM*pi
      
      return
      end
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccc Loop Functions ccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

! one-point function
      double precision function A0(m2)
      
      implicit none
     
      
      double precision m2,mu2,Qs
      
      common /scale/ Qs
      
      mu2=Qs**2
      
      if(m2.eq.0) then
      A0=0d0
      else
      A0=m2-m2*Log(m2/mu2)
      end if
      
      return
      end

*----------------------------------------------------------------------
!two-point function
!taken from hdecay and slightly modified
      double complex function B0(s,m12t,m22t,mu2)

      implicit none 

      double precision s,mu2,m12,m22,sm,m12t,m22t
      double complex nlog,x1,x2,zkappa
      double precision pi
      
      pi=2d0*ASin(1d0)
      
      if(m12t.ge.m22t) then
      m12=m22t
      m22=m12t
      else
      m12=m12t
      m22=m22t
      end if
      
      sm=s - m22 + m12
      zkappa=sqrt(dcmplx(sm**2 - 4*s*(m12+(0d0,1d-50))))

      if (s.le.1D-10) then
         if (abs(m12-m22).le.10d-8) then
           if (m12.le.1D-10) then
            B0=0.D0
           else
            B0=-dlog(m12/mu2) 
           endif
         else
          if (m22.le.1D-10) then
           if (m12.le.1D-10) then
            B0=0.D0
           else
            B0=1.D0 - m12/(m12-m22)*dlog(m12/mu2)
           endif
          else
           if (m12.le.1D-10) then
            B0=1.D0 + m22/(m12-m22)*dlog(m22/mu2) 
           else
            B0=1.D0 - m12/(m12-m22)*dlog(m12/mu2)
     &                 + m22/(m12-m22)*dlog(m22/mu2) 
           endif
          endif
         endif
      else 
         if ((m12.le.1D-10).and.(m22.le.1D-10)) then 
            B0=2.D0 - dlog(s/mu2) + pi *(0d0,1d0)
         elseif ((abs(m12-s).le.1D-10).and.(m22.le.1D-10)) then 
            B0=2.D0 - dlog(m12/mu2)
         elseif ((abs(m22-s).le.1D-10).and.(m12.le.1D-10)) then 
            B0=2.D0 - dlog(m22/mu2)
         elseif (m12.le.1D-10) then
            B0=2.D0 - (s-m22)/s*nlog( (m22-s)/m22 )
     &                 - dlog(m22/mu2)
         elseif (m22.le.1D-10) then
            B0=2.D0 - (s-m12)/s*nlog( (m12-s)/m12 ) 
     &                 - dlog(m12/mu2)
         else
            x1=dcmplx( (s-m22+m12+zkappa)/(2.D0*s) )
            x2=dcmplx( (s-m22+m12-zkappa)/(2.D0*s) )
            B0= 2.D0+ dlog(mu2/m22) 
     &                       + x1*cdlog(1.D0-1.D0/x1) 
     &                       + x2*cdlog(1.D0-1.D0/x2)
            B0=CONJG(B0)
         endif
      endif 

      return
      end


*----------------------------------------------------------------------
! function needed in B0
      double complex function nlog(x)
      
      implicit none
      
      double precision x,pi
      
      pi=2d0*ASin(1d0)
      
      if(x.GE.0) then
      nlog=dlog(x)
      else
      nlog=dlog(abs(x))-(0d0,1d0)*pi
      end if
      
      return 
      end
     


* ---------------------------------------------------------------------
!Deriavte of the two-point function
!taken from hdecay and slightly modified
      double complex function DB0(s,m12,m22)
      
      implicit none 

      double precision s,m12,m22 
      double complex zkappa,x1,x2
     

      zkappa=cdsqrt(dcmplx(s**2+m12**2+m22**2
     &                    -2.D0*(s*m12+s*m22+m12*m22)+(0d0,1d-50)))

      if (s.le.1D-10) then
         if (abs(m12-m22).le.1d-10) then
            DB0=1.D0/(6.D0*m12)
         elseif(m12.le.1d-10) then
            DB0=1d0/2d0/m22
         elseif(m22.le.1d-10) then
            DB0=1d0/2d0/m12
         else
            DB0=( (m12+m22)/2.D0 
     &        - m12*m22/(m12-m22)*dlog(m12/m22) )/(m12-m22)**2 
         endif
      elseif ((abs(s-m12).le.1d-10).and.(m22.le.1.D-10)) then 
         DB0=( -1.D0 + dlog(m12)/2.D0 )/m12
      elseif ((abs(s-m22).le.1d-10).and.(m12.le.1.D-10)) then 
         DB0=( -1.D0 + dlog(m22)/2.D0 )/m22
      else 
         x1=dcmplx( (s-m22+m12+zkappa)/(2.D0*s) )
         x2=dcmplx( (s-m22+m12-zkappa)/(2.D0*s) )
         DB0=( -1.D0 + ( x1*(1.D0-x1)*cdlog(1.D0-1.D0/x1)
     &                     - x2*(1.D0-x2)*cdlog(1.D0-1.D0/x2) )  
     &                                                  /(x1-x2) )/s
      endif 

      return
      end
      
      
* ---------------------------------------------------------------------
c nhung change here to correct a mistake 22/3/2019
      double complex function B1(s,m12,m22,mu2)
      
      implicit none
      
      double precision s,m12,m22,mu2
      double complex B0
      double precision A0k
      
      if(s.le.1d-10) then
        if(abs(m12-m22).le.10d-8) then
          B1 = -B0(s,m12,m22,mu2)/2d0
        else
          if(m12.le.1d-10) then
            B1 = -(B0(s,m12,m22,mu2)/2d0-0.25d0)
          elseif(m22.le.1d-10) then
            B1 = -(B0(s,m12,m22,mu2)/2d0+0.25d0)
          else
            B1 = -(1d0-Log(m22/mu2)+m12**2/(m12-m22)**2*Log(m22/m12)
     .           +(m12+m22)/(m12-m22)/2d0)/2d0
          endif
        endif
      else
        B1 = 1d0/s/2d0*(A0k(m12,mu2)-A0k(m22,mu2)-(s-m22+m12)*B0(s,m12,m22,mu2))
      end if
      
      return
      end
c nhung change here to correct a mistake 22/3/2019    
      
* ---------------------------------------------------------------------

      double complex function DB1(p2,m12,m22,mu2)
      
      implicit none
      
      double precision p2,m12,m22,mu2
      double complex B0,DB0,B1
      
      if(p2.ge.1d-10) then
      DB1 = 1/2d0/p2*(-2*B1(p2,m12,m22,mu2)-B0(p2,m12,m22,mu2)
     .      -(m12-m22+p2)*DB0(p2,m12,m22))
      else
         if(abs(m12-m22).le.1d-10) then
         DB1=-DB0(p2,m12,m22)/2d0
         else
         DB1=0d0 !p2=0 is not implemented!!!!!!!!!!!!
      !The case p2=0 is not needed here, because in all expressions
      ! only 'p2*DB1' appears...
         end if
      end if
      
      return
      end
      
      
      double complex function B00(p2,m12,m22,mu2)
      
      implicit none
      
      double precision p2,m12,m22,mu2
      double complex B0,B1
      double precision A0
      
      B00=(A0(m22)+2d0*m12*B0(p2,m12,m22,mu2)+
     .      (p2-m22+m12)*B1(p2,m12,m22,mu2)+m12+m22-p2/3d0)/6d0

      return
      end

*----------------------------------------------------------------------

      double complex function DB00(p2,m12,m22,mu2)
      
      implicit none
      
      double precision p2,m12,m22,mu2
      double complex DB0,DB1,B1
      
      DB00=(2d0*m12*DB0(p2,m12,m22)+(p2-m22+m12)*DB1(p2,m12,m22,mu2)
     .      + B1(p2,m12,m22,mu2)-1d0/3d0)/6d0
      
      return 
      end
      
     
* ---------------------------------------------------------------------

      double complex function B0i(bbn,p2,m12,m22)
      
      implicit none
      
      double precision p2,m12,m22
      double precision Qs,mu2
      double complex B0,B1,B00,DB0,DB1,DB00
      
      integer bbn
      integer bb0,bb1,bb00,dbb0,dbb1,dbb00
      
      common /scale/ Qs
      common /funcID/ bb0,bb1,bb00,dbb0,dbb1,dbb00
      
      
      mu2=Qs**2
      
      if(bbn.eq.bb0) then
      B0i=B0(p2,m12,m22,mu2)
      elseif(bbn.eq.bb1) then
      B0i=B1(p2,m12,m22,mu2)
      elseif(bbn.eq.bb00) then
      B0i=B00(p2,m12,m22,mu2)
      elseif(bbn.eq.dbb0) then
      B0i=DB0(p2,m12,m22)
      elseif(bbn.eq.dbb1) then
      B0i=DB1(p2,m12,m22,mu2)
      elseif(bbn.eq.dbb00) then
      B0i=DB00(p2,m12,m22,mu2)
      else
      print*, 'Unknown function ID in loop function...'
      B0i=0d0
      end if

      return
      end
      
      
* ---------------------------------------------------------------------
           
      double precision function ReB0i(bbn,p2,m12,m22)
      
      implicit none

      integer bbn
      double precision p2,m12,m22
      double complex B0i
      
      ReB0i=Realpart(B0i(bbn,p2,m12,m22))
      
      return
      end
      

***********************************************************************
* Routines for the diagonalization of complex matrices (by Thomas Hahn)
*(arXiv:physics/0607103)
***********************************************************************

* HEigensystem.F
* diagonalization of a Hermitian n-by-n matrix using the Jacobi algorithm
* code adapted from the "Handbook" routines for complex A
* (Wilkinson, Reinsch: Handbook for Automatic Computation, p. 202)
* this file is part of the Diag library
* last modified 9 Aug 11 th

#include "diag-f.h"


************************************************************************
** HEigensystem diagonalizes a Hermitian n-by-n matrix.
** Input: n, A = n-by-n matrix, Hermitian
** (only the upper triangle of A needs to be filled).
** Output: d = vector of eigenvalues, U = transformation matrix
** these fulfill diag(d) = U A U^+ = U A U^-1 with U unitary.

        subroutine HEigensystem(n, A,ldA, d, U,ldU, sort)
        implicit none
        integer n, ldA, ldU, sort
        Complex A(ldA,*), U(ldU,*)
        Real d(*)

        integer p, q, j
        Real red, off, thresh
        Real t, delta, invc, s
        Complex x, y, Apq
        Real ev(2,MAXDIM)

        integer sweep
        common /nsweeps/ sweep

        Real Sq
        Complex c
        Sq(c) = Re(c*Conjugate(c))

        if( n .gt. MAXDIM ) then
          print *, "Dimension too large"
          d(1) = -999
          return
        endif

        do p = 1, n
          ev(1,p) = 0
          ev(2,p) = Re(A(p,p))
          d(p) = ev(2,p)
        enddo

        do p = 1, n
          do q = 1, n
            U(q,p) = 0
          enddo
          U(p,p) = 1
        enddo

        red = .04D0/n**4

        do sweep = 1, 50
          off = 0
          do q = 2, n
            do p = 1, q - 1
              off = off + Sq(A(p,q))
            enddo
          enddo
          if( .not. off .gt. SYM_EPS ) goto 1

          thresh = 0
          if( sweep .lt. 4 ) thresh = off*red

          do q = 2, n
            do p = 1, q - 1
              Apq = A(p,q)
              off = Sq(Apq)
              if( sweep .gt. 4 .and. off .lt.
     &              SYM_EPS*(ev(2,p)**2 + ev(2,q)**2) ) then
                A(p,q) = 0
              else if( off .gt. thresh ) then
                t = .5D0*(ev(2,p) - ev(2,q))
                t = 1/(t + sign(sqrt(t**2 + off), t))

                delta = t*off
                ev(1,p) = ev(1,p) + delta
                ev(2,p) = d(p) + ev(1,p)
                ev(1,q) = ev(1,q) - delta
                ev(2,q) = d(q) + ev(1,q)

                invc = sqrt(delta*t + 1)
                s = t/invc
                t = delta/(invc + 1)

                do j = 1, p - 1
                  x = A(j,p)
                  y = A(j,q)
                  A(j,p) = x + s*(Conjugate(Apq)*y - t*x)
                  A(j,q) = y - s*(Apq*x + t*y)
                enddo

                do j = p + 1, q - 1
                  x = A(p,j)
                  y = A(j,q)
                  A(p,j) = x + s*(Apq*Conjugate(y) - t*x)
                  A(j,q) = y - s*(Apq*Conjugate(x) + t*y)
                enddo

                do j = q + 1, n
                  x = A(p,j)
                  y = A(q,j)
                  A(p,j) = x + s*(Apq*y - t*x)
                  A(q,j) = y - s*(Conjugate(Apq)*x + t*y)
                enddo

                A(p,q) = 0

                do j = 1, n
                  x = U(p,j)
                  y = U(q,j)
                  U(p,j) = x + s*(Apq*y - t*x)
                  U(q,j) = y - s*(Conjugate(Apq)*x + t*y)
                enddo
              endif
            enddo
          enddo

          do p = 1, n
            ev(1,p) = 0
            d(p) = ev(2,p)
          enddo
        enddo

        print *, "Bad convergence in HEigensystem"

1        if( sort .eq. 0 ) return

* sort the eigenvalues

        do p = 1, n - 1
          j = p
          t = d(p)
          do q = p + 1, n
            if( sort*(t - d(q)) .gt. 0 ) then
              j = q
              t = d(q)
            endif
          enddo

          if( j .ne. p ) then
            d(j) = d(p)
            d(p) = t
            do q = 1, n
              x = U(p,q)
              U(p,q) = U(j,q)
              U(j,q) = x
            enddo
          endif
        enddo
        end

* SEigensystem.F
* diagonalization of a complex symmetric n-by-n matrix using
* the Jacobi algorithm
* code adapted from the "Handbook" routines for complex A
* (Wilkinson, Reinsch: Handbook for Automatic Computation, p. 202)
* this file is part of the Diag library
* last modified 9 Aug 11 th



************************************************************************
** SEigensystem diagonalizes a complex symmetric n-by-n matrix.
** Input: n, A = n-by-n matrix, complex symmetric
** (only the upper triangle of A needs to be filled).
** Output: d = vector of eigenvalues, U = transformation matrix
** these fulfill diag(d) = U A U^T = U A U^-1 with U U^T = 1.

        subroutine SEigensystem(n, A,ldA, d, U,ldU, sort)
        implicit none
        integer n, ldA, ldU, sort
        Complex A(ldA,*), U(ldU,*), d(*)

        integer p, q, j
        Real red, off, thresh
        Complex delta, t, invc, s
        Complex x, y
        Complex ev(2,MAXDIM)

        integer sweep
        common /nsweeps/ sweep

        Real Sq
        Complex c
        Sq(c) = Re(c*Conjugate(c))

        if( n .gt. MAXDIM ) then
          print *, "Dimension too large"
          d(1) = -999
          return
        endif

        do p = 1, n
          ev(1,p) = 0
          ev(2,p) = A(p,p)
          d(p) = ev(2,p)
        enddo

        do p = 1, n
          do q = 1, n
            U(q,p) = 0
          enddo
          U(p,p) = 1
        enddo

        red = .04D0/n**4

        do sweep = 1, 50
          off = 0
          do q = 2, n
            do p = 1, q - 1
              off = off + Sq(A(p,q))
            enddo
          enddo
          if( .not. off .gt. SYM_EPS ) goto 1

          thresh = 0
          if( sweep .lt. 4 ) thresh = off*red

          do q = 2, n
            do p = 1, q - 1
              delta = A(p,q)
              off = Sq(delta)
              if( sweep .gt. 4 .and. off .lt.
     &              SYM_EPS*(Sq(ev(2,p)) + Sq(ev(2,q))) ) then
                A(p,q) = 0
              else if( off .gt. thresh ) then
                x = .5D0*(ev(2,p) - ev(2,q))
                y = sqrt(x**2 + delta**2)
                t = x - y
                s = x + y
                if( Sq(t) .lt. Sq(s) ) t = s

                t = delta/t
                delta = delta*t
                ev(1,p) = ev(1,p) + delta
                ev(2,p) = d(p) + ev(1,p)
                ev(1,q) = ev(1,q) - delta
                ev(2,q) = d(q) + ev(1,q)

                invc = sqrt(t**2 + 1)
                s = t/invc
                t = t/(invc + 1)

                do j = 1, p - 1
                  x = A(j,p)
                  y = A(j,q)
                  A(j,p) = x + s*(y - t*x)
                  A(j,q) = y - s*(x + t*y)
                enddo

                do j = p + 1, q - 1
                  x = A(p,j)
                  y = A(j,q)
                  A(p,j) = x + s*(y - t*x)
                  A(j,q) = y - s*(x + t*y)
                enddo

                do j = q + 1, n
                  x = A(p,j)
                  y = A(q,j)
                  A(p,j) = x + s*(y - t*x)
                  A(q,j) = y - s*(x + t*y)
                enddo

                A(p,q) = 0

                do j = 1, n
                  x = U(p,j)
                  y = U(q,j)
                  U(p,j) = x + s*(y - t*x)
                  U(q,j) = y - s*(x + t*y)
                enddo
              endif
            enddo
          enddo

          do p = 1, n
            ev(1,p) = 0
            d(p) = ev(2,p)
          enddo
        enddo

        print *, "Bad convergence in SEigensystem"

1        if( sort .eq. 0 ) return

* sort the eigenvalues by their real part

        do p = 1, n - 1
          j = p
          t = d(p)
          do q = p + 1, n
            if( sort*(Re(t) - Re(d(q))) .gt. 0 ) then
              j = q
              t = d(q)
            endif
          enddo

          if( j .ne. p ) then
            d(j) = d(p)
            d(p) = t
            do q = 1, n
              x = U(p,q)
              U(p,q) = U(j,q)
              U(j,q) = x
            enddo
          endif
        enddo
        end

* SVD.F
* singular value decomposition of an m-by-n matrix
* this file is part of the Diag library
* last modified 9 Aug 11 th




************************************************************************
** SVD performs a singular value decomposition.
** Input: m, n, A = m-by-n matrix.
** Output: d = nm-vector of singular values,
** V = nm-by-m left transformation matrix,
** W = nm-by-n right transformation matrix, nm = min(m, n),
** these fulfill diag(d) = V^* A W^+.

        subroutine SVD(m, n, Ao,ldA, d, Vo,ldV, Wo,ldW, sort)
        implicit none
        integer m, n, ldA, ldV, ldW, sort
        Complex Ao(ldA,*), Vo(ldV,*), Wo(ldW,*)
        Real d(*)

        integer nx, nm, p, q, px, qx, j, rev, pi(MAXDIM)
        Real red, off, thresh
        Real t, dv, dw, xv, xw, invc
        Complex App, Apq, Aqp, Aqq
        Complex x, y, sv, sw, tv, tw, f
        Complex VW(MAXDIM,MAXDIM,0:2)

* note: for better cache efficiency, the Vx, Wx arrays
* contain the *transpose* of the transformation matrices
        Complex V(MAXDIM,MAXDIM)
        Complex W(MAXDIM,MAXDIM)
        Complex A(MAXDIM,MAXDIM)
        equivalence (VW(1,1,0), V)
        equivalence (VW(1,1,1), W)
        equivalence (VW(1,1,2), A)

        integer sweep
        common /nsweeps/ sweep

        Real Sq
        Complex c
        Sq(c) = Re(c*Conjugate(c))

        nx = max(m, n)

        if( nx .gt. MAXDIM ) then
          print *, "Dimension too large"
          d(1) = -999
          return
        endif

        do p = 1, nx
          do q = 1, nx
            V(q,p) = 0
            W(q,p) = 0
            A(q,p) = 0
          enddo
          V(p,p) = 1
          W(p,p) = 1
        enddo

        rev = ibits(m - n, 15, 1)
        if( rev .eq. 1 ) then
          do p = 1, n
            do q = 1, m
              A(p,q) = Ao(q,p)
            enddo
          enddo
        else
          do p = 1, n
            do q = 1, m
              A(q,p) = Ao(q,p)
            enddo
          enddo
        endif

        red = .01D0/nx**4

        do sweep = 1, 50
          off = 0
          do q = 2, nx
            do p = 1, q - 1
              off = off + Sq(A(p,q)) + Sq(A(q,p))
            enddo
          enddo
          if( .not. off .gt. EPS ) goto 1

          thresh = 0
          if( sweep .lt. 4 ) thresh = off*red

          do q = 2, nx
            do p = 1, q - 1
              px = p
              qx = q
              if( Sq(A(p,p)) + Sq(A(q,q)) .lt.
     &            Sq(A(p,q)) + Sq(A(q,p)) ) then
                px = q
                qx = p
              endif

              App = A(px,p)
              Aqq = A(qx,q)
              Apq = A(px,q)
              Aqp = A(qx,p)
              off = Sq(Apq) + Sq(Aqp)
              if( sweep .gt. 4 .and.
     &            off .lt. EPS*(Sq(App) + Sq(Aqq)) ) then
                A(px,q) = 0
                A(qx,p) = 0
              else if( off .gt. thresh ) then
                xv = Re((App - Aqq)*Conjugate(App + Aqq))
                xw = Re((Apq - Aqp)*Conjugate(Apq + Aqp))
                dv = .5D0*(xv + xw)
                dw = .5D0*(xv - xw)

                tv = Conjugate(App)*Aqp + Aqq*Conjugate(Apq)
                tw = Conjugate(App)*Apq + Aqq*Conjugate(Aqp)
c                t = sqrt(dv**2 + Sq(tv))
                t = sqrt(dw**2 + Sq(tw))

                xv = min(abs(dv + t), abs(dw + t))
                xw = min(abs(dv - t), abs(dw - t))
                if( xv + xw .gt. DBL_EPS ) then
                  t = sign(t, xv - xw)
                  tv = tv/(dv + t)
                  tw = tw/(dw + t)
                else
                  tv = 0
                  tw = Apq/App
                endif

                invc = sqrt(1 + Sq(tv))
                sv = tv/invc
                tv = tv/(invc + 1)

                invc = sqrt(1 + Sq(tw))
                sw = tw/invc
                tw = tw/(invc + 1)

                do j = 1, nx
                  x = A(j,p)
                  y = A(j,q)
                  A(j,p) = x + Conjugate(sw)*(y - tw*x)
                  A(j,q) = y - sw*(x + Conjugate(tw)*y)
                  x = A(px,j)
                  y = A(qx,j)
                  A(p,j) = x + Conjugate(sv)*(y - tv*x)
                  A(q,j) = y - sv*(x + Conjugate(tv)*y)
                enddo

                A(p,p) = invc*(App + Conjugate(sv)*(Aqp - tv*App))
                A(q,p) = 0
                A(p,q) = 0
                A(q,q) = invc*(Aqq - sv*(Apq + Conjugate(tv)*Aqq))

                do j = 1, nx
                  x = V(j,px)
                  y = V(j,qx)
                  V(j,p) = x + sv*(y - Conjugate(tv)*x)
                  V(j,q) = y - Conjugate(sv)*(x + tv*y)
                enddo

                do j = 1, nx
                  x = W(j,p)
                  y = W(j,q)
                  W(j,p) = x + sw*(y - Conjugate(tw)*x)
                  W(j,q) = y - Conjugate(sw)*(x + tw*y)
                enddo
                goto 2
              endif

              if( p .ne. px ) then
                do j = 1, nx
                  x = A(p,j)
                  A(p,j) = A(q,j)
                  A(q,j) = x
                enddo

                do j = 1, nx
                  x = V(j,p)
                  V(j,p) = V(j,q)
                  V(j,q) = x
                enddo
              endif

2              continue
            enddo
          enddo
        enddo

        print *, "Bad convergence in SVD"

1        continue

        nm = min(m, n)

* make the diagonal elements nonnegative

        do p = 1, nm
          d(p) = abs(A(p,p))
          if( d(p) .gt. DBL_EPS .and. d(p) .ne. Re(A(p,p)) ) then
            f = A(p,p)/d(p)
            do q = 1, nm
              W(q,p) = W(q,p)*f
            enddo
          endif
        enddo

* sort the singular values

        do p = 1, nm
          pi(p) = p
        enddo

        do p = 1, nm
          j = p
          t = d(p)
          if( sort .ne. 0 ) then
            do q = p + 1, nm
              if( sort*(t - d(q)) .gt. 0 ) then
                j = q
                t = d(q)
              endif
            enddo
          endif

          d(j) = d(p)
          d(p) = t

          q = pi(j)
          pi(j) = pi(p)

          do j = 1, m
            Vo(p,j) = VW(j,q,rev)
          enddo
          do j = 1, n
            Wo(p,j) = VW(j,q,1-rev)
          enddo
        enddo
        end

* TakagiFactor.F
* computes the Takagi factorization of a complex symmetric matrix
* code adapted from the "Handbook" routines
* (Wilkinson, Reinsch: Handbook for Automatic Computation, p. 202)
* this file is part of the Diag library
* last modified 9 Aug 11 th




************************************************************************
** TakagiFactor factorizes a complex symmetric n-by-n matrix
** Input: n, A = n-by-n matrix, complex symmetric
** (only the upper triangle of A needs to be filled).
** Output: d = vector of diagonal values, U = transformation matrix
** these fulfill diag(d) = U^* A U^+ with U unitary.

        subroutine TakagiFactor(n, A,ldA, d, U,ldU, sort)
        implicit none
        integer n, ldA, ldU, sort
        Complex A(ldA,*), U(ldU,*)
        Real d(*)

        integer p, q, j
        Real red, off, thresh
        Real sqp, sqq, t, invc
        Complex f, x, y
        Complex ev(2,MAXDIM)

        integer sweep
        common /nsweeps/ sweep

        Real Sq
        Complex c
        Sq(c) = Re(c*Conjugate(c))

        if( n .gt. MAXDIM ) then
          print *, "Dimension too large"
          d(1) = -999
          return
        endif

        do p = 1, n
          ev(1,p) = 0
          ev(2,p) = A(p,p)
        enddo

        do p = 1, n
          do q = 1, n
            U(q,p) = 0
          enddo
          U(p,p) = 1
        enddo

        red = .04D0/n**4

        do sweep = 1, 50
          off = 0
          do q = 2, n
            do p = 1, q - 1
              off = off + Sq(A(p,q))
            enddo
          enddo
          if( .not. off .gt. SYM_EPS ) goto 1

          thresh = 0
          if( sweep .lt. 4 ) thresh = off*red

          do q = 2, n
            do p = 1, q - 1
              off = Sq(A(p,q))
              sqp = Sq(ev(2,p))
              sqq = Sq(ev(2,q))
              if( sweep .gt. 4 .and.
     &            off .lt. SYM_EPS*(sqp + sqq) ) then
                A(p,q) = 0
              else if( off .gt. thresh ) then
                t = .5D0*abs(sqp - sqq)
                if( t .gt. 0 ) then
                  f = sign(1D0, sqp - sqq)*
     &              (ev(2,q)*Conjugate(A(p,q)) +
     &               Conjugate(ev(2,p))*A(p,q))
                else
                  f = 1
                  if( sqp .ne. 0 ) f = sqrt(ev(2,q)/ev(2,p))
                endif
                t = t + sqrt(t**2 + Sq(f))
                f = f/t

                ev(1,p) = ev(1,p) + A(p,q)*Conjugate(f)
                ev(2,p) = A(p,p) + ev(1,p)
                ev(1,q) = ev(1,q) - A(p,q)*f
                ev(2,q) = A(q,q) + ev(1,q)

                t = Sq(f)
                invc = sqrt(t + 1)
                f = f/invc
                t = t/(invc*(invc + 1))

                do j = 1, p - 1
                  x = A(j,p)
                  y = A(j,q)
                  A(j,p) = x + (Conjugate(f)*y - t*x)
                  A(j,q) = y - (f*x + t*y)
                enddo

                do j = p + 1, q - 1
                  x = A(p,j)
                  y = A(j,q)
                  A(p,j) = x + (Conjugate(f)*y - t*x)
                  A(j,q) = y - (f*x + t*y)
                enddo

                do j = q + 1, n
                  x = A(p,j)
                  y = A(q,j)
                  A(p,j) = x + (Conjugate(f)*y - t*x)
                  A(q,j) = y - (f*x + t*y)
                enddo

                A(p,q) = 0

                do j = 1, n
                  x = U(p,j)
                  y = U(q,j)
                  U(p,j) = x + (f*y - t*x)
                  U(q,j) = y - (Conjugate(f)*x + t*y)
                enddo
              endif
            enddo
          enddo

          do p = 1, n
            ev(1,p) = 0
            A(p,p) = ev(2,p)
          enddo
        enddo

        print *, "Bad convergence in TakagiFactor"

1        continue

* make the diagonal elements nonnegative

        do p = 1, n
          d(p) = abs(A(p,p))
          if( d(p) .gt. DBL_EPS .and. d(p) .ne. Re(A(p,p)) ) then
            f = sqrt(A(p,p)/d(p))
            do q = 1, n
              U(p,q) = U(p,q)*f
            enddo
          endif
        enddo

        if( sort .eq. 0 ) return

* sort the eigenvalues

        do p = 1, n - 1
          j = p
          t = d(p)
          do q = p + 1, n
            if( sort*(t - d(q)) .gt. 0 ) then
              j = q
              t = d(q)
            endif
          enddo

          if( j .ne. p ) then
            d(j) = d(p)
            d(p) = t
            do q = 1, n
              x = U(p,q)
              U(p,q) = U(j,q)
              U(j,q) = x
            enddo
          endif
        enddo
        end

c -------------------------------------------------------------------- c

      subroutine MHpmfromMHi(MHgiv,Hnum,prec,output)
      
      implicit none
      
      double precision MHgiv,prec
      
      integer Hnum
      
      double precision MH1,MH2,MHpm1lorig,MHpmc,Mh1lorig(5)
      
      double precision ep
      
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
     
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double precision Mh1l(5),ZH1l(5,5),MHpm1l
      
      double precision diff1
      
      logical MHpmInput,CPcon,mQrun,onelswitch
      
      integer counter
      
      character*120 output
      
      logical mainrun,calcmasseserror
      
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /H1loop/ MHpm1l,MH1l,ZH1l
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
      common /calcerror/mainrun,calcmasseserror
      
      MHpmInput=.TRUE.
      
      MHpm1lorig=MHpm1l
      MH1lorig=MH1l
      MHpmc=MHpm1l
      
      ep=1d-6
      
      MH2=Mh1l(Hnum)
      
      counter=0
      do while(abs(MH2-MHgiv).ge.prec)
      counter=counter+1
      
      
      MHpm=(1d0+ep)*MHpmc
      
      call treelevelmasses
        
      call HMass(Mh,ZH5,Mh1l,ZH1l,MHpm,MHpm1l)
      
      MH1=MH1l(Hnum)
      
      
      MHpm=MHpmc
      
      call treelevelmasses
        
      call HMass(Mh,ZH5,Mh1l,ZH1l,MHpm,MHpm1l)
      
      MH2=MH1l(Hnum)

      
      diff1=(-MH2+MH1)/(ep*MHpmc)
      if(abs(diff1).le.1d-1) then
!       write(*,*) "Slope too flat for Newton method!"
      calcmasseserror=.TRUE.
      MHpm=MHpm1lorig
      Mh1l=MH1lorig
      
      return
      end if

      MHpmc=MHpmc-(MH2-MHgiv)/diff1
      
      if(counter.ge.100) then
!       write(*,*) "Newton method does not converge!"
      calcmasseserror=.TRUE.
      MHpm=MHpm1lorig
      Mh1l=MH1lorig
      return
      end if
      
      end do
      
!       write(*,*) counter,MHpm1l,MH2,MHgiv
      
      call writeoutput(output)
      
      MHpm=MHpm1lorig
      Mh1l=MH1lorig
      
      return
      end
      
      
      subroutine writeoutput(output)
      
      implicit none
      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
     
      double precision RAk,RAl
      
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
     
      double precision absM3, phiM3
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
     
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double precision Qs
      
      double precision MHd2,MHu2,ms2
       
      double precision Mh1l(5),ZH1l(5,5)
      
      double precision MHpm1l
      
      double precision ReAlambdaDR,ReAkappaDR,ImAlambdaDR,ImAkappaDR
      
      double precision absAtDR,phiAtDR,mq2DR,mur2dR
      
      double precision pi
      integer nin,nout,ninlha
      parameter (pi = 2d0*Asin(1d0))
      parameter (nin=43,nout=44,ninlha=45)
      
      logical MHpmInput,CPcon,mQrun,onelswitch
      
      integer EDMswitch
      
      integer i,j
      
      double precision mtpole,mbMSmb
     
      double precision g1Qs,g2Qs,g3Qs,YtQs,YbQs
      
      character*120 output
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /gluino/ absM3,phiM3
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
     
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /topoles/ mtpole,mbMsmb
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /H1loop/ MHpm1l,MH1l,ZH1l
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
      common /slhapara/ RAl,RAk
      common /DRpara/ ReAlambdaDR,ReAkappaDR,ImAlambdaDR,ImAkappaDR
      common /DRstop/ absAtDR,phiAtDR,mq2DR,mur2dR
      common /EDMselec/ EDMswitch
      common /scale/ Qs
c nhung added for atat correction 22/3/2019
      integer ilevel,igauge
      common/ilevelATAT/ilevel,igauge
c end nhung added 22/3/2019  
c nhung added for the output of Higgs WFR factor 6/2021
      double complex ZHfac(5,5)
      common /Zfactor/ ZHfac 
c end nhung added for the output of Higgs WFR factor 6/2021
      call runpara(g1Qs,g2Qs,g3Qs,YtQs,YbQs,MHd2,MHu2,ms2)
      
      open(nout,file=output)
      
      write(nout,99) 'MODSEL'
      write(nout,102) 3,1,'NMSSM'
      if (CPcon) then
      write(nout,102) 5,0,'CP is conserved'
      else
      write(nout,102) 5,2,'general CP violation'
      end if
      if (onelswitch) then
      write(nout,102) 6,1,'one-loop Higgs masses'
      else
c nhung change here for atat correction 6/2021
	if (ilevel.eq.2)then
      write(nout,102) 6,2,'two-loop O(alpha_t alpha_s) Higgs masses'
	elseif(ilevel.eq.3)then
      write(nout,102) 6,3,'two-loop O(alpha_t alpha_s+ alpha_t^2) Higgs 
     .masses'
	elseif(ilevel.eq.4)then
      write(nout,102) 6,4,'two-loop O(alpha_t alpha_s+(alpha_t+alpha_lambda+alpha_kappa)^2) Higgs 
     .masses'
	endif
c end nhung change here 6/2021
      end if
      if(mQrun) then
c nhung change here for atat correction 22/3/2019
	if (igauge.ne.1)then
      write(nout,102) 7,1,'DR renormalization in top/stop-sector,O(alpha
     ._s,alpha_t)'
	else
      write(nout,102) 7,2,'DR renormalization in top/stop-sector,O(alpha
     ._s,alpha_t,alpha)'
	endif
      else
      write(nout,102) 7,3,'OS renormalization in top/stop-sector'
c end nhung change here 22/3/2019
      end if
      if(MHpmInput) then
      write(nout,102) 8,0,'Renormalization (1), MHpm as Input'
      else
      write(nout,102) 8,1,'Renormalization (2), Alambda as Input'
      end if
      
      if(.not. CPcon) then
      write(nout,102) 10,EDMswitch,'0: no EDMs calculated (default),
     . 1: EDMs calcluated, 2: detailed output'
      end if
      write(nout,98)

c MMM changed 10/8/2013 and 23/3/2019
      write(nout,99) 'SPINFO'
      write(nout,61) 1,'CalcMasses   # loop Higgs mass calculator'
      write(nout,61) 2,'4.00         # version number'
      write(nout,98)
c end MMM changed 10/8/2013
      
      write(nout,99) 'SMINPUTS'
      write(nout,100) 1,1/alpha,'alpha_em^(-1)'
      write(nout,100) 2,Gf,'G_F'
      write(nout,100) 3,alphaS,'alpha_s'
      write(nout,100) 4,Mz,'Mz'
      write(nout,100) 5,mbMSmb,'m_b^MS(mb)'
      write(nout,100) 6,mtpole,'m_t'
      write(nout,100) 7,mtau,'m_tau'
      write(nout,100) 9,Mw,'Mw'
      write(nout,100) 11,me,'m_e'
      write(nout,100) 13,mmu,'m_mu'
      write(nout,100) 21,md,'m_d'
      write(nout,100) 22,mu,'m_u'
      write(nout,100) 23,ms,'m_s'
      write(nout,100) 24,mc,'m_c'
      write(nout,98)
      
!       write(nout,99) 'MINPAR'
!       write(nout,100) 3,tanb,'tanbeta'
!       write(nout,100) 23,0d0,'mu'
!       write(nout,98) 
      
      write(nout,99) 'EXTPAR'
      write(nout,100) 0,Qs,'input scale'
c MMM changed 10/8/2013
      if(CPcon) then
c end MMM changed 10/8/2013
         write(nout,100) 1,absM1*Cos(phiM1),'M1'
         write(nout,100) 2,absM2*Cos(phiM2),'M2'
         write(nout,100) 3,absM3*Cos(phiM3),'M3'
         write(nout,100) 11,absAtDR*Cos(phiAtDR),'At'
         write(nout,100) 12,absAd(3)*Cos(phiAd(3)),'Ab'
         write(nout,100) 13,absAe(3)*Cos(phiAe(3)),'Atau'
         write(nout,100) 14,absAu(2)*Cos(phiAu(2)),'Ac'
         write(nout,100) 15,absAd(2)*Cos(phiAd(2)),'As'
         write(nout,100) 16,absAe(2)*Cos(phiAe(2)),'Amu'
         write(nout,100) 17,absAu(1)*Cos(phiAu(1)),'Au'
         write(nout,100) 18,absAd(1)*Cos(phiAd(1)),'Ad'
         write(nout,100) 19,absAe(1)*Cos(phiAe(1)),'Ae'
         write(nout,100) 25,tanb,'tanbeta'
         if(MHpmInput) then
         write(nout,100) 27,MHpm,'MHpm'
         end if
         write(nout,100) 31,Sqrt(ml2(1)),'m_~eL'
         write(nout,100) 32,Sqrt(ml2(2)),'m_~muL'
         write(nout,100) 33,Sqrt(ml2(3)),'m_~tauL'
         write(nout,100) 34,Sqrt(mer2(1)),'m_~eR'
         write(nout,100) 35,Sqrt(mer2(2)),'m_~muR'
         write(nout,100) 36,Sqrt(mer2(3)),'m_~tauR'
         write(nout,100) 41,Sqrt(mq2(1)),'m_~q1L'
         write(nout,100) 42,Sqrt(mq2(2)),'m_~q2L'
         write(nout,100) 43,Sqrt(mq2DR),'m_~q3L'
         write(nout,100) 44,Sqrt(mur2(1)),'m_~uR'
         write(nout,100) 45,Sqrt(mur2(2)),'m_~cR'
         write(nout,100) 46,Sqrt(mur2DR),'m_~tR'
         write(nout,100) 47,Sqrt(mdr2(1)),'m_~dR'
         write(nout,100) 48,Sqrt(mdr2(2)),'m_~sR'
         write(nout,100) 49,Sqrt(mdr2(3)),'m_~bR'
         write(nout,100) 61,abslambda*Cos(phil),'lambda'
         write(nout,100) 62,abskappa*Cos(phik),'kappa'
         write(nout,100) 63,ReAlambdaDR,'Alambda'
         write(nout,100) 64,ReAkappaDR,'Akappa'
c MMM changed 10/8/2013
         write(nout,100) 65,abslambda*vS/Sqrt(2d0)*Cos(phil+phis),'mu_ef
     .f'
      elseif(.not. CPcon) then
        write(nout,100) 1,absM1*Cos(phiM1),'Re(M1)'
         write(nout,100) 2,absM2*Cos(phiM2),'Re(M2)'
         write(nout,100) 3,absM3*Cos(phiM3),'Re(M3)'
         write(nout,100) 11,absAtDR*Cos(phiAtDR),'Re(At)'
         write(nout,100) 12,absAd(3)*Cos(phiAd(3)),'Re(Ab)'
         write(nout,100) 13,absAe(3)*Cos(phiAe(3)),'Re(Atau)'
         write(nout,100) 14,absAu(2)*Cos(phiAu(2)),'Re(Ac)'
         write(nout,100) 15,absAd(2)*Cos(phiAd(2)),'Re(As)'
         write(nout,100) 16,absAe(2)*Cos(phiAe(2)),'Re(Amu)'
         write(nout,100) 17,absAu(1)*Cos(phiAu(1)),'Re(Au)'
         write(nout,100) 18,absAd(1)*Cos(phiAd(1)),'Re(Ad)'
         write(nout,100) 19,absAe(1)*Cos(phiAe(1)),'Re(Ae)'
         write(nout,100) 25,tanb,'tanbeta'
         if(MHpmInput) then
         write(nout,100) 27,MHpm,'MHpm'
         end if
         write(nout,100) 31,Sqrt(ml2(1)),'m_~eL'
         write(nout,100) 32,Sqrt(ml2(2)),'m_~muL'
         write(nout,100) 33,Sqrt(ml2(3)),'m_~tauL'
         write(nout,100) 34,Sqrt(mer2(1)),'m_~eR'
         write(nout,100) 35,Sqrt(mer2(2)),'m_~muR'
         write(nout,100) 36,Sqrt(mer2(3)),'m_~tauR'
         write(nout,100) 41,Sqrt(mq2(1)),'m_~q1L'
         write(nout,100) 42,Sqrt(mq2(2)),'m_~q2L'
         write(nout,100) 43,Sqrt(mq2DR),'m_~q3L'
         write(nout,100) 44,Sqrt(mur2(1)),'m_~uR'
         write(nout,100) 45,Sqrt(mur2(2)),'m_~cR'
         write(nout,100) 46,Sqrt(mur2DR),'m_~tR'
         write(nout,100) 47,Sqrt(mdr2(1)),'m_~dR'
         write(nout,100) 48,Sqrt(mdr2(2)),'m_~sR'
         write(nout,100) 49,Sqrt(mdr2(3)),'m_~bR'
         write(nout,100) 61,abslambda*Cos(phil),'Re(lambda)'
         write(nout,100) 62,abskappa*Cos(phik),'Re(kappa)'
         write(nout,100) 63,ReAlambdaDR,'Re(Alambda)'
         write(nout,100) 64,ReAkappaDR,'Re(Akappa)'   
         write(nout,100) 65,abslambda*vS/Sqrt(2d0)*Cos(phil+phis),
     .'Re(mu_eff)'
      endif
c end MMM changed 10/8/2013
      write(nout,98)
      
      if(.not. CPcon) then
c MMM changed 10/8/2013
         write(nout,99) 'IMEXTPAR'
         write(nout,100) 0,Qs,'input scale'
         write(nout,100) 1,absM1*Sin(phiM1),'Im(M1)'
         write(nout,100) 2,absM2*Sin(phiM2),'Im(M2)'
         write(nout,100) 3,absM3*Sin(phiM3),'Im(M3)'
         write(nout,100) 11,absAtDR*Sin(phiAtDR),'Im(At)'
         write(nout,100) 12,absAd(3)*Sin(phiAd(3)),'Im(Ab)'
         write(nout,100) 13,absAe(3)*Sin(phiAe(3)),'Im(Atau)'
         write(nout,100) 14,absAu(2)*Sin(phiAu(2)),'Im(Ac)'
         write(nout,100) 15,absAd(2)*Sin(phiAd(2)),'Im(As)'
         write(nout,100) 16,absAe(2)*Sin(phiAe(2)),'Im(Amu)'
         write(nout,100) 17,absAu(1)*Sin(phiAu(1)),'Im(Au)'
         write(nout,100) 18,absAd(1)*Sin(phiAd(1)),'Im(Ad)'
         write(nout,100) 19,absAe(1)*Sin(phiAe(1)),'Im(Ae)'
         write(nout,100) 61,abslambda*Sin(phil),'Im(lambda)'
         write(nout,100) 62,abskappa*Sin(phik),'Im(kappa)'
         write(nout,100) 63,ImAlambdaDR,'Im(Alambda)'
         write(nout,100) 64,ImAkappaDR,'Im(Akappa)'
         write(nout,100) 65,abslambda*vS/Sqrt(2d0)*Sin(phil+phis),'Im(m
     .u_eff)'
c end MMM changed 10/8/2013
      write(nout,98)
      endif
      
      if(.not. CPcon) then
      write(nout,99) 'CMPLX'
      write(nout,100) 3,phiu,'phi_u'
      write(nout,98)
      end if
      
      write(nout,99) 'MASS'
      write(nout,100) 24,Mw,'Mw'
      if(CPcon) then
      write(nout,100) 25,Mh1l(3),'H_1'
      write(nout,100) 35,Mh1l(4),'H_2'
      write(nout,100) 36,Mh1l(1),'A_1'
      write(nout,100) 45,Mh1l(5),'H_3'
      write(nout,100) 46,Mh1l(2),'A_2'
      else
      write(nout,100) 25,Mh1l(1),'H_1'
      write(nout,100) 35,Mh1l(2),'H_2'
      write(nout,100) 36,Mh1l(3),'H_3'
      write(nout,100) 45,Mh1l(4),'H_4'
      write(nout,100) 46,Mh1l(5),'H_5'
      end if
      write(nout,100) 37,MHpm1l,'H_pm'
      if(mQrun) then
        write(nout,100) 6,mt,'m_t^DR'
      endif
      write(nout,100) 1000001,MSd(1),'~d_1'
      write(nout,100) 1000002,MSu(1),'~u_1'
      write(nout,100) 1000003,MSd(3),'~s_1'
      write(nout,100) 1000004,MSu(3),'~c_1'
      write(nout,100) 1000005,MSd(5),'~b_1'
      write(nout,100) 1000006,MSu(5),'~t_1'
      write(nout,100) 2000001,MSd(2),'~d_2'
      write(nout,100) 2000002,MSu(2),'~u_2'
      write(nout,100) 2000003,MSd(4),'~s_2'
      write(nout,100) 2000004,MSu(4),'~c_2'
      write(nout,100) 2000005,MSd(6),'~b_2'
      write(nout,100) 2000006,MSu(6),'~t_2'
      write(nout,100) 1000011,MSe(1),'~e_1'
      write(nout,100) 1000013,MSe(3),'~mu_1'
      write(nout,100) 1000015,MSe(5),'~tau_1'
      write(nout,100) 2000011,MSe(2),'~e_2'
      write(nout,100) 2000013,MSe(4),'~mu_2'
      write(nout,100) 2000015,MSe(6),'~tau_2'
      write(nout,100) 1000012,MSv(1),'~nu_e'
c MMM changed 18/8/2013
      write(nout,100) 1000014,MSv(2),'~nu_mu'
      write(nout,100) 1000016,MSv(3),'~nu_tau'
c end MMM changed 18/8/2013
      write(nout,100) 1000021,absM3,'~g'
      write(nout,100) 1000024,MCha(1),'Cha_1'
      write(nout,100) 1000037,MCha(2),'Cha_2'
      write(nout,100) 1000022,MChi(1),'Chi_1'
      write(nout,100) 1000023,MChi(2),'Chi_2'
      write(nout,100) 1000025,MChi(3),'Chi_3'
      write(nout,100) 1000035,MChi(4),'Chi_4'
      write(nout,100) 1000045,MChi(5),'Chi_5'
      write(nout,98)

      if(CPcon) then
      write(nout,99) 'NMHMIX'
      do i=1,3
      do j=1,3
      write(nout,103) i,j,ZH1l(i+2,j+2),'ZH',i,j
      end do
      end do
      write(nout,98)
      write(nout,99) 'NMAMIX'
      write(nout,101) 1,1,ZH1l(1,1)*Sin(beta),'ZA(1,1)*sinbeta'
      write(nout,101) 1,2,ZH1l(1,1)*Cos(beta),'ZA(1,1)*cosbeta'
      write(nout,101) 1,3,ZH1l(1,2),'ZA(1,2)'
      write(nout,101) 2,1,ZH1l(2,1)*Sin(beta),'ZA(2,1)*sinbeta'
      write(nout,101) 2,2,ZH1l(2,1)*Cos(beta),'ZA(2,1)*cosbeta'
      write(nout,101) 2,3,ZH1l(2,2),'ZA(2,2)'
      else
c MMM changed 10/8/2013
      write(nout,99) 'NMHMIXC'
c end MMM changed 10/8/2013
      write(nout,101) 1,1, ZH1l(1,3),'ZH(h1,hd)'
      write(nout,101) 1,2, ZH1l(1,4),'ZH(h1,hu)'
      write(nout,101) 1,3, ZH1l(1,5),'ZH(h1,hs)'
      write(nout,101) 1,4, ZH1l(1,1),'ZH(h1,a)'
      write(nout,101) 1,5, ZH1l(1,2),'ZH(h1,as)'
      write(nout,101) 2,1, ZH1l(2,3),'ZH(h2,hd)'
      write(nout,101) 2,2, ZH1l(2,4),'ZH(h2,hu)'
      write(nout,101) 2,3, ZH1l(2,5),'ZH(h2,hs)'
      write(nout,101) 2,4, ZH1l(2,1),'ZH(h2,a)'
      write(nout,101) 2,5, ZH1l(2,2),'ZH(h2,as)'
      write(nout,101) 3,1, ZH1l(3,3),'ZH(h3,hd)'
      write(nout,101) 3,2, ZH1l(3,4),'ZH(h3,hu)'
      write(nout,101) 3,3, ZH1l(3,5),'ZH(h3,hs)'
      write(nout,101) 3,4, ZH1l(3,1),'ZH(h3,a)'
      write(nout,101) 3,5, ZH1l(3,2),'ZH(h3,as)'
      write(nout,101) 4,1, ZH1l(4,3),'ZH(h4,hd)'
      write(nout,101) 4,2, ZH1l(4,4),'ZH(h4,hu)'
      write(nout,101) 4,3, ZH1l(4,5),'ZH(h4,hs)'
      write(nout,101) 4,4, ZH1l(4,1),'ZH(h4,a)'
      write(nout,101) 4,5, ZH1l(4,2),'ZH(h4,as)'
      write(nout,101) 5,1, ZH1l(5,3),'ZH(h5,hd)'
      write(nout,101) 5,2, ZH1l(5,4),'ZH(h5,hu)'
      write(nout,101) 5,3, ZH1l(5,5),'ZH(h5,hs)'
      write(nout,101) 5,4, ZH1l(5,1),'ZH(h5,a)'
      write(nout,101) 5,5, ZH1l(5,2),'ZH(h5,as)'
      write(nout,98)
      end if
      
      write(nout,99) 'STOPMIX'
      do i=1,2
      do j=1,2
c MMM changed 10/8/2013
c      write(nout,103) i,j,Realpart(ZU(i+4,j+4)),'ZU',i,j
         if(CPcon) then
            write(nout,103) i,j,Realpart(ZU(i+4,j+4)),'ZU',i,j
         elseif(.not. CPcon) then
            write(nout,103) i,j,Realpart(ZU(i+4,j+4)),'Re(ZU)',i,j
         endif
c end MMM changed 10/8/2013
      end do
      end do
      write(nout,98)
      
      if(.not. CPcon) then
      write(nout,99) 'IMSTOPMIX'
      do i=1,2
      do j=1,2
c MMM changed 10/8/2013
c      write(nout,103) i,j,Imagpart(ZU(i+4,j+4)),'ZU',i,j
      write(nout,103) i,j,Imagpart(ZU(i+4,j+4)),'Im(ZU)',i,j
c end MMM changed 10/8/2013
      end do
      end do
      write(nout,98)
      end if
      
      write(nout,99) 'SBOTMIX'
      do i=1,2
      do j=1,2
c MMM changed 10/8/2013
c      write(nout,103) i,j,Realpart(ZD(i+4,j+4)),'ZD',i,j
         if(CPcon) then
            write(nout,103) i,j,Realpart(ZD(i+4,j+4)),'ZD',i,j
         elseif(.not. CPcon) then
            write(nout,103) i,j,Realpart(ZD(i+4,j+4)),'Re(ZD)',i,j
         endif
c end MMM changed 10/8/2013
      end do
      end do
      write(nout,98)
      
      if(.not. CPcon) then
      write(nout,99) 'IMSBOTMIX'
      do i=1,2
      do j=1,2
c MMM changed 10/8/2013
c      write(nout,103) i,j,Imagpart(ZD(i+4,j+4)),'ZD',i,j
      write(nout,103) i,j,Imagpart(ZD(i+4,j+4)),'Im(ZD)',i,j
c end MMM changed 10/8/2013
      end do
      end do
      write(nout,98)
      end if
      
      write(nout,99) 'STAUMIX'
      do i=1,2
      do j=1,2
c MMM changed 
c      write(nout,103) i,j,Realpart(ZE(i+4,j+4)),'ZE',i,j
         if(CPcon) then
            write(nout,103) i,j,Realpart(ZE(i+4,j+4)),'ZE',i,j
         elseif(.not. CPcon) then
            write(nout,103) i,j,Realpart(ZE(i+4,j+4)),'Re(ZE)',i,j
         endif
c end MMM changed
      end do
      end do
      write(nout,98)
      
      if(.not. CPcon) then
      write(nout,99) 'IMSTAUMIX'
      do i=1,2
      do j=1,2
c MMM changed 10/8/2013
c      write(nout,103) i,j,Imagpart(ZE(i+4,j+4)),'ZE',i,j
      write(nout,103) i,j,Imagpart(ZE(i+4,j+4)),'Im(ZE)',i,j
c end MMM changed 10/8/2013
      end do
      end do
      write(nout,98)
      end if
      
      write(nout,99) 'VMIX'
      do i=1,2
      do j=1,2
c MMM changed 10/8/2013
c      write(nout,103) i,j,Realpart(UP(i,j)),'V',i,j
         if(CPcon) then
            write(nout,103) i,j,Realpart(UP(i,j)),'V',i,j
         elseif(.not. CPcon) then
            write(nout,103) i,j,Realpart(UP(i,j)),'Re(V)',i,j
         endif
c end MMM changed 1078/2013
      end do
      end do
      write(nout,98)
      
      if(.not. CPcon) then
      write(nout,99) 'IMVMIX'
      do i=1,2
      do j=1,2
c MMM changed 10/8/2013
c      write(nout,103) i,j,Imagpart(UP(i,j)),'V',i,j
      write(nout,103) i,j,Imagpart(UP(i,j)),'Im(V)',i,j
c end MMM changed 10/8/2013
      end do
      end do
      write(nout,98)
      end if
      
      write(nout,99) 'UMIX'
      do i=1,2
      do j=1,2
c MMM changed 10/8/2013
c      write(nout,103) i,j,Realpart(UM(i,j)),'U',i,j
         if(CPcon) then
            write(nout,103) i,j,Realpart(UM(i,j)),'U',i,j
         elseif(.not. CPcon) then
            write(nout,103) i,j,Realpart(UM(i,j)),'Re(U)',i,j
         endif
c end MMM changed 10/8/2013
      end do
      end do
      write(nout,98)
      
      if(.not. CPcon) then
      write(nout,99) 'IMUMIX'
      do i=1,2
      do j=1,2
c MMM changed 10/8/2013
c      write(nout,103) i,j,Imagpart(UM(i,j)),'U',i,j
      write(nout,103) i,j,Imagpart(UM(i,j)),'U',i,j
c end MMM changed 10/8/2013
      end do
      end do
      write(nout,98)
      end if
      
      write(nout,99) 'NMNMIX'
      do i=1,5
      do j=1,5
c MMM changed 10/8/2013
c      write(nout,103) i,j,Realpart(NN(i,j)),'ZN',i,j
         if(CPcon) then
            write(nout,103) i,j,Realpart(NN(i,j)),'ZN',i,j
         elseif(.not. CPcon) then
            write(nout,103) i,j,Realpart(NN(i,j)),'Re(ZN)',i,j
         endif
c end MMM changed 10/8/2013
      end do
      end do
      write(nout,98)
      
      if(.not. CPcon) then
      write(nout,99) 'IMNMNMIX'
      do i=1,5
      do j=1,5
c MMM changed 10/8/2013
c      write(nout,103) i,j,Imagpart(NN(i,j)),'ZN',i,j
      write(nout,103) i,j,Imagpart(NN(i,j)),'Im(ZN)',i,j
c end MMM changed 10/8/2013
      end do
      end do
      write(nout,98)
      end if
      
      write(nout,104) 'MSOFT',Qs
c MMM changed 10/8/2013
c      write(nout,100) 1,absM1*Cos(phiM1),'M1'
c      write(nout,100) 2,absM2*Cos(phiM2),'M2'
c      write(nout,100) 3,absM3*Cos(phiM3),'M3'
      if(CPcon) then
         write(nout,100) 1,absM1*Cos(phiM1),'M1'
         write(nout,100) 2,absM2*Cos(phiM2),'M2'
         write(nout,100) 3,absM3*Cos(phiM3),'M3'
      elseif(.not. CPcon) then
         write(nout,100) 1,absM1*Cos(phiM1),'Re(M1)'
         write(nout,100) 2,absM2*Cos(phiM2),'Re(M2)'
         write(nout,100) 3,absM3*Cos(phiM3),'Re(M3)'
      endif
c end MMM changed 10/8/2013
      write(nout,100) 21,MHd2,'M_Hd^2'
      write(nout,100) 22,MHu2,'M_Hu^2'
      write(nout,100) 31,Sqrt(ml2(1)),'m_~eL'
      write(nout,100) 32,Sqrt(ml2(2)),'m_~muL'
      write(nout,100) 33,Sqrt(ml2(3)),'m_~tauL'
      write(nout,100) 34,Sqrt(mer2(1)),'m_~eR'
      write(nout,100) 35,Sqrt(mer2(2)),'m_~muR'
      write(nout,100) 36,Sqrt(mer2(3)),'m_~tauR'
      write(nout,100) 41,Sqrt(mq2(1)),'m_~q1L'
      write(nout,100) 42,Sqrt(mq2(2)),'m_~q2L'
      write(nout,100) 43,Sqrt(mq2DR),'m_~q3L'
      write(nout,100) 44,Sqrt(mur2(1)),'m_~uR'
      write(nout,100) 45,Sqrt(mur2(2)),'m_~cR'
      write(nout,100) 46,Sqrt(mur2DR),'m_~tR'
      write(nout,100) 47,Sqrt(mdr2(1)),'m_~dR'
      write(nout,100) 48,Sqrt(mdr2(2)),'m_~sR'
      write(nout,100) 49,Sqrt(mdr2(3)),'m_~bR'
      write(nout,100) 70,ms2,'m_s^2'
      write(nout,98)
      
      if (.not. CPcon) then
      write(nout,104) 'IMMSOFT',Qs
c MMM changed 10/8/2013
c      write(nout,100) 1,absM1*Sin(phiM1),'M1'
c      write(nout,100) 2,absM2*Sin(phiM2),'M2'
c      write(nout,100) 3,absM3*Sin(phiM3),'M3'
      write(nout,100) 1,absM1*Sin(phiM1),'Im(M1)'
      write(nout,100) 2,absM2*Sin(phiM2),'Im(M2)'
      write(nout,100) 3,absM3*Sin(phiM3),'Im(M3)'
c end MMM changed 10/8/2013
      write(nout,98)
      end if
      
      write(nout,104) 'GAUGE',Qs
      write(nout,100) 1,g1Qs,'g1(Qs)'
      write(nout,100) 2,g2Qs,'g2(Qs)'
      write(nout,100) 3,g3Qs,'g3(Qs)'
      write(nout,98)
      
      write(nout,104) 'YU',Qs
      write(nout,101) 3,3,YtQs,'Yt(Qs)'
      write(nout,98)
      
      write(nout,104) 'YD',Qs
      write(nout,101) 3,3,YbQs,'Yb(Qs)'
      write(nout,98)
      
      write(nout,104) 'YE',Qs
      write(nout,101) 3,3,Ye(3),'Ytau(Qs)'
      write(nout,98)
      

c nhung added for the output of Higgs WFR factor 6/2021
      if (CPcon) then
      write(nout,99) 'ReZFacA'
      do i=1,2
      do j=1,2
      write(nout,103) i,j,DBLE(ZHfac(i,j)),'Re_ZFA',i,j
      end do
      end do
      write(nout,98)
      write(nout,99) 'ImZFacA'
      do i=1,2
      do j=1,2
      write(nout,103) i,j,DIMAG(ZHfac(i,j)),'Im_ZFA',i,j
      end do
      end do
      write(nout,98)
      write(nout,99) 'ReZFacH'
      do i=1,3
      do j=1,3
      write(nout,103) i,j,DBLE(ZHfac(i+2,j+2)),'Re_ZFH',i,j
      end do
      end do
      write(nout,98)
      write(nout,99) 'ImZFacH'
      do i=1,3
      do j=1,3
      write(nout,103) i,j,DIMAG(ZHfac(i+2,j+2)),'Im_ZFH',i,j
      end do
      end do
      write(nout,98)
      !!
      else
      write(nout,99) 'ReZFacH'
      do i=1,5
      do j=1,5
      write(nout,103) i,j,DBLE(ZHfac(i,j)),'Re_ZFH',i,j
      end do
      end do
      write(nout,98)
      write(nout,99) 'ImZFacH'
      do i=1,5
      do j=1,5
      write(nout,103) i,j,DIMAG(ZHfac(i,j)),'Im_ZFH',i,j
      end do
      end do
      write(nout,98)
      end if
c end nhung added for the output of Higgs WFR factor 6/2021

     
98    format('#')
99    format('Block',1x,A)
c MMM changed 10/8/2013
61    format(1x,I5,3x,A)
c end MMM changed 10/8/2013
100   format(1x,I9,3x,1P,E16.8,0P,3x,'#',1x,A)
101   format(1x,I2,1x,I2,3x,1P,E16.8,0P,3x,'#',1x,A)
102   format(1x,I2,1x,I2,3x,'#',1x,A)
103   format(1x,I2,1x,I2,3x,1P,E16.8,0P,3x,'#',1x,A,'(',I1,',',I1,')')
104   format('Block',1x,A,1x,'Q=',E16.8)
      
      close(nout)
      
      return
      end
